Package: GREENeR
Type: Package
Title: Geospatial Regression Equation for European Nutrient Losses
        (GREEN)
Version: 1.0.2
Authors@R: c(person("A.", "Udias", role = "aut",
                     email = "angeludias@gmail.com"),
              person("B.", "Grizzetti", role = "aut", 
                     email = "bruna.grizzetti@jrc.it"), 
              person("O.", "Vigiak", role = "aut", 
                     email = "Olga.VIGIAK@ec.europa.eu"),
              person("J.", "Gomez", role = "aut", 
                     email = "javier.gomez@urjc.es"),
              person("C.", "Alfaro", role = c("aut", "cre"),
                     email = "c.alfarog@gmail.com"),
              person("A.", "Aloe", role = "aut", 
                     email = "alberto.aloe@jrc.it"))
Author: A. Udias [aut],
  B. Grizzetti [aut],
  O. Vigiak [aut],
  J. Gomez [aut],
  C. Alfaro [aut, cre],
  A. Aloe [aut]
Maintainer: C. Alfaro <c.alfarog@gmail.com>
Description: Tools and methods to apply the model Geospatial Regression Equation
    for European Nutrient losses (GREEN); 
    Grizzetti et al. (2005) <doi:10.1016/j.jhydrol.2004.07.036>; 
    Grizzetti et al. (2008);
    Grizzetti et al. (2012) <doi:10.1111/j.1365-2486.2011.02576.x>; 
    Grizzetti et al. (2021) <doi:10.1016/j.gloenvcha.2021.102281>. 
License: GPL-3
Encoding: UTF-8
LazyData: true
Imports: FME (>= 1.3.6.1), data.table (>= 1.13.6), reshape2 (>= 1.4.4),
        ggplot2 (>= 3.3.5), graphics (>= 3.6.1), sf (>= 1.0-2), dplyr
        (>= 1.0.7), magrittr (>= 2.0.1), tmap (>= 4.3), gridExtra (>=
        2.3), classInt (>= 0.4-3), grDevices (>= 3.5), networkD3 (>=
        0.4), parallelly (>= 1.30.0)
Depends: R (>= 4.1)
RoxygenNote: 7.3.3
URL: https://github.com/calfarog/GREENeR,
        https://calfarog.github.io/GREENeR/
BugReports: https://github.com/calfarog/GREENeR/issues
Suggests: knitr, rmarkdown, codetools
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-22 12:19:27 UTC; cesar.alfaro
Repository: CRAN
Date/Publication: 2026-04-22 12:50:09 UTC
Built: R 4.5.2; ; 2026-04-22 14:44:05 UTC; unix

Package: GREENeR
Type: Package
Title: Geospatial Regression Equation for European Nutrient Losses
        (GREEN)
Version: 1.0.2
Authors@R: c(person("A.", "Udias", role = "aut",
                     email = "angeludias@gmail.com"),
              person("B.", "Grizzetti", role = "aut", 
                     email = "bruna.grizzetti@jrc.it"), 
              person("O.", "Vigiak", role = "aut", 
                     email = "Olga.VIGIAK@ec.europa.eu"),
              person("J.", "Gomez", role = "aut", 
                     email = "javier.gomez@urjc.es"),
              person("C.", "Alfaro", role = c("aut", "cre"),
                     email = "c.alfarog@gmail.com"),
              person("A.", "Aloe", role = "aut", 
                     email = "alberto.aloe@jrc.it"))
Author: A. Udias [aut],
  B. Grizzetti [aut],
  O. Vigiak [aut],
  J. Gomez [aut],
  C. Alfaro [aut, cre],
  A. Aloe [aut]
Maintainer: C. Alfaro <c.alfarog@gmail.com>
Description: Tools and methods to apply the model Geospatial Regression Equation
    for European Nutrient losses (GREEN); 
    Grizzetti et al. (2005) <doi:10.1016/j.jhydrol.2004.07.036>; 
    Grizzetti et al. (2008);
    Grizzetti et al. (2012) <doi:10.1111/j.1365-2486.2011.02576.x>; 
    Grizzetti et al. (2021) <doi:10.1016/j.gloenvcha.2021.102281>. 
License: GPL-3
Encoding: UTF-8
LazyData: true
Imports: FME (>= 1.3.6.1), data.table (>= 1.13.6), reshape2 (>= 1.4.4),
        ggplot2 (>= 3.3.5), graphics (>= 3.6.1), sf (>= 1.0-2), dplyr
        (>= 1.0.7), magrittr (>= 2.0.1), tmap (>= 4.3), gridExtra (>=
        2.3), classInt (>= 0.4-3), grDevices (>= 3.5), networkD3 (>=
        0.4), parallelly (>= 1.30.0)
Depends: R (>= 4.1)
RoxygenNote: 7.3.3
URL: https://github.com/calfarog/GREENeR,
        https://calfarog.github.io/GREENeR/
BugReports: https://github.com/calfarog/GREENeR/issues
Suggests: knitr, rmarkdown, codetools
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-22 12:19:27 UTC; cesar.alfaro
Repository: CRAN
Date/Publication: 2026-04-22 12:50:09 UTC
Built: R 4.5.2; ; 2026-04-22 17:11:09 UTC; unix

# Generated by roxygen2: do not edit by hand

export(LakeRetent_plot)
export(N4_sankey)
export(calib_boxplot)
export(calib_dot)
export(calib_green)
export(compare_calib)
export(gof)
export(green_shares)
export(input_Tserie)
export(input_Tserie_area)
export(input_maps)
export(input_plot)
export(nutrient_maps)
export(nutrient_tserie)
export(nutrient_tserie_darea)
export(read_NSdata)
export(read_geometry)
export(references_plot)
export(region_nut_balance)
export(scatter_plot)
export(select_params)
export(shreve)
export(simobs_annual_plot)
import(dplyr)
import(magrittr)
importFrom(classInt,classIntervals)
importFrom(data.table,":=")
importFrom(data.table,as.data.table)
importFrom(data.table,setkey)
importFrom(dplyr,across)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,everything)
importFrom(dplyr,group_by)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_area)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_color_brewer)
importFrom(ggplot2,scale_fill_brewer)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylim)
importFrom(grDevices,colorRampPalette)
importFrom(graphics,abline)
importFrom(graphics,barplot)
importFrom(graphics,boxplot)
importFrom(graphics,grid)
importFrom(graphics,hist)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(gridExtra,grid.arrange)
importFrom(magrittr,"%>%")
importFrom(networkD3,JS)
importFrom(networkD3,sankeyNetwork)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,parLapply)
importFrom(parallel,parSapply)
importFrom(parallel,stopCluster)
importFrom(parallelly,availableCores)
importFrom(reshape2,dcast)
importFrom(reshape2,melt)
importFrom(sf,st_area)
importFrom(sf,st_read)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,coefficients)
importFrom(stats,cor)
importFrom(stats,density)
importFrom(stats,lm)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(tmap,tm_fill)
importFrom(tmap,tm_layout)
importFrom(tmap,tm_scalebar)
importFrom(tmap,tm_shape)
importFrom(tmap,tmap_arrange)
importFrom(utils,read.csv)
importFrom(utils,write.csv)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# GREENeR <img src='man/figures/logo.png' align="right" height="139" />

<!-- badges: start -->

[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/GREENeR?color=green)](https://cran.r-project.org/package=GREENeR)
[![R-CMD-check](https://github.com/calfarog/GREENeR/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/calfarog/GREENeR/actions/workflows/R-CMD-check.yaml)
i[![License: GPL
v3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)
[![](https://cranlogs.r-pkg.org/badges/grand-total/GREENeR?color=green)](https://cran.r-project.org/package=GREENeR)
<!-- badges: end -->

The goal of GREENeR is to provide a conceptual model to assess nutrient
loads (total nitrogen and total phosphorous) from a basin given diffuse
and point emissions to the land and river network. The model comprises
two major nutrient pathways: diffuse sources that undergo retention in
the land phase (basin retention) before reaching the stream, and point
sources that are directly discharged into surface waters. Once in the
river network, all sources are reduced by the in-stream retention (river
retention).

## Installation

Install the GREENeR package via CRAN:

``` r
install.packages("GREENeR")
```

You can also install the development version of GREENeR from GitHub
with:

``` r
# install.packages("devtools")
devtools::install_github("calfarog/GREENeR")
```

## Illustrative Example

This is a basic example which shows how GREENeR works, for a more
detailed illustration, please see our
[vignettes](https://calfarog.github.io/GREENeR/articles/GREENeR.html).

GREENeR functionalities are illustrated for the [Lay
river](https://en.wikipedia.org/wiki/Lay_(river)) basin, in France.
Please note that the basin data are extracted from a pan-European
dataset (Grizzetti et al. 2021) for the purpose of showing the package
tools and the analysis steps for a generic region.

## Model calibration

The quality of model calibration depends on the quality and number of
available observations. To run the calibration process for a scenario
(function `calib_green()`), the following settings must be defined:

1.  The expected range for each parameter. This range is defined by two
    vectors of three values, one for the lower limits and one for the
    upper limits of the three parameters. The values correspond to each
    of the parameters of the model in sequence: `alpha_P`, `alpha_L`,
    and `sd_coeff`.

2.  The number of iterations to be performed during the calibration
    process. The higher the number of iterations, the more likely it is
    to achieve quality parameters, but the longer the computation time
    required. Although it depends on the catchment and the proposed
    intervals, it is recommended to run at least 200 iterations to have
    enough information to continue the calibration process.

3.  The years to be included in the calibration process.

This is a basic example which shows you how to solve a common problem:

``` r
library(GREENeR)

data("catch_data_TN")
data("annual_data_TN")

# Parameter for the calibration of the model
# the lower limits for all params (alpha_P, alpha_L, sd_coef)
low <- c(10, 0.000, 0.1) 
# the upper limits for all params (alpha_P, alpha_L, sd_coef)  
upp <- c(50, 0.08, 0.9)       

# number of iterations
n_iter <- 2000    

# years in which the model should be executed
years <- c(2003:2009)
```

A calibration of TN in the Lay basin is performed with the parameters
indicated above. The process is parallelised and will use all the cores
of the computer. The computation time depends on the computer, the
basin, and the number of iterations.

``` r
# execution of the calibration  
DF_calib_TN <- calib_green(catch_data_TN, annual_data_TN, n_iter, low, upp, years)
```

The calibration function applies a Latin Hypercube sampling scheme to
the three parameters within the possible range (defined by lower and
upper limits) and evaluates model performance (predictions against
available observations) for the calibration period (specified in years)
by calculating several “goodness-of-fit” metrics. The function returns a
data frame with parameters and goodness-of-fit scores that can be
further analyzed.

The function applies the following goodness-of-fit metrics (Althoff and
Rodrigues 2021):

- NSE: Nash-Sutcliffe efficiency
- rNSE: Relative Nash-Sutcliffe efficiency
- mNSE: Modified Nash-Sutcliffe efficiency
- cp: Coefficient of Persistence
- VE: Volumetric Efficiency
- KGE: Kling-Gupta efficiency
- d: Index of Agreement
- md: Modified index of agreement
- rd: Relative Index of Agreement
- r: Linear correlation coefficient
- R2: Coefficient of determination
- PBIAS: Percent bias
- MAE: mean absolute error
- RMSE: Root mean square error
- ME: Mean error
- MSE: Mean square error
- NRMSE: Normalized Root Mean Square Error

## Exploring calibration results and visualization tools

Choosing the right goodness-of-fit metric to select the best set of
parameters for a model largely depends on the overall study scope, the
area of interest (e.g. high or low load, upper or lower catchment area),
and the available observation dataset (size and quality). The library
includes a series of functions to examine the calibration results and
help select the most suitable set of parameters.

##### Calibration boxplot

``` r
# Generating the box plots  
rateBS <-  5  #  percent of parameters selected from the whole calibration set
calib_boxplot(DF_calib_TN, rateBS)
```

<img src="man/figures/calibration_boxplotTN.png" width="100%" />

##### Calibration dot plot

``` r
# Generating the dot plots  
Gof_mes <- "NSE"
calib_dot(DF_calib_TN, Gof_mes)
```

<img src="man/figures/calibration_dotTN.png" width="100%" />

##### Calibration scatter plot

``` r
# Generating the scatter plots  
Gof_mes <- "NSE"
scatter_plot(DF_calib_TN, Gof_mes)
```

<img src="man/figures/calibration_scatterTN.png" width="100%" />

## References

Althoff, D., & Rodrigues, L. N. (2021). Goodness-of-fit criteria for
hydrological models: Model calibration and performance assessment.
Journal of Hydrology, 600, 126674.

Grizzetti, B. and Vigiak, O. and Udias, A. and Aloe, A. and Zanni, M.
and Bouraoui, F. and Pistocchi, A. and Dorati, C. and Friedland, R. and
De Roo, A. and others (2021). How EU policies could reduce nutrient
pollution in European inland and coastal waters. Global Environmental
Change, 69, 102281. <https://doi.org/10.1016/j.gloenvcha.2021.102281>

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----message=FALSE------------------------------------------------------------
# load the GREENeR package
library(GREENeR)

## -----------------------------------------------------------------------------
# load the topological sequence of catchments and complementary info
data(catch_data_TN)
head(catch_data_TN)

## -----------------------------------------------------------------------------
# load the sources of TN for each year and catchment
data(annual_data_TN)
head(annual_data_TN)

# load the sources of TP for each year and catchment
data(annual_data_TP)
head(annual_data_TP)

## -----------------------------------------------------------------------------
# load the geographical information of the basin (require for some functionalities)
data(sh_file)
head(sh_file)

## ----message=FALSE, eval=FALSE------------------------------------------------
# # the barplot for the Lay TN and TP Scenarios
# input_plot(annual_data_TN, sh_file, "Lay", "B")
# input_plot(annual_data_TP, sh_file, "Lay", "B")

## ----echo=FALSE, out.width='100%', fig.cap="Barplot for the Lay TN scenario."----
knitr::include_graphics('figures/barplot_LayTN.png')

## ----echo=FALSE, out.width='100%', fig.cap="Barplot for the Lay TP scenario."----
knitr::include_graphics('figures/barplot_LayTP.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# # the density plots for the Lay TN and TP Scenarios
# input_plot(annual_data_TN, "Lay", "D",coef_SD=1, sh_file)
# input_plot(annual_data_TP, "Lay", "D",coef_SD=1, sh_file)

## ----echo=FALSE, out.width='100%', fig.cap="Density plot for the Lay TN scenario."----
knitr::include_graphics('figures/densityplot_LayTN.png')

## ----echo=FALSE, out.width='100%', fig.cap="Density plot for the Lay TP scenario."----
knitr::include_graphics('figures/densityplot_LayTP.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# # the time serie plot type 1 (areas)
# input_Tserie(catch_data_TN, annual_data_TN, "Lay", "gr1")
# input_Tserie(catch_data_TP, annual_data_TP, "Lay", "gr1")

## ----echo=FALSE, out.width="45%", out.height="25%",fig.cap="Time series plot type 1 (areas) for TN and TP.",fig.show='hold',fig.align='center'----
knitr::include_graphics(c("figures/timeserieTN_type1.png","figures/timeserieTP_type1.png"))

## ----message=FALSE, eval=FALSE------------------------------------------------
# # the time serie plot type 2 (lines)
# input_Tserie(catch_data_TN, annual_data_TN,  "Lay", "gr2")
# input_Tserie(catch_data_TP, annual_data_TP,  "Lay", "gr2")

## ----echo=FALSE, out.width="45%", out.height="25%",fig.cap="Time series plot type 2 (lines) for TN and TP.",fig.show='hold',fig.align='center'----
knitr::include_graphics(c("figures/timeserieTN_type2.png","figures/timeserieTP_type2.png"))

## ----echo=FALSE, eval=FALSE---------------------------------------------------
# # the time serie plot type 3 (areas by year and km2)
# input_Tserie_area(catch_data_TN, annual_data_TN, sh_file, "Lay", "gr1")
# input_Tserie_area(catch_data_TP, annual_data_TP, sh_file, "Lay", "gr1")

## ----echo=FALSE, out.width="45%", out.height="25%",fig.cap="Time series plot type 3 (areas by year and km2) for TN and TP.",fig.show='hold',fig.align='center'----
knitr::include_graphics(c("figures/timeserieTN_type3.png","figures/timeserieTP_type3.png"))

## ----message=FALSE, eval=FALSE------------------------------------------------
# # the time serie plot type 4 (by km2 and Shreve)
# input_Tserie_area(catch_data_TN, annual_data_TN, sh_file, "Lay", "gr2")
# input_Tserie_area(catch_data_TP, annual_data_TP, sh_file, "Lay", "gr2")

## ----echo=FALSE, out.width="45%", out.height="25%",fig.cap="Time series plot type 4, thatcompares the levels of nutrient inputs in three zones of the basin.",fig.show='hold',fig.align='center'----
knitr::include_graphics(c("figures/timeserieTN_type4.png","figures/timeserieTP_type4.png"))

## ----message=FALSE, eval=FALSE------------------------------------------------
# # the Input Load Map by source type 1 (kt/year)
# input_maps(catch_data_TN, annual_data_TN, sh_file, "gr1", legend_position = 3)

## ----echo=FALSE, out.width='100%', fig.cap="Input map by source type 1 for TN scenario"----
#![Input maps.](figures/input_maps.png){width=75%} #300px for HTML
knitr::include_graphics('figures/input_mapsTN_type1.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# # the Input Load Map by source type 1 (kt/year)
# input_maps(catch_data_TP, annual_data_TP, sh_file, "gr1", legend_position = 3)

## ----echo=FALSE, out.width='100%', fig.cap="Input map by source type 1 for TP scenario"----
#![Input maps.](figures/input_maps.png){width=75%} #300px for HTML
knitr::include_graphics('figures/input_mapsTP_type1.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# # the Input Load Map by source type 2 (kt/y/km2)
# input_maps(catch_data_TN, annual_data_TN, sh_file, "gr2", legend_position = 3)

## ----echo=FALSE, out.width='100%', fig.cap="Input map by source type 2 for TN scenario"----
knitr::include_graphics('figures/input_mapsTN_type2.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# # the Input Load Map by source type 2 (kt/y/km2)
# input_maps(catch_data_TP, annual_data_TP, sh_file, "gr2", legend_position = 3)

## ----echo=FALSE, out.width='100%', fig.cap="Input map by source type 2 for TP scenario"----
knitr::include_graphics('figures/input_mapsTP_type2.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# LakeRetent_plot(catch_data_TN)

## ----echo=FALSE, out.width='100%', fig.cap="Lake retention values distribution for TN scenario"----
knitr::include_graphics('figures/LakeRetent_plot.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# TN_ref_values <- annual_data_TN[!is.na(annual_data_TN$YearlyMass),]
# nrow(TN_ref_values)
# table(TN_ref_values$HydroID,TN_ref_values$YearValue)
# 
# references_plot(annual_data_TN)

## ----echo=FALSE, out.width='100%', fig.cap="Observed references distribution for TN scenario"----
knitr::include_graphics('figures/references.png')

## ----message=FALSE------------------------------------------------------------
# Parameter for the calibration of the model
# the lower limits for all params (alpha_P, alpha_L, sd_coef)
low <- c(10, 0.000, 0.1) 
# the upper limits for all params (alpha_P, alpha_L, sd_coef)  
upp <- c(50, 0.08, 0.9)       

# number of iterations
n_iter <- 200 

# years in which the model should be executed
years <- c(2003:2009)

## ----message=FALSE, eval=FALSE------------------------------------------------
# # execution of the calibration
# DF_calib_TN <- calib_green(catch_data_TN, annual_data_TN, n_iter, low, upp, years)
# DF_calib <- data.frame(DF_calib)

## ----message=FALSE, eval=FALSE------------------------------------------------
# # Generating the box plots
# rateBS <-  5  #  percent of parameters selected from the whole calibration set
# calib_boxplot(DF_calib_TN, rateBS)

## ----echo=FALSE, out.width='100%'---------------------------------------------
knitr::include_graphics('figures/calibration_boxplotTN.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# # Generating the dot plots
# Gof_mes <- "NSE"
# calib_dot(DF_calib_TN, Gof_mes)

## ----echo=FALSE, out.width='100%'---------------------------------------------
knitr::include_graphics('figures/calibration_dotTN.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# # Generating the scatter plots
# Gof_mes <- "NSE"
# scatter_plot(DF_calib_TN, Gof_mes)

## ----echo=FALSE, out.width='100%'---------------------------------------------
knitr::include_graphics('figures/calibration_scatterTN.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# Gof_mes <- "NSE" # according the NSE goodness of fit metric
# NSE_bestParams <- select_params(DF_calib_TN, Gof_mes)
# NSE_bestParams
# Param_NSE2 <- as.numeric(NSE_bestParams[2:4])

## ----message=FALSE, eval=FALSE------------------------------------------------
# Gof_mes <- "rNSE"
# rNSE_bestParams <- select_params(DF_calib_TN,Gof_mes)
# rNSE_bestParams
# Param_rNSE2 <- as.numeric(rNSE_bestParams[2:4])

## ----message=FALSE, eval=FALSE------------------------------------------------
# # annual scatter plot comparing observed vs modeled loads by year
# label_plot <- "NSE best params for TN in the Lay"
# Themx <- max(annual_data_TN$YearlyMass, na.rm = TRUE)
# simobs_annual_plot(catch_data_TN, annual_data_TN, Param_NSE2[1], Param_NSE2[2], Param_NSE2[3], years, label_plot, Themx)

## ----echo=FALSE, out.width='100%'---------------------------------------------
knitr::include_graphics('figures/annual_scatter_vs_modeled_loadsTN.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# setPlabels <- c("NSE", "rNSE")
# label_plot <- "Comparing two sets of parameters for Lay TN"
# compare_calib(catch_data_TN, annual_data_TN, Param_NSE2[1], Param_NSE2[2], Param_NSE2[3], Param_rNSE2[1], Param_rNSE2[2], Param_rNSE2[3], years, label_plot, setPlabels)

## ----echo=FALSE, out.width='100%'---------------------------------------------
knitr::include_graphics('figures/comparing_twosets_paramsTN.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# # Computing the nutrient balance
# years <- c(2003:2009)
# Nut_bal_TN <- region_nut_balance(catch_data_TN, annual_data_TN, Param_NSE2[1], Param_NSE2[2], Param_NSE2[3], years, atm_coeff = 0.38)
# # Plot the sankey plot with the result of the balance
# sank <- N4_sankey(Nut_bal_TN)

## ----echo=FALSE, out.width='100%'---------------------------------------------
knitr::include_graphics('figures/n4sankey.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# years <- c(2003:2009)
# basin_sa <- green_shares(catch_data_TN, annual_data_TN, Param_NSE2[1], Param_NSE2[2], Param_NSE2[3], years)

## ----message=FALSE, eval=FALSE------------------------------------------------
# # The title of the plot
# plot_title <- "Time series Load Output for the Lay Basin"
# outFile <- tempfile(pattern = "OutletLoad", fileext = ".csv")
# nutrient_tserie(basin_sa, plot_title, "gr1", outFile)

## ----echo=FALSE, out.width='100%'---------------------------------------------
knitr::include_graphics('figures/timeserie_loadoutputLay_type1.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# # Output Load Basin average time series (lines)
# nutrient_tserie(basin_sa, plot_title, "gr2")

## ----echo=FALSE, out.width='100%'---------------------------------------------
knitr::include_graphics('figures/timeserie_loadoutputLay_type2.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# # Total Load in the Basin Outlet by source apportionment time series (lines)
# nutrient_tserie_darea(basin_sa, sh_file, "Lay Basin", "gr1")

## ----echo=FALSE, out.width='100%'---------------------------------------------
knitr::include_graphics('figures/timeserie_totalloadLay.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# style <- "log10"
# nutrient_maps(basin_sa, sh_file, "gr1", style, legend_position = 3)

## ----echo=FALSE, out.width='100%'---------------------------------------------
knitr::include_graphics('figures/nutrient_maps_outputLoadsType1.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# # Basin Output Total Load  Maps
# nutrient_maps(basin_sa, sh_file, style, "gr2", legend_position = 3)

## ----echo=FALSE, out.width='100%'---------------------------------------------
knitr::include_graphics('figures/nutrient_maps_outputLoadsType2.png')

## ----message=FALSE, eval=FALSE------------------------------------------------
# # Basin Output Specific Load by km2 Maps
# nutrient_maps(basin_sa, sh_file, style, "gr3", legend_position = 3)

## ----echo=FALSE, out.width='100%'---------------------------------------------
knitr::include_graphics('figures/nutrient_maps_outputLoadsType3.png')

#' @title Sh file
#' @description An sf (simple features) object  that includes attributes and
#' geometries in the form of a data frame. Data frame (or tibble) with rows of
#' features, columns of attributes, and a special geometry column that contains
#' the spatial aspects of the features
#' @format Simple feature collection with 6 features and 14 fields:
#' \describe{
#'   \item{\code{OBJECTID}}{integer Identifier of the spatial object (one for
#'   each catchment)}
#'   \item{\code{HydroID}}{integer Unique catchment identifier}
#'   \item{\code{NextDownID}}{integer Unique identifier of the catchment to
#'   which the catchment goes}
#'   \item{\code{BasinID}}{integer Identifier of the Basin to all  catchment
#'   belong}
#'   \item{\code{JunctionID}}{integer Unique identifier of the catchment to
#'   which the catchment goes. Note that for the outlet JunctionID == -1}
#'   \item{\code{AreaSqkm}}{double Area of each catchment}
#'   \item{\code{DrainAreaS}}{double Drain Area of each catchment}
#'   \item{\code{Shreve}}{integer Indicates Shreve order, i.e. in headwater = 1;
#'   at the outlet Shreve = max(Shreve)}
#'   \item{\code{geometry}}{sfc objects represent the geometry of a single
#'   feature and contains information about the feature’s coordinates,
#'   dimension, and type of geometry}
#'}
#' @keywords internal
"sh_file"

# 'br2': Weighted R2
# Coef. of  determination multiplied by the coef. of the regression line#
# This index allows accounting for the discrepancy in the magnitude of two signals
# under or overpredictions, (depicted by 'b') as well as their dynamics (depicted by R2).
# 'obs'   : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim'   : numeric 'data.frame', 'matrix' or 'vector' with simulated values
# 'Result': weighted R2 between 'sim' and 'obs'
#' @importFrom stats coefficients cor lm
#' @noRd
br2 <- function (sim, obs, na.rm=TRUE){

  x.lm <- stats::lm(sim ~ obs - 1)

  b <- as.numeric(stats::coefficients(x.lm)["obs"]   )

  r2 <- (rPearson(sim, obs))^2

  br2 <- ifelse(b <= 1, r2*abs(b), r2/abs(b))

  return(br2)

}

# 'P': Coefficient of Persistence
# PME ranges from 0 to 1, with PME = 1 being the optimal value.
# PME values should be larger than 0.0 to indicate a minimally acceptable
# model performance (Gupta et al., 1999
# 'obs'   : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim'   : numeric 'data.frame', 'matrix' or 'vector' with simulated values
# 'Result': Persistence Index Efficiency between 'sim' and 'obs'
cp <- function (sim, obs, na.rm=TRUE){

  n <- length(obs)

  denominator <- sum( ( obs[2:n] - obs[1:(n-1)] )^2 )

  if (denominator != 0) {

    cp <- ( 1 - ( sum( (obs[2:n] - sim[2:n])^2 ) / denominator ) )

  } else stop("'sum((obs[2:n]-obs[1:(n-1))^2)=0', it is not possible to compute 'P'")

  return(cp)

}

# 'IoA': Index of Agreement
# Index of Agreement (Willmott et al., 1984) range from 0.0 to 1.0
# and the closer to 1 the better the performance of the model
# 'obs'   : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim'   : numeric 'data.frame', 'matrix' or 'vector' with simulated values
# 'Result': Index of Agreement between 'sim' and 'obs'
d <- function (sim, obs, na.rm=TRUE){

  Om <- mean(obs)

  denominator <- sum( ( abs(sim - Om) + abs(obs - Om)  )^2 )

  if (denominator != 0) {

    d <- 1 - ( sum( (obs - sim)^2 ) / denominator )

  } else stop("'sum((abs(sim-Om)+abs(obs-Om))^2)=0', it is not possible to compute 'IoA'")

  return(d)

}

#' GOF
#'
#' @param obs numeric 'data.frame', 'matrix' or 'vector' with observed values
#' @param sim numeric 'data.frame', 'matrix' or 'vector' with simulated values
#' @param digits Number of digits
#' @param na.rm Should remove NA
#' @export
#' @keywords internal
#' @returns A data frame
gof <- function(sim, obs, na.rm=TRUE, digits=2){
  ME     <- me(sim, obs, na.rm=na.rm)
  MAE    <- mae(sim, obs, na.rm=na.rm)
  MSE    <- mse(sim, obs, na.rm=na.rm)
  RMSE   <- rmse(sim, obs, na.rm=na.rm)
  NRMSE  <- nrmse(sim, obs, na.rm=na.rm)
  RSR    <- rsr(sim, obs, na.rm=na.rm)
  rSD    <- rSD(sim, obs, na.rm=na.rm)
  PBIAS  <- pbias(sim, obs, na.rm=na.rm)
  NSE    <- NSE(sim, obs, na.rm=na.rm)
  mNSE   <- mNSE(sim, obs, na.rm=na.rm)
  rNSE   <- rNSE(sim, obs, na.rm=na.rm)
  d      <- d(sim, obs, na.rm=na.rm)
  md     <- md(sim, obs, na.rm=na.rm)
  rd     <- rd(sim, obs, na.rm=na.rm)
  cp     <- cp(sim, obs, na.rm=na.rm)
  r      <- rPearson(sim, obs)
  bR2    <- br2(sim, obs, na.rm=na.rm)
  KGE    <- KGE(sim, obs, na.rm=na.rm)
  VE     <- VE(sim, obs, na.rm=na.rm)
  R2 <- r^2

  gof <- rbind(ME, MAE, MSE, RMSE, NRMSE, PBIAS, RSR, rSD, NSE, mNSE, rNSE, d, md, rd, cp, r, R2, bR2, KGE, VE)

  # rownames(gof)[5] <- "NRMSE %"
  # rownames(gof)[6] <- "PBIAS %"

  gof <- round(gof, digits)

  return(gof)

}

# 'KGE': Kling-Gupta Efficiency
# 'obs' : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim' : numeric 'data.frame', 'matrix' or 'vector' with simulated values
# 's'   : scaling factors.
# 'Result': Kling-Gupta Efficiency between 'sim' and 'obs'
KGE <- function(sim, obs, s=c(1,1,1), na.rm=TRUE,
                        method="2009") {

  # Mean values
  mean.sim <- mean(sim, na.rm=na.rm)
  mean.obs <- mean(obs, na.rm=na.rm)

  # Standard deviations
  sigma.sim <- sd(sim, na.rm=na.rm)
  sigma.obs <- sd(obs, na.rm=na.rm)

  # Pearson product-moment correlation coefficient
  r <- rPearson(sim, obs)

  # Alpha is a measure of relative variability between simulated and observed values (See Ref1)
  Alpha <- sigma.sim / sigma.obs

  # Beta is the ratio between the mean of the simulated values to the mean of observations
  Beta <- mean.sim / mean.obs

  # CV.sim is the coefficient of variation of the simulated values [dimensionless]
  # CV.obs is the coefficient of variation of the observations [dimensionless]
  CV.sim <- sigma.sim / mean.sim
  CV.obs <- sigma.obs / mean.obs

  # Gamma is the variability ratio, which is used instead of Alpha (See Ref2)
  Gamma <- CV.sim / CV.obs

  # Variability ratio depending on 'method'
  if(method=="2012") {
    vr     <- Gamma
    vr.stg <- "Gamma"
  } else {
    vr     <- Alpha
    vr.stg <- "Alpha"
  } # ELSE end

  # KGE Computation
  if ( (mean.obs != 0) | (sigma.obs != 0) ) {
    KGE <- 1 - sqrt( (s[1]*(r-1))^2 + (s[2]*(vr-1))^2 + (s[3]*(Beta-1))^2 )
  } else {
    if ( mean.obs != 0)  warning("Warning: 'mean(obs)==0'. Beta = Inf")
    if ( sigma.obs != 0) warning("Warning: 'sd(obs)==0'. ", vr.stg, " = Inf")
    KGE <- NA
  }

  out <- KGE

  return(out)

}

# 'mae': Mean Absolute Error
# 'obs'   : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim'   : numeric 'data.frame', 'matrix' or 'vector' with simulated values
mae <- function(sim, obs, na.rm = TRUE){
  mae <- mean( abs(sim - obs), na.rm = na.rm)

  return(mae)
}

# 'md': Modified Index of Agreement
# Index of Agreement (Willmott et al., 1984) range from 0.0 to 1.0
# and the closer to 1 the better the performance of the model
# 'obs' : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim' : numeric 'data.frame', 'matrix' or 'vector' with simulated values
# 'Result': Modified Index of Agreement between 'sim' and 'obs'
md <- function (sim, obs, j=1, na.rm=TRUE){

  Om <- mean(obs)

  denominator <- sum( ( abs(sim - Om) + abs(obs - Om)  )^j )

  if (denominator != 0) {

    d1 <- 1 - ( sum( ( abs(obs - sim) )^j ) / denominator )

  } else {
    d1 <- NA
    warning("'sum((abs(sim-Om)+abs(obs-Om))^j)=0', it is not possible to compute 'md'")
  }

  return(d1)

}

# 'me': Mean Error
# 'obs'   : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim'   : numeric 'data.frame', 'matrix' or 'vector' with simulated values
me <- function(sim, obs, na.rm = TRUE){
  me <- mean( sim - obs, na.rm = na.rm)

  return(me)
}

# 'mNSE': Modified Nash-sutcliffe Efficiency
# 'obs' : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim' : numeric 'data.frame', 'matrix' or 'vector' with simulated values
# 'Result': Modified Nash-sutcliffe Efficiency between 'sim' and 'obs'
mNSE <- function (sim, obs, j=1, na.rm=TRUE){

  denominator <- sum( abs(obs - mean(obs))^j )

  if (denominator != 0) {

    NS1 <- 1 - ( sum( abs(obs - sim)^j ) / denominator )

  } else {
    NS1 <- NA
    warning("'sum(abs(obs - mean(obs))^j)=0', it is not possible to compute 'mNSE'")
  }

  return(NS1)

}

# 'mse': Mean Squared Error
# 'obs'   : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim'   : numeric 'data.frame', 'matrix' or 'vector' with simulated values
mse <- function(sim, obs, na.rm = TRUE){
  mse <- mean( (sim - obs)^2, na.rm = na.rm)

  return( mse )
}

# 'nrmse': Normalized Root Mean Square Error
# 'obs'   : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim'   : numeric 'data.frame', 'matrix' or 'vector' with simulated values
nrmse <- function(sim, obs, na.rm=TRUE){
  cte <- sd(obs, na.rm=na.rm)

  rmse <- rmse(sim, obs, na.rm)

  if (max(obs, na.rm= na.rm) - min(obs, na.rm= na.rm) != 0) {

    nrmse <- rmse / cte

  } else {
    nrmse <- NA
    warning("'obs' is constant, it is not possible to compute 'nrmse'")
  } # ELSE end

  return( round( 100*nrmse, 1) )
}

# 'NSE': Nash-sutcliffe Efficiency
# 'obs'   : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim'   : numeric 'data.frame', 'matrix' or 'vector' with simulated values
NSE <- function(sim, obs, na.rm=TRUE){
  denominator <- sum( (obs - mean(obs))^2 )

  if (denominator != 0) {

    NS <- 1 - ( sum( (obs - sim)^2 ) / denominator )

  } else {
    NS <- NA
    warning("'sum((obs - mean(obs))^2)=0' => it is not possible to compute 'NSE'")
  } # ELSE end

  return(NS)
}

# 'pbias': Percent Bias
# 'obs'   : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim'   : numeric 'data.frame', 'matrix' or 'vector' with simulated values
pbias <- function (sim, obs, na.rm=TRUE){

  n <- length(obs)

  denominator <- sum( obs )

  if (denominator != 0) {

    pbias <- 100 * ( sum( sim - obs ) / denominator )

  } else {
    pbias <- NA
    warning("'sum((obs)=0', it is not possible to compute 'pbias'")
  }

  return( round(pbias, 1) )

}

# 'rd': Relative Index of Agreement
# Relative Index of Agreement (Willmott et al., 1984) range from 0.0 to 1.0
# and the closer to 1 the better the performance of the model
# 'obs' : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim' : numeric 'data.frame', 'matrix' or 'vector' with simulated values
# 'Result': Modified Index of Agreement between 'sim' and 'obs'
# This index was developed to  be sensitive to systematic over- or
# under-prediction, in particular during low flow conditions.
# This index quantify the difference between simulated and observed values
# in a relative way, in order to significatively reduce the influence of
# the absolute differences of high flows and to give more weight to
# over- or under-prediction of low flows. At the same time, differences in
# low flows become more important, because they are looked in a relative way.
rd <- function (sim, obs, na.rm=TRUE){

  Om <- mean(obs)

  denominator <- sum( ( ( abs(sim - Om) + abs(obs - Om) ) / Om )^2 )

  if (denominator != 0) {

    rd <- 1 - ( sum( ( (obs - sim) / obs)^2 ) / denominator )

  } else {
    rd <- NA
    warning("'sum( ( ( abs(sim-Om) + abs(obs-Om) ) / Om )^2 ) = 0', it is not possible to compute 'rd'")
  }

  return(rd)

}

# 'rmse': Root Mean Square Error
# 'obs'   : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim'   : numeric 'data.frame', 'matrix' or 'vector' with simulated values
rmse <- function(sim, obs, na.rm=TRUE){
  rmse <- sqrt(mean((sim - obs)^2, na.rm = na.rm))

  return(rmse)
}

# 'rNSE': Relative Nash-sutcliffe Efficiency
# Nash-Sutcliffe efficiency not "inflated" by squared values
# Essentially, the closer the model efficiency is to 1, the more accurate the model is.
# 'obs' : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim' : numeric 'data.frame', 'matrix' or 'vector' with simulated values
# 'Result': Modified Nash-sutcliffe Efficiency between 'sim' and 'obs'
rNSE <- function (sim, obs, na.rm=TRUE){

  zero.index <- which(obs==0)
  if (length(zero.index > 0) ) {
    warning("'rNSE' can not be computed: some elements in 'obs' are zero !", call.=FALSE)
  }

  denominator <- sum( ( ( obs - mean(obs) ) / mean(obs) )^2 )

  if (denominator != 0) {

    rNSE <- 1 - ( sum( ( (obs - sim) / obs )^2 ) / denominator )

  } else {
    rNSE <- NA
    warning("'sum( ( ( obs - mean(obs) ) / mean(obs) )^2 ) = 0', it is not possible to compute 'rNSE'")
  }

  return(rNSE)

}

#   rPearson
# The 'r.Pearson' coefficient ranges from −1 to 1.
# A value of 1 shows that a linear equation describes the relationship
# perfectly and positively, with all data points lying on the same line
# and with Y increasing with X.
# A score of −1 shows that all data points lie on a single line but
# that Y increases as X decreases.
# A value of 0 shows that a linear model is not needed – that there
# is no linear relationship between the variables.
# 'obs' : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim' : numeric 'data.frame', 'matrix' or 'vector' with simulated values
#' @importFrom stats coefficients cor lm
rPearson <- function(sim, obs, na.rm=TRUE) {

  rPearson <- stats::cor(sim, obs, method="pearson", use="pairwise.complete.obs")

  return(rPearson)

}

# 'rSD': Ratio of Standard Deviations
# 'obs'   : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim'   : numeric 'data.frame', 'matrix' or 'vector' with simulated values
rSD <- function (sim, obs, na.rm=TRUE){

  denominator <- sd(obs, na.rm = na.rm)

  if (denominator != 0) {

    rSD <- sd(sim, na.rm= na.rm) / sd(obs, na.rm= na.rm)

  } else {
    rSD <- NA
    warning("'sd(obs)=0', it is not possible to compute 'rSD'")
  }

  return(rSD)

}

# 'rsr': Ratio of RMSE to the Standard Deviation of the Observations
# 'obs'   : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim'   : numeric 'data.frame', 'matrix' or 'vector' with simulated values
rsr <- function(sim, obs, na.rm=TRUE){
  rmse <- rmse(sim=sim, obs=obs, na.rm=na.rm)

  #Standard deviation of the observations
  sd.obs <- sd(obs, na.rm=na.rm)

  if ( sd.obs > 0 ) {

    rsr <- rmse / sd.obs

  } else {
    rsr <- NA
    warning("'sd(obs)=0', it is not possible to compute 'RSR'")
  } # ELSE end

  return( rsr )
}

# 'VE': Volumetric Efficiency
# 'obs'   : numeric 'data.frame', 'matrix' or 'vector' with observed values
# 'sim'   : numeric 'data.frame', 'matrix' or 'vector' with simulated values
# 'Result': Mean Absolute Error between 'sim' and 'obs', in the same units of 'sim' and 'obs'
VE <- function (sim, obs, na.rm=TRUE){

  denominator <- sum(obs, na.rm=na.rm)

  if (denominator != 0) {
    ve <- 1 - ( sum( abs(sim-obs) ) / denominator )
  } else {
    ve <- NA
    warning("'sum((obs)=0' => it is not possible to compute 'VE'")
  }

  return(ve)

}

#'
#' @title Calculate shreve
#'
#' @param catch_data data frame.
#'
#' @importFrom stats aggregate
#'
#' @return A data frame
#'
#' @keywords internal
#'
calculate_shreve <- function(catch_data) {

  catch_data$NewShreve <- NA

  all_orig <- unique(catch_data$HydroID)
  all_desting <- unique(catch_data$To_catch)
  whole_hydroID <- unique(c(AllOrig, AllDesting))
  Whole_hydroID <- setdiff(WholeHydroID, -1)
  remai_hydroId <- Whole_hydroID

  shr1 <- setdiff(AllOrig, AllDesting)
  catch_data[catch_data$Shreve == 1, "HydroID"]

  catch_data$NewShreve[catch_data$HydroID %in% shr1 ] <- 1
  the_srh_table <- rbind(data.frame(HydroID = shr1, shreve = 1))

  remai_hydroId <- setdiff(remai_hydroId, shr1)

  while (length(remai_hydroId) > 0) {

    the_scen2 <- catch_data[catch_data$To_catch %in% RemaiHydroId, ]
    dest_to_remov <- the_scen2$To_catch[is.na(the_scen2$NewShreve)]
    the_scen3 <- the_scen2[!(the_scen2$To_catch %in% dest_to_remov), ]

    nex_shr <- stats::aggregate(the_scen3$NewShreve,
                                by = list(the_scen3$To_catch), max)
    names(nex_shr) <-c("HydroID ","NewShreve")

    nex_shr$NewShreve <- nex_shr$NewShreve + 1

    catch_data$NewShreve[catch_data$HydroID %in% nex_shr$HydroID] <-
      nex_shr$NewShreve

    remai_hydroId <- setdiff(remai_hydroId, nex_shr$HydroID)
    print(paste("Length of remain HydroID", length(remai_hydroId)))

  }

  catch_data
}

#'
#' @title Boxplot of best parameters
#'
#' @description Returns boxplots of best model parameters ranked according to
#' different goodness-of-fit measures, and also boxplot with the distribution of
#'  the parameters values.
#'
#' @param df_cb data frame. Table with the result of the calibration process.
#' @param rate_bs numeric. Rate (\%) of parameters selected from the whole set
#' produced in the calibration.
#'
#' @importFrom graphics par boxplot points
#'
#' @return Multiple boxplots
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(catch_data_TP)
#' data(annual_data_TP)
#' # the parameter for the calibration of the model
#' n_iter <- 2 # number of iterations
#' # the lower limits for all params (alpha_P, alpha_L, sd_coef)
#' low <- c(10, 0.000, 0.1)
#' # the upper limits for all params (alpha_P, alpha_L, sd_coef)
#' upp <- c(70, 0.3,  0.9)
#' # years in which the model should be executed
#' years <- 1990:2018
#' # execution of the calibration
#' df_calib <- calib_green(catch_data_TP, annual_data_TP, n_iter, low, upp,
#' years)
#' # Generating the box plots
#' rateBS <- 5 # rate of best set of parameter to include in the plots
#' calib_boxplot(df_calib, rateBS)
#' }
#'
#' @export
#'
calib_boxplot <- function(df_cb, rate_bs) {


  num_best <- nrow(df_cb) * rate_bs / 100

  bes_par <- NULL
  top_best <- NULL
  ind_mtr <- c(18,6,9,8,5,10, 17, 19, 15, 20 )    # PBIAS.. cp  NSE    mNSE    rNSE  R2
  for(ind in ind_mtr) {
    # ind <- 19
    print(ind)
    if (ind == 6 | ind == 5) {
      irow <- which(abs(df_cb[, ind]) ==  min(abs(df_cb[, ind]),
                                              na.rm = TRUE))[1]
    } else if (ind != 6 & ind != 5) {
      irow <- which(df_cb[,ind] == max(df_cb[, ind], na.rm = TRUE))[1]
    }
    bes_par <- rbind(bes_par, data.frame(metric = names(df_cb)[ind],
                                         df_cb[irow, ]))

    if (ind == 6 | ind == 5) {
      top_best <-
        rbind(top_best,
              data.frame(metric = names(df_cb)[ind],
                         df_cb[order(abs(df_cb[, ind]))[1:num_best], ]))

    } else if (ind != 6 & ind != 5) {
      top_best <-
        rbind(top_best,
              data.frame(metric = names(df_cb)[ind],
                         df_cb[order(-df_cb[,ind])[1:num_best] ,]))
    }


  }

  unique(bes_par$metric)

  graphics::par(mfrow = c(3, 4), mar = c(4.1, 3.5, 2.1, 0.5))
  graphics::boxplot(top_best$KGE ~ top_best$metric,
                    las = 2,
                    main = "KGE",
                    xlab = "",
                    ylab = "")
  graphics::boxplot(top_best$NSE ~ top_best$metric,
                    las = 2,
                    main = "NSE",
                    xlab = "",
                    ylab = "")
  graphics::boxplot(top_best$mNSE ~ top_best$metric,
                    las = 2,
                    main = "mNSE",
                    xlab = "",
                    ylab = "")
  graphics::boxplot(top_best$rNSE ~ top_best$metric,
                    las = 2,
                    main = "rNSE",
                    xlab = "",
                    ylab = "")
  graphics::boxplot(top_best$PBIAS ~ top_best$metric,
                    las = 2,
                    main ="PBIAS",
                    xlab = "",
                    ylab = "")
  graphics::boxplot(top_best$R2 ~ top_best$metric,
                    las = 2,
                    main = "R2",
                    xlab = "",
                    ylab = "")
  graphics::boxplot(top_best$NRMSE ~ top_best$metric,
                    las = 2,
                    main = "NRMSE",
                    xlab = "",
                    ylab = "")
  graphics::boxplot(top_best$cp ~ top_best$metric,
                    las = 2,
                    main = "cp",
                    xlab = "",
                    ylab = "")

  bes_par <- bes_par[order(bes_par$metric), ]
  graphics::boxplot(top_best$alpha_P ~ top_best$metric,
                    las = 2,
                    main = "alpha_P",
                    xlab = "",
                    ylab = "")
  graphics::points(bes_par["alpha_P"], col = "red", pch = 18)
  graphics::boxplot(top_best$alpha_L ~ top_best$metric,
                    las = 2,
                    main = "alpha_L",
                    xlab = "",
                    ylab = "")
  graphics::points(bes_par["alpha_L"], col="red",pch=18)
  graphics::boxplot(top_best$sd_coeff ~ top_best$metric,
                    las = 2,
                    main = "sd_coeff",
                    xlab = "",
                    ylab = "")
  graphics::points(bes_par["sd_coeff"], col = "red", pch = 18)
  graphics::boxplot(top_best$SumOutLoad ~ top_best$metric,
                    las = 2,
                    main ="SumOutLoad",
                    xlab = "",
                    ylab = "")
  graphics::points(bes_par["SumOutLoad"], col = "red", pch = 18)

}

#'
#' @title Dot plot of goodness-of-fit metric vs parameter value
#'
#' @description Dot plot of goodness-of-fit metric vs parameters value
#'
#' @param df_cb data frame. A table with the result of the calibration process.
#' @param param character. Goodness of fit measures. See alternatives link "NSE"
#' "rNSE", "NSE", "mNSE", "MAE", "PBIAS", "cp", "R2".
#'
#' @importFrom grDevices colorRampPalette
#' @importFrom classInt classIntervals
#' @importFrom graphics par layout plot legend
#'
#' @return Multiple dot plots
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(catch_data_TN)
#' data(annual_data_TN)
#' # the parameter for the calibration of the model
#' n_iter <- 2 # number of iterations
#' # the lower limits for all params (alpha_P, alpha_L, sd_coef)
#' low <- c(10, 0.000, 0.1)
#' # the upper limits for all params (alpha_P, alpha_L, sd_coef)
#' upp <- c(70, 0.3,  0.9)
#' # years in which the model should be executed
#' years <- 1990:2018
#' # execution of the calibration
#' df_calib <- calib_green(catch_data_TN, annual_data_TN, n_iter, low, upp,
#' years)
#' # Generating the dot plots
#' gof_mes <- "NSE"
#' calib_dot(df_calib, gof_mes)
#' }
#'
#' @export
#'
calib_dot <- function(df_cb, param) {

  rbPal <- grDevices::colorRampPalette(
    c('red4', 'red', 'orange', 'green', "green4", "cyan1"))

  intervals <- classInt::classIntervals(df_cb[, param],
                                        n = min(nrow(df_cb), 6),
                                        style = "quantile",
                                        warnSmallN = FALSE)
  cuts <- cut(df_cb[, param], intervals$brks, labels = FALSE,
              include.lowest = TRUE)
  df_cb$colors <- rbPal(min(nrow(df_cb), 6))[as.numeric(cuts)]

  oldpar <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(oldpar))
  graphics::par(mar = c(4.1, 4.1, 2.1, 2.1))

  graphics::layout(mat = matrix(c(1, 2, 3, 4),
                                nrow = 2,
                                ncol = 2))


  with(df_cb, {
    graphics::plot(alpha_P, alpha_L, col = df_cb$colors,
                   xlab = "alpha_P", ylab = "alpha_L", pch = 20)
  })

  with(df_cb, {
    graphics::plot(alpha_P, sd_coeff, col = df_cb$colors,
                   xlab = "alpha_P", ylab = "sd_coeff", pch = 20)
  })

  with(df_cb, {
    graphics::plot(sd_coeff, alpha_L, col = df_cb$colors,
                   xlab = "sd_coeff", ylab = "alpha_L", pch = 20)
  })

  DF_leg <- data.frame(
    cbind(lower = round(intervals$brks[1:(length(intervals$brks) - 1)],
                        digits = 2),
          upper = round(intervals$brks[2:length(intervals$brks)], digits = 2)))

  labels <- paste0("(" , DF_leg$lower," , " , DF_leg$upper, "]")
  graphics::plot(NULL, xaxt = 'n', yaxt = 'n', bty = 'n',
                 ylab = '', xlab = '', xlim = 0:1, ylim = 0:1)
  graphics::legend("center", title = param, legend = labels,
                   col = rbPal(min(nrow(df_cb), 6)),
                   pch = 16, bty = "n", cex = 1.5)

}

#'
#' @title Plot comparing observed vs modeled loads for two set of parameters
#'
#' @description Returns a scatter plot comparing observed versus modeled loads
#' obtained with two model parameter sets
#'
#' @param catch_data data frame. Definition of the topological sequence of
#' catchments.
#' @param annual_data data frame. Sources of nutrient for each year and
#' catchments.
#' @param alpha_p1 numeric. The basin retention coefficient of the first set of
#' parameters.
#' @param alpha_l1 numeric. The river retention coefficient of the first set of
#' parameters.
#' @param sd_coef1 numeric. Fraction of domestic diffuse sources that reaches
#' the stream network of the first set of parameters.
#' @param alpha_p2 numeric. The basin retention coefficient of the second set of
#' parameters.
#' @param alpha_l2 numeric. The river retention coefficient of the second set of
#' parameters.
#' @param sd_coef2 numeric. Fraction of domestic diffuse sources that reaches
#' the stream network of the second set of parameters.
#' @param name_basin character. Name of the basin (title of the plot).
#' @param years numeric. Years to be shown in the plot.
#' @param setPlabels character. Labels identifying each set of parameter.
#'
#' @importFrom graphics par plot abline legend
#'
#' @return A scatter plot and a list with two data frames with model GREEN
#' applied to two model parameter sets
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(catch_data_TN)
#' data(annual_data_TN)
#'
#' # the first set of parameters to assess the basin model
#' alpha_p <- 35.09
#' alpha_l <- 0.02
#' sd_coef <- 0.2
#'
#' # the second set of parameters to assess the basin model
#' alpha_p2 <- 41.23
#' alpha_l2 <- 0.0015
#' sd_coef2 <- 0.6
#'
#' # years in which the plot will we shown
#' years <- 1990:2018
#'
#' nameBasin <- "Lay"
#'
#' # generating the scatter plot comparing two set of parameters observed
#' # versus modeled loads by year
#' setPlabels <- c("bestNSE","bestR2")
#' compare_calib(catch_data_TN, annual_data_TN, alpha_p , alpha_l, sd_coef,
#' alpha_p2, alpha_l2, sd_coef2, years, nameBasin, setPlabels)
#' }
#'
#' @export
#'
compare_calib <- function(catch_data, annual_data, alpha_p1, alpha_l1,
                          sd_coef1, alpha_p2, alpha_l2, sd_coef2, years,
                          name_basin, setPlabels) {

  df_scen_global <- launch_green(catch_data, annual_data, alpha_p1, alpha_l1,
                                 sd_coef1, years)
  df_scen_local <- launch_green(catch_data, annual_data, alpha_p2, alpha_l2,
                                sd_coef2, years)

  df_gl <- df_scen_global[df_scen_global$To_catch == -1,]
  df_lo <- df_scen_local[df_scen_local$To_catch == -1,]

  df_observ_g <- df_scen_global[!is.na(df_scen_global$ObsLoad), ]
  df_observ_l <- df_scen_local[!is.na(df_scen_local$ObsLoad), ]

  df_observ_g <- df_observ_g %>%
    rename(PredictLoad = CatchLoad)

  gof_glo <- gof(sim = df_observ_g$PredictLoad,
                 obs = df_observ_g$ObsLoad,
                 digits = 4)
  gof_loc <- gof(sim = df_observ_l$CatchLoad,
                 obs = df_observ_l$ObsLoad,
                 digits = 4)

  result <- cbind(as.data.frame(gof_glo), as.data.frame(gof_loc))
  result$index <- rownames(result)
  result <- result[, c(3, 1, 2)]
  result[, 2:3] <- round(result[, 2:3], 3)
  names(result) <-c("index", setPlabels[1], setPlabels[2])
  result$gl <- apply( result , 1 , paste , collapse = "  " )

  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))
  graphics::par(mfrow = c(1, 1))
  graphics::plot(df_observ_g$ObsLoad,
                 df_observ_g$PredictLoad,
                 main = name_basin,
                 xlab = "Observed",
                 ylab = "Predicted",
                 xlim = c(0, max(df_observ_g$ObsLoad, df_observ_l$ObsLoad)),
                 ylim = c(0, max(df_observ_g$PredictLoad,
                                 df_observ_l$CatchLoad)))
  graphics::points(df_observ_l$ObsLoad,
                   df_observ_l$CatchLoad,
                   col = "red", pch = 8 )
  graphics::abline(coef = c(0, 1))
  graphics::legend('right', legend = c(setPlabels[1], setPlabels[2]) ,
                   col=c("black","red"), bty = 'n', pch = 15, cex = 1.3)
  graphics::legend('topleft', legend = result$gl[c(6, 9, 10, 11, 15, 17)],
                   col = "black", bty = 'n', cex = 1.0,
                   title = paste0("       ", setPlabels[1], "      ", setPlabels[2]))


  list(df_gl ,df_lo)
}

#
#' @title Create levels
#'
#' @description This function create the levels for the data frame.
#'
#' @param df,no_sf data frame.
#'
#' @return A list
#'
#' @importFrom stats quantile
#'
#' @keywords internal
#'
create_levels <- function(df, no_sf){

  df <- merge(df, no_sf, by = "HydroID")

  theQuantile <- stats::quantile(df$Shreve)
  ShrLevel1 <- df[df$Shreve <= theQuantile[3], ]$HydroID
  ShrLevel2 <- df[df$Shreve > theQuantile[3] &
                    df$Shreve <= theQuantile[4], ]$HydroID
  ShrLevel3 <- df[df$Shreve > theQuantile[4] &
                    df$Shreve <= theQuantile[5], ]$HydroID

  df$shrLevel <- NA
  if(!is.logical(df$shrLevel)) {
    df$shrLevel <- as.character(df$shrLevel)
  }
  df[df$HydroID %in% ShrLevel1, ]$shrLevel <- "level1"
  df[df$HydroID %in% ShrLevel2, ]$shrLevel <- "level2"
  df[df$HydroID %in% ShrLevel3, ]$shrLevel <- "level3"

  list("df" = df, "ShrLevel1" = ShrLevel1, "ShrLevel2" = ShrLevel2,
       "ShrLevel3" = ShrLevel3)
}

#
#' @title Preprocessing data for scenario summary
#'
#' @description This function iterates through a list of parameters to generate multiple
#' thematic maps based on a shared spatial dataset. It automates the styling,
#' scaling, and labeling for each map to ensure consistency across a series.
#'
#' @param hydroSf data frame.
#' @param long_basin numeric. The longitude of the basin.
#' @param unit character. The unit of the data.
#' @param param_list list. A list with the maps to plot.
#' @param style character. The style of the plot.
#' @param legend_position numeric. It indicates de position of the legend of the
#' plot. (Default: 1)
#' @return A list with maps
#'
#' @keywords internal
#'
create_lits_of_maps <- function(hydroSf, long_basin, unit, legend_position,
                                param_list, style){

  scale_barRef <- c(0, floor(long_basin / 8), floor(long_basin / 4))
  scale_barTextS <- 0.7

  map_list <- list()
  for (i in param_list){
    map <- create_map(hydroSf, i[1], style, scale_barRef, scale_barTextS,
                      paste(i[2], unit), palette = i[3],
                      legend_position = legend_position)
    map_list <- c(map_list, list(map))
  }

  map_list

}

#
#' @title Create map
#'
#' @description This function create a tm map.
#'
#' @param hydroSf data frame.
#' @param var_name character. The name of a data variable that it is contains
#' in data frame
#' @param style_map,scale_barRefs,scale_barTexts numeric.
#' @param title character. The title of the plot
#' @param palette character. The colors of the map
#' @param legend_position numeric. It indicates de position of the legend of the
#' plot. (Default: 1)
#'
#' @return A plot with map of the basin
#'
#' @importFrom tmap tm_shape tm_fill tm_layout tm_scalebar
#'
#' @keywords internal
#'
create_map <- function(hydroSf, var_name, style_map, scale_barRefs,
                       scale_barTexts, title, palette, legend_position = 1){

  tm <- tmap::tm_shape(hydroSf) +
    tmap::tm_fill(col = var_name,
                  style = style_map,
                  title = title,
                  palette = palette) +
    tmap::tm_scalebar(breaks = scale_barRefs,
                       text.size = scale_barTexts)

  if (legend_position == 1){
    position <- c("left", "bottom")
  } else if (legend_position == 2) {
    position <- c("left", "top")
  } else if(legend_position == 3) {
    position <- c("right", "bottom")
  } else if(legend_position == 4) {
    position <- c("right", "top")
  } else {
    stop("Invalid value for legend position.")
  }

  tm <- tm + tmap::tm_layout(legend.position = position,
                             legend.format = list(digits = 1))

  tm
}

#
#' @title Evolution plot
#'
#' @description This function plot the evolution of the data
#'
#' @param data data frame.
#' @param title character. The plot title
#' @param xaxis.title character. The title of x axis
#' @param yaxis.title character. The title of y axis
#' @param x character. The name of variable in data to represent in x axis
#' @param y character. The name of variable in data to represent in y axis
#' @param colour character. The name of variable in data to represent colour
#' @param wrap character. The name of variable in data to represent wrap
#' @return No return value, called for the side effect of drawing a plot
#'
#' @importFrom ggplot2 ggplot aes_string geom_line theme_bw labs
#' scale_color_brewer facet_wrap
#' @importFrom stats as.formula
#'
#' @keywords internal
#'
evolution_plot <- function(data, title = NULL, xaxis.title = NULL, yaxis.title,
                           x, y, colour = NULL, wrap = NULL){
  p <- ggplot2::ggplot(data = data,
                       ggplot2::aes_string(x = x, y = y, colour = colour)) +
    ggplot2::geom_line(alpha = 1.5, size = 1.2) +
    ggplot2::theme_bw(base_size = 15) +
    ggplot2::labs(title = title, y = yaxis.title, x = xaxis.title)  +
    ggplot2::scale_color_brewer(palette = "Set1", name = "") +
    ggplot2::theme(legend.position = "bottom", legend.direction = "horizontal")

  if (is.null(xaxis.title))
    p <- p + ggplot2::theme(axis.title.x = ggplot2::element_blank(),
                            axis.text.x  = ggplot2::element_blank())

  if (!is.null(wrap))
    p <- p + ggplot2::facet_wrap(stats::as.formula(paste("~", wrap)))

  p

}


#
#' @title Evolution plot area
#'
#' @description This function generates a stacked area plot to visualize the
#' temporal evolution of a variable. The function automatically adjusts the
#' X-axis scale based on the input data range and applies specific ColorBrewer
#' palettes according to the dimensions of the dataset.
#'
#' @param annual_data,new_data data frame.
#' @param title_plot character. The plot title
#' @param xaxis.title character. The title of x axis
#' @param yaxis.title character. The title of y axis
#' @param x character. The name of variable in data to represent in x axis
#' @param y character. The name of variable in data to represent in y axis
#' @param colour character. The name of variable in data to represent colour
#' @param wrap character. The name of variable in data to represent wrap
#' @return A plot
#'
#' @importFrom ggplot2 ggplot aes_string geom_area theme_bw labs
#' scale_fill_brewer theme element_blank
#'
#' @keywords internal
#'
evolution_plot_area <- function(annual_data, new_data, title_plot = NULL,
                                xaxis.title = NULL, yaxis.title, x, y,
                                colour = NULL, wrap = NULL){

  ymin <- min(annual_data$YearValue)
  ymax <- max(annual_data$YearValue)
  p <- ggplot2::ggplot(data = new_data,
                       ggplot2::aes_string(x = x, y = y, fill = colour)) +
    ggplot2::geom_area(alpha = 1.5, size = 1.2) +
    ggplot2::theme_bw(base_size = 15) +
    ggplot2::labs(title = title_plot, y = yaxis.title, x = xaxis.title) +
    ggplot2::theme(legend.position = "bottom",
                   legend.text = element_text(size = 12),
                   legend.margin = margin(t = -0.5, unit = 'cm') ) +
    ggplot2::guides(fill = guide_legend(nrow = 1, byrow = TRUE, title = "")) +
    ggplot2::scale_x_continuous(breaks = seq(ymin, ymax, by = 2))

  if (length(annual_data) == 17 | length(annual_data) == 14 ) {
    p <- p + ggplot2::scale_fill_brewer(palette = "Set1", name = "")
  } else if(length(annual_data) == 15 | length(annual_data) == 12 ) {
    p <- p + ggplot2::scale_fill_brewer(palette = "Dark2", name = "")
  }

  if (is.null(xaxis.title))
    p <- p + ggplot2::theme(axis.title.x = ggplot2::element_blank(),
                            axis.text.x  = ggplot2::element_blank())

  p

}

#
#' @title Density plot
#'
#' @description This function calculates and plots kernel density estimations
#' for normalized hydrological indices. The function normalizes selected
#' variables by a reference column, computes statistical parameters (mean and
#' standard deviation) to automatically scale the axes, and overlays density
#' lines for comparison across different data groups.
#'
#' @param df_plot data frame.
#' @param plot_index numeric. The indexes to plot
#' @param basin_name character. The name of the basin
#' @param cSD numeric. The standard deviation
#' @return No return value, called for the side effect of drawing a plot
#'
#' @importFrom graphics legend plot lines
#' @importFrom stats density sd
#'
#' @keywords internal
#'
gr_density_plot <- function(df_plot, plot_index, basin_name, cSD) {

  index1 <- length(df_plot)

  df <- df_plot[, c(plot_index)] / df_plot[, index1]
  table <- sapply(df, function(x) c("Stand dev" = sd(x),
                                    "Mean"= mean(x,na.rm = TRUE) ,
                                    "xlim"= mean(x,na.rm = TRUE) + cSD * sd(x),
                                    "ylim"= max(density(x)$y)))

  xlim <- c(0, max(table[c("xlim"), ]))
  ylim <- c(0, max(table[c("ylim"), ]))

  plot(1,
       type = "n",
       xlab = expression("t/y/" ~ km^2 ~ "" ),
       ylab = "Density",
       cex.lab = 1.5,
       las = 1,
       xlim = xlim,
       ylim = ylim,
       main = basin_name)
  colors <- c("blue","green","red")


  for (i in seq_len(length(plot_index))) {
    lines(density(df[, i]), col = colors[i])
  }

  legend("topright",
         legend = names(df),
         lty = 1,
         col = colors,
         bty = "n",
         cex = 1.5)

}

#
#' @title Map average load input by source
#'
#' @description Map showing the mean load input by source
#'
#' @param catch_data data frame. Definition of the topological sequence of
#' catchments.
#' @param annual_data data frame. Sources of nutrient for each year and
#' catchments.
#' @param sh_file sf object. The spatial information.
#' @param plot.type character. Alternatives of the map: input load (kt) by type
#' divided by year and catchment. “gr1”: by km2; “gr2”: by year/km2.
#' @param style_map character. Alternatives to create the intervals in the maps.
#' Chosen style: one of "fixed", "sd", "equal", "pretty", "quantile", "kmeans",
#' "hclust", "bclust", "fisher", "jenks".
#' @param scale_barTextS numeric. To modify the size of the text in the legend.
#' @param legend_position numeric. Legend position: 1 (default): "right",
#' "bottom"; 2: "left", "up"; 3: "right", "bottom"; 4: "right", "up".
#'
#' @return No return value, called for the side effect of drawing a plot
#'
#' @importFrom sf st_area
#' @importFrom dplyr group_by summarise across everything
#' @importFrom magrittr %>%
#' @importFrom reshape2 melt
#' @importFrom ggplot2 scale_x_continuous facet_wrap
#' @importFrom gridExtra grid.arrange
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(catch_data_TN)
#' data(annual_data_TN)
#' data(sh_file)
#' # the Input Load Map by source type 1 (lines)
#' input_maps(catch_data_TN, annual_data_TN, sh_file, plot.type = "gr1",
#' legend_position = 2)
#' # the Input Load Map by source type 2 (lines & area)
#' input_maps(catch_data_TN, annual_data_TN, sh_file, plot.type = "gr2",
#' legend_position = 2)
#' }
#'
#' @export
#'
input_maps <- function(catch_data, annual_data, sh_file, plot.type,
                       style_map = "fisher", scale_barTextS = 0.7,
                       legend_position = 1){

  sh_file$AreaSkm <- sf::st_area(sh_file) / 1000000
  sh_file$AreaSkm <- as.numeric(sh_file$AreaSkm)
  basin_area <- as.numeric(sum(sf::st_area(sh_file)) / 1000000)
  long_basin <- sqrt(basin_area)

  df_no_sf <- as.data.frame(sh_file)
  df_no_sf <- df_no_sf[, c("HydroID", "DrainAreaS", "AreaSkm" )]
  annual_data_aux <- merge(annual_data, df_no_sf, by = "HydroID")

  index1 <- length(annual_data_aux) - 5
  index2 <- length(annual_data_aux) - 7
  index3 <- length(annual_data_aux) - 6
  index4 <- length(annual_data_aux)

  switch (
    plot.type,
    "gr1" = {
      df_load_hydro <- annual_data_aux[, c(1, 3, 5:index1)] %>%
        dplyr::group_by(HydroID) %>%
        dplyr::summarise(dplyr::across(dplyr::everything(), list(mean)))

      df_load_hydro <- data.frame(df_load_hydro)

      df_load_hydro$TOTAL <- rowSums(df_load_hydro[,c(3:index2)])

      if(length(annual_data_aux) == 14){
        names(df_load_hydro)<- c("HydroID", "Year", "Bg", "Min", "Man",
                                 "Sd", "PS", "TOTAL")
      } else if(length(annual_data_aux) == 16){
        names(df_load_hydro)<- c("HydroID", "Year", "Atm", "Min", "Man",
                                 "Fix", "Soil", "Sd", "Ps", "TOTAL")
      }

      hydroSf_merge <- merge(sh_file, df_load_hydro[,c(1,3:index3)],
                             by = "HydroID")

      print(multiple_map(hydroSf_merge,
                         length(annual_data_aux),
                         long_basin,
                         unit = "t/y",
                         legend_position = legend_position))
    },
    "gr2" = {
      df_temp <- annual_data_aux[, c(1, 3, 5:index1, index4)]
      df_temp[, 3:index2] <- df_temp[, 3:index2] / df_temp[, c("AreaSkm")]

      df_load_hydroSkm <- df_temp[, 1:index2] %>%
        dplyr::group_by(HydroID) %>%
        dplyr::summarise(dplyr::across(dplyr::everything(), list(mean)))

      df_load_hydroSkm  <- data.frame(df_load_hydroSkm)
      df_load_hydroSkm$TOTAL <- rowSums(df_load_hydroSkm[, 3:index2])

      if (length(annual_data_aux) == 14){
        names(df_load_hydroSkm)<- c("HydroID", "Year", "Bg", "Min", "Man",
                                    "Sd", "PS", "TOTAL")
      } else if(length(annual_data_aux) == 16){
        names(df_load_hydroSkm)<- c("HydroID", "Year", "Atm", "Min", "Man",
                                    "Fix", "Soil", "Sd", "Ps", "TOTAL")
      }

      sf_mergeSkm <- merge(sh_file, df_load_hydroSkm[,c(1,3:index3)],
                           by = "HydroID")
      print(multiple_map(sf_mergeSkm, length(annual_data_aux), long_basin,
                         unit = "t/y/km2", legend_position = legend_position))
    },
    {
      stop("Variable plot.type should be 'gr1' or 'gr2'")
    }
  )

}

#
#' @title Plot input load by source
#'
#' @description A grouped barplot representing the average input load by source
#' for the whole basin or a three density plots showing the distribution of
#' nutrient sources (7 for nitrogen, 5 for phosphorous).
#'
#' @param annual_data data frame. Sources of nutrient for each year and
#' catchments.
#' @param sh_file sf object. The spatial information.
#' @param basin_name character. The title of the plot.
#' @param plot.type character. Possible values: Bar plot ("B") or
#' Density plot (“D”).
#' @param coef_SD numeric. The standard deviation coefficient.
#' @return No return value, called for the side effect of drawing a plot
#'
#' @importFrom sf st_area
#' @importFrom graphics par barplot
#' @import dplyr
#' @import magrittr
#'
#' @examples
#' # the data of the TN scenario
#' data(annual_data_TN)
#' data(sh_file)
#' # The name of the basin
#' basin_name <- "Lay"
#' # the barplot
#' input_plot(annual_data = annual_data_TN, basin_name = basin_name, plot.type = "B")
#' # the density plots
#' input_plot(annual_data_TN, sh_file, basin_name, "D")
#'
#' @export
#'
input_plot <- function(annual_data, sh_file, basin_name, plot.type,
                       coef_SD = 1.0){


  if (plot.type == "D") {
    sh_file$AreaSkm <- sf::st_area(sh_file) / 1000000
    sh_file$AreaSkm <- as.numeric(sh_file$AreaSkm)

    df_no_sf <- as.data.frame(sh_file)
    df_no_sf <- df_no_sf[, c("HydroID","DrainAreaS","AreaSkm" )]
    annual_data <- merge(annual_data, df_no_sf, by = "HydroID")

    oldpar <- par(no.readonly = TRUE)
    on.exit(par(oldpar))
    graphics::par(mfrow=c(1, 3))
    gr_density_plot(annual_data, c(7, 6), basin_name, coef_SD)
    indexD1 <- c(length(annual_data) - 6, length(annual_data) - 5)
    gr_density_plot(annual_data, indexD1, basin_name, coef_SD)

    if (length(annual_data) == 16) {
      gr_density_plot(annual_data, c(5, 8, 9), basin_name, coef_SD)
    }
  } else if (plot.type == "B") {

    index1 <- length(annual_data) - 3
    index2 <- length(annual_data) - 6

    df_load_tot_type_year <- annual_data[, c(2, 5:index1)] %>%
      dplyr::group_by(YearValue) %>%
      dplyr::summarise(dplyr::across(dplyr::everything(), list(sum)))

    df_load_tot_type_year <- data.frame(df_load_tot_type_year)

    LoadAvg <- colMeans(df_load_tot_type_year[, c(2:index2)])

    if (length(annual_data) == 12) {
      names(LoadAvg)<- c("Bg","Min","Man","Sd","PS")
    } else if (length(annual_data) == 14) {
      names(LoadAvg)<- c("Atm","Min","Man","Fix","Soil","Sd","PS")
    }

    oldpar <- par(no.readonly = TRUE)
    on.exit(par(oldpar))
    graphics::par(mfrow=c(1, 1))
    graphics::barplot(LoadAvg / 1000,
                      col = grDevices::rainbow(7),
                      border = "white",
                      font.axis = 2,
                      beside = TRUE,
                      legend = rownames(LoadAvg),
                      xlab = "Inputs",
                      ylab = " Mean Anual Input by Source (kt/y)",
                      font.lab = 2,
                      las = 1)
  } else {
    stop("Variable plot.type must be 'D' for density plot or 'B' for barplot.")
  }

}

#
#' @title Time series of annual load inputs by source
#'
#' @description Creates a time series plot showing basin inputs by
#' source
#'
#' @param catch_data data frame. Definition of the topological sequence of
#' catchments.
#' @param annual_data data frame. Sources of nutrient for each year and
#' catchments.
#' @param sh_file sf object. The spatial information.
#' @param basin_name character. The title of the plot
#' @param plot.type character. Alternative of the plot: “gr1”: stacked area;
#' “gr2”: lines & area.
#' @return A time-series plot
#'
#' @importFrom sf st_area
#' @importFrom stats quantile
#' @importFrom dplyr group_by summarise across select everything
#' @importFrom magrittr %>%
#' @importFrom reshape2 melt
#' @importFrom ggplot2 scale_x_continuous facet_wrap
#' @importFrom gridExtra grid.arrange
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(catch_data_TN)
#' data(annual_data_TN)
#' data(sh_file)
#' # The title of the plot
#' plotTitle <- "Time series for the Lay Basin"
#' # the time serie plot 1 (lines)
#' input_Tserie(catch_data_TN, annual_data_TN, sh_file, plotTitle, "gr1")
#' # the time serie plot 2 (lines & area)
#' input_Tserie(catch_data_TN, annual_data_TN, sh_file, plotTitle, "gr2")
#' }
#'
#' @export
input_Tserie <- function(catch_data, annual_data, sh_file, basin_name,
                         plot.type){

  # 1 Tg (teragramos): 10^12 gr
  # 1 Tg (teragramos): 10^6 t (Toneladas) :  10^9 kg (kilo gramos):
  # 1 Tg (teragramos): 10^3 kt (miles de Toneladas):
  #str(annual_data)

  index1 <- length(annual_data) - 3
  index2 <- length(annual_data) - 6
  index3 <- length(annual_data)
  index4 <- index2 + 1

  DF_LoadTot_Type_Year <- annual_data[, c(2, 5:index1)] %>%
    dplyr::group_by(YearValue) %>%
    dplyr::summarise(dplyr::across(dplyr::everything(), list(sum)))  # error codigo original era mean

  DF_LoadTot_Type_Year <- data.frame(DF_LoadTot_Type_Year)

  if(length(annual_data)==12){
    names(DF_LoadTot_Type_Year)<- c("Year","Bg","Min","Man","Sd","PS")
  }else if(length(annual_data)==14){
    names(DF_LoadTot_Type_Year)<- c("Year","Atm","Min","Man","Fix","Soil","Sd","Ps")
  }

  ymin <- min(DF_LoadTot_Type_Year$Year)
  ymax <- max(DF_LoadTot_Type_Year$Year)

  DF_LoadTot_Type_Year_melt <- reshape2::melt(DF_LoadTot_Type_Year,
                                              id.vars = "Year")

  # para que mantenga el orden de los colores
  if(length(annual_data)==12){
    new_order <- c("Min","Man","Bg","Sd","PS")
    DF_LoadTot_Type_Year_melt$variable <- factor(DF_LoadTot_Type_Year_melt$variable, levels = new_order)
  }else if(length(annual_data)==14){
    new_order <- c("Min","Man","Atm","Fix","Soil","Sd","Ps")
    DF_LoadTot_Type_Year_melt$variable <- factor(DF_LoadTot_Type_Year_melt$variable, levels = new_order)
  }

  #  sum(DF_LoadTot_Type_Year_melt[DF_LoadTot_Type_Year_melt$Year == 1990, ]$value)

  DF_LoadTot_Type_Year_melt$value = DF_LoadTot_Type_Year_melt$value / 1000000    #convierto ton en Tg


  switch (
    plot.type,
    "gr1" = {

      p1 <- evolution_plot_area(annual_data = annual_data,
                                new_data = DF_LoadTot_Type_Year_melt,
                                title_plot = paste0(basin_name," Input Time Series"),
                                yaxis.title = "Input by Source (Tg/y)",
                                xaxis.title = "Year",
                                x = "Year",
                                y = "value",
                                colour = "variable")
      print(p1)
    },
    "gr2" = {

      p1 <-  evolution_plot(data = DF_LoadTot_Type_Year_melt,
                            yaxis.title = "Input by Source (Tg/y)",
                            xaxis.title = "Year", x = "Year", y = "value",
                            colour = "variable")
      p1 <- p1 +
        ggplot2::theme(legend.position = "bottom", legend.text = element_text(size = 12),
                       legend.margin = margin(t = -0.5, unit = 'cm') )+
        ggplot2::guides(colour = guide_legend(nrow = 1, byrow = TRUE, title = "")) +
        ggplot2::scale_x_continuous(breaks = seq(ymin, ymax, by = 2))
      # print(p1)
      #  ggplot2::scale_x_continuous(breaks = seq(ymin, ymax, by = 2))


      DF_LoadTot_Type_Year$Total <- rowSums(DF_LoadTot_Type_Year[,2:index2] )   # anaddo columna con el total
      # DF_LoadTot_Type_Year$Total <- DF_LoadTot_Type_Year$Total / 1000
      DF_LoadTot_Type_Year_aux <- DF_LoadTot_Type_Year %>% dplyr::select(Year, Total)

      DF_LoadTot_Type_Year_aux$Total <- DF_LoadTot_Type_Year_aux$Total/ 1000000

      p2 <-  evolution_plot(data = DF_LoadTot_Type_Year_aux,
                            title = paste0(basin_name," Input Time Series"),
                            yaxis.title = "Total Input (Tg/y)",
                            xaxis.title = "Year", x = "Year", y = "Total")
      p2 <- p2 +
        ggplot2::scale_x_continuous(breaks = seq(ymin, ymax, by = 2))

      print(gridExtra::grid.arrange(p2, p1, nrow = 2))
    },
    {
      stop("Variable plot.type should be 'gr1', 'gr2' ")
    }

  )


  return(DF_LoadTot_Type_Year)

}

#
#' @title Time series of annual load inputs by source and km2
#'
#' @description Creates a time series plot showing basin inputs by
#' source
#'
#' @param catch_data data frame. Definition of the topological sequence of
#' catchments.
#' @param annual_data data frame. Sources of nutrient for each year and
#' catchments.
#' @param sh_file sf object. The spatial information.
#' @param basin_name character. The title of the plot
#' @param plot.type character. Alternative of the plot:  “gr1”:  stacked area by km2; “gr2”  lines & area by km2 and Shreve.
#' @return A time-series plot
#'
#' @importFrom sf st_area
#' @importFrom stats quantile
#' @importFrom dplyr group_by summarise across select everything
#' @importFrom magrittr %>%
#' @importFrom reshape2 melt
#' @importFrom ggplot2 scale_x_continuous facet_wrap
#' @importFrom gridExtra grid.arrange
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(catch_data_TN)
#' data(annual_data_TN)
#' data(sh_file)
#' # The title of the plot
#' plotTitle <- "Time series for the Lay Basin"
#' # the time serie plot 1 (by km2)
#' input_Tserie(catch_data_TN, annual_data_TN, sh_file, plotTitle, "gr1")
#' # the time serie plot 2 (by km2 and Shreve)
#' input_Tserie(catch_data_TN, annual_data_TN, sh_file, plotTitle, "gr2")
#' }
#' # catch_data <- The_Scen[[1]]
#' # annual_data <- The_Scen[[2]]
#' # sh_file <- The_Sf_shape
#' @export
input_Tserie_area <- function(catch_data, annual_data, sh_file, basin_name,
                              plot.type){

  sh_file$AreaSkm <- sf::st_area(sh_file) / 1000000  #conversion km2
  sh_file$AreaSkm <- as.numeric(sh_file$AreaSkm)
  basin_area <- as.numeric(sum(sf::st_area(sh_file)) / 1000000)

  df_no_sf <- as.data.frame(sh_file)
  df_no_sf <- df_no_sf[, c("HydroID", "DrainAreaS", "AreaSkm")]

  annual_data <- merge(annual_data, df_no_sf, by = "HydroID")

  theQuantile <- stats::quantile(catch_data$Shreve)
  ShrLevel1 <- catch_data[catch_data$Shreve <= theQuantile[3], ]$HydroID
  ShrLevel2 <- catch_data[catch_data$Shreve > theQuantile[3] &
                            catch_data$Shreve <= theQuantile[4], ]$HydroID
  ShrLevel3 <- catch_data[catch_data$Shreve > theQuantile[4] &
                            catch_data$Shreve <= theQuantile[5], ]$HydroID

  annual_data$shrLevel <- NA
  annual_data[annual_data$HydroID %in% ShrLevel1, ]$shrLevel <- "level1"
  annual_data[annual_data$HydroID %in% ShrLevel2, ]$shrLevel <- "level2"
  annual_data[annual_data$HydroID %in% ShrLevel3, ]$shrLevel <- "level3"

  sh_sr1 <- sh_file[sh_file$HydroID %in% ShrLevel1, ]
  sh_sr2 <- sh_file[sh_file$HydroID %in% ShrLevel2, ]
  sh_sr3 <- sh_file[sh_file$HydroID %in% ShrLevel3, ]

  sh_sr1Area <- as.numeric(sum(sf::st_area(sh_sr1)) / 1000000)
  sh_sr2Area <- as.numeric(sum(sf::st_area(sh_sr2)) / 1000000)
  sh_sr3Area <- as.numeric(sum(sf::st_area(sh_sr3)) / 1000000)

  index1 <- length(annual_data) - 6
  index2 <- length(annual_data) - 9
  index3 <- length(annual_data)
  index4 <- index2 + 1

  df_load_tot_type_year <- annual_data[, c(3, 5:index1)] %>%
    dplyr::group_by(YearValue) %>%
    dplyr::summarise(dplyr::across(dplyr::everything(), list(sum)))

  df_load_tot_type_year <- data.frame(df_load_tot_type_year)

  if (length(annual_data) == 15) {
    names(df_load_tot_type_year)<- c("Year", "Bg", "Min", "Man", "Sd", "PS")
  } else if (length(annual_data) == 17) {
    names(df_load_tot_type_year)<- c("Year", "Atm", "Min", "Man", "Fix", "Soil",
                                     "Sd", "Ps")
  }

  ymin <- min(df_load_tot_type_year$Year)
  ymax <- max(df_load_tot_type_year$Year)

  switch (
    plot.type,
    # LAS sos SIGUIENTES NECESITAN SHAPE FILE
    "gr1" = {
      df_load_km2_type_year <- df_load_tot_type_year
      df_load_km2_type_year[, 2:index2] <-
        df_load_km2_type_year[, 2:index2] / basin_area

      df_load_avg_type_year_melt <- reshape2::melt(df_load_km2_type_year,
                                                   id.vars = "Year")

      # para que mantenga el orden de los colores
      if(length(annual_data)==15){
        new_order <- c("Min","Man","Bg","Sd","PS")
        df_load_avg_type_year_melt$variable <- factor(df_load_avg_type_year_melt$variable, levels = new_order)
      }else if(length(annual_data)==17){
        new_order <- c("Min","Man","Atm","Fix","Soil","Sd","Ps")
        df_load_avg_type_year_melt$variable <- factor(df_load_avg_type_year_melt$variable, levels = new_order)
      }


      # en esta funcion el primer parametro es solo para saber si TP o TN y los anos
      p1 <- evolution_plot_area(annual_data = annual_data,
                                new_data = df_load_avg_type_year_melt,
                                title_plot = paste0(basin_name,
                                               " Annual Input Time Series"),
                                yaxis.title = "Input by source (t/y/km2)",
                                xaxis.title = "Year",
                                x = "Year",
                                y = "value",
                                colour = "variable") +
        ggplot2::scale_x_continuous(breaks = seq(ymin, ymax, by = 2))

      print(p1)
    },
    "gr2" = {
      df_load_km2_type_shr_year <- annual_data[, c(3, index3, 5:index1)] %>%
        dplyr::group_by(YearValue, shrLevel) %>%
        dplyr::summarise(dplyr::across(dplyr::everything(), list(sum)),
                         .groups = "rowwise")

      df_load_km2_type_shr_year <- data.frame(df_load_km2_type_shr_year)

      if (length(annual_data) == 15){
        names(df_load_km2_type_shr_year) <- c("Year", "shrLevel", "Bg", "Min",
                                              "Man", "Sd", "PS")
      } else if (length(annual_data) == 17) {
        names(df_load_km2_type_shr_year) <- c("Year", "shrLevel", "Atm", "Min",
                                              "Man", "Fix", "Soil", "Sd", "PS")
      }

      df_load_km2_type_shr_year[
        df_load_km2_type_shr_year$shrLevel == "level1", c(3:index4)] <-
        df_load_km2_type_shr_year[
          df_load_km2_type_shr_year$shrLevel == "level1",
          c(3:index4)] / sh_sr1Area

      df_load_km2_type_shr_year[
        df_load_km2_type_shr_year$shrLevel == "level2", c(3:index4)] <-
        df_load_km2_type_shr_year[
          df_load_km2_type_shr_year$shrLevel == "level2", c
          (3:index4)] / sh_sr2Area

      df_load_km2_type_shr_year[
        df_load_km2_type_shr_year$shrLevel == "level3", c(3:index4)] <-
        df_load_km2_type_shr_year[
          df_load_km2_type_shr_year$shrLevel == "level3",
          c(3:index4)] / sh_sr3Area

      df_load_avg_type_shr_year_melt <-
        reshape2::melt(df_load_km2_type_shr_year,
                       id.vars = c("Year","shrLevel"))


      # para que mantenga el orden de los colores
      if(length(annual_data)==15){
        new_order <- c("Min","Man","Bg","Sd","PS")
        df_load_avg_type_shr_year_melt$variable <- factor(df_load_avg_type_shr_year_melt$variable, levels = new_order)
      }else if(length(annual_data)==17){
        new_order <- c("Min","Man","Atm","Fix","Soil","Sd","Ps")
        df_load_avg_type_shr_year_melt$variable <- factor(df_load_avg_type_shr_year_melt$variable, levels = new_order)
      }


      df_load_avg_type_shr_year_melt$etiquetas <-
        factor(df_load_avg_type_shr_year_melt$shrLevel,
               labels = c(paste0("Upper Part= " ,  floor(sh_sr1Area)," km2"),
                          paste0("Middle Part= ", floor(sh_sr2Area)," km2"),
                          paste0("Lower Part= " ,  floor(sh_sr3Area)," km2") ))

      p1 <- evolution_plot(data = df_load_avg_type_shr_year_melt,
                           title = paste0(basin_name," Time Series of Loads"),
                           yaxis.title = "Input by source (t/y/km2)",
                           xaxis.title = "Year",
                           x = "Year",
                           y = "value",
                           colour = "variable") +
        ggplot2::theme(legend.position = "bottom",
                       legend.text = element_text(size = 12),
                       legend.margin = margin(t = -0.5, unit = 'cm') ) +
        ggplot2::guides(colour = guide_legend(nrow = 1,
                                              byrow = TRUE,
                                              title = "")) +
        ggplot2::facet_wrap(~ etiquetas)
      print(p1)
    },
    {
      stop("Variable plot.type should be 'gr1', 'gr2'")
    }

  )

}

#'
#' @title Lake retention values summary
#'
#' @description Summary of the reference values in the stations
#'
#' @param catch_data_TN data frame. Sources of nutrient for each year and
#' catchments.
#'
#' @return barplot & histogram-density
#'
#' @importFrom graphics hist
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(catch_data_TN)
#' LakeRetent_plot(catch_data_TN)
#' }
#'
#' @export
#'
LakeRetent_plot <- function(catch_data_TN) {

  # Calcular porcentaje de valores nulos
  porcentaje_nulos <- mean(catch_data_TN$LakeFrRet == 0) * 100

  par(mfrow = c(1,2))
  # Crear histograma con densidad
  graphics::hist(catch_data_TN$LakeFrRet, probability = TRUE, breaks = 30,
       main = "Lake Retention Fraction",
       sub = paste("Percentage of null values: ", round(porcentaje_nulos, 2), "%"),
       xlab = "Values", col = "lightblue")

  # Superponer curva de densidad
  densidad <- density(catch_data_TN$LakeFrRet)
  lines(densidad, col = "darkblue", lwd = 2)

  # sin considerar los nulos
  densidad <-density(catch_data_TN$LakeFrRet[catch_data_TN$LakeFrRet!=0])
  plot(densidad, main = "Lake Retention (only no null)",col = "lightblue", lwd = 2)


}

#
#' @title Load DA map
#'
#' @description This function creates a tmap
#'
#' @param hydroSf data frame.
#' @param long_basin numeric.
#' @param title string. The title of the map
#' @param style_map character. The style of the map
#' @param legend_position numeric. It indicates de position of the legend of the
#' plot. (Default: 1)
#' @return A plot with map of the basin
#'
#' @importFrom tmap tm_shape tm_fill tm_layout tm_scalebar
#'
#' @keywords internal
#'
load_map <- function(hydroSf, long_basin, title, style_map,
                     legend_position = 1) {

  scale_barRef <- c(0, floor(long_basin / 8), floor(long_basin / 4))
  scale_barTextS <- 0.7
  palet <-"YlOrRd"

  print(
    create_map(hydroSf, "CatchLoad", style_map, scale_barRef,
               scale_barTextS, title, palet, legend_position = legend_position))
}

#
#' @title Load SA map
#'
#' @description This function creates a tmap
#'
#' @param hydroSf_merge data frame.
#' @param refN_P numeric. The number of variables of the data frame
#' @param long_basin numeric.
#' @param unity character. The units of hydroSf
#' @param style_map character. The style of the map
#' @param legend_position numeric. It indicates de position of the legend of the
#' plot. (Default: 1)
#' @return A plot with map of the basin
#'
#' @importFrom tmap tm_shape tm_fill tm_layout tm_scalebar
#'
#' @keywords internal
#'
load_SA_map <- function(hydroSf_merge, refN_P, long_basin, unity, style_map,
                        legend_position = 1) {

  c_man <- c("Man_1", "Manure", "YlOrRd")
  c_min <- c("Min_1", "Mineral", "YlOrRd")
  c_sd <- c("Sd_1", "Sc.Dwellings", "YlOrRd")
  c_ps <- c("Ps_1", "PointS", "YlOrRd")
  c_tot <- c("CatchLoad_1", "CatchLoad", "-viridis")

  if(refN_P == 24) {
    c_atm <- c("Atm_1", "Atmospheric", "YlOrRd")

    c_soil <- c("Soil_1", "Soil", "YlOrRd")

    c_fix <- c("Fix_1", "Fix", "YlOrRd")

    param_list <- list(c_man, c_min, c_ps, c_sd, c_fix, c_atm, c_soil, c_tot)
    map_list <- create_lits_of_maps(hydroSf_merge, long_basin, unity,
                                    legend_position, param_list, style_map)
  } else if(refN_P == 22) {
    c_bg <- c("Bg_1", "Background", "YlOrRd")

    param_list <- list(c_man, c_min, c_ps, c_sd, c_bg, c_tot)
    map_list <- create_lits_of_maps(hydroSf_merge, long_basin, unity,
                                    legend_position, param_list, style_map)

  } else {
    stop("The number of columns of the data frame does not match.")
  }

  print(
    tmap::tmap_arrange(map_list)
  )

}

#' @title Multiple Map
#'
#' @description This function creates an arrange of tmap
#'
#' @param hydroSf data frame.
#' @param refN_P numeric. The number of variables of the data frame
#' @param long_basin numeric.
#' @param unit character. The units of hydroSf
#' @param legend_position numeric. It indicates de position of the legend of the
#' plot. (Default: 1)
#'
#' @importFrom tmap tmap_arrange
#'
#' @return A plot with map of the basin
#'
#' @keywords internal
#'
multiple_map <- function(hydroSf, refN_P, long_basin, unit,
                         legend_position = 1){

  style_map <- "fisher"

  c_man <- c("Man", "Manure", "YlOrRd")
  c_min <- c("Min", "Mineral", "YlOrRd")
  c_sd <- c("Sd", "Sc.Dwellings", "YlOrRd")
  c_tot <- c("TOTAL", "TOT.Diff.", "-viridis")

  if (refN_P == 16) {
    c_ps <- c("Ps", "PointS", "-magma")
    c_fix <- c("Fix", "Fix", "YlOrRd")
    c_atm <- c("Atm", "Atmospheric", "YlOrRd")
    c_soil <- c("Soil", "Soil", "YlOrRd")

    param_list <- list(c_man, c_min, c_fix, c_sd, c_atm, c_soil, c_tot, c_ps)
    map_list <- create_lits_of_maps(hydroSf, long_basin, unit, legend_position,
                                    param_list, style_map)
    mmap <- tmap::tmap_arrange(map_list)
  } else if (refN_P == 14) {
    c_ps <- c("PS", "PointS", "-magma")
    c_bg <- c("Bg", "BackGr.", "YlOrRd")
    param_list <- list(c_man, c_min, c_sd, c_bg, c_tot, c_ps)
    map_list <- create_lits_of_maps(hydroSf, long_basin, unit, legend_position,
                                    param_list, style_map)
    mmap <- tmap::tmap_arrange(map_list)
  }

  mmap

}

#
#' @title Map average load output by source
#'
#' @description Creates maps showing basin output total or by source loads
#'
#' @param green_file data frame of GREEN model results from green_shares()
#' function. Nutrient Load by source apportionment of nutrient for each year
#' and catchments.
#' @param sh_file sf object. The spatial information of the basin.
#' @param plot.type character. Alternatives of the map: “gr1”: output load
#' (kt/y) by source; “gr2”: Total Load, log10 (kt/y); “gr3”: Total Load
#' by km2 (kt/year/km2).
#' @param style character. The style of the plot.
#' @param legend_position numeric. Legend position: 1 (default): "right",
#' "bottom"; 2: "left", "up"; 3: "right", "bottom"; 4: "right", "up".
#' @return No return value, called for the side effect of drawing a plot
#'
#' @importFrom sf st_area
#' @importFrom reshape2 melt
#' @importFrom ggplot2 ggplot aes geom_line theme_bw labs scale_color_brewer
#' facet_wrap theme guides guide_legend element_text margin element_blank
#' @importFrom gridExtra grid.arrange
#' @importFrom dplyr group_by summarise across everything
#' @importFrom magrittr %>%
#' @importFrom tmap tmap_arrange
#' @importFrom graphics par barplot
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(catch_data_TN)
#' data(annual_data_TN)
#' data(sh_file)
#' # the parameter to assess the basin model
#' alpha_p <- 35.09
#' alpha_l <- 0.02
#' sd_coef <- 0.2
#' # years in which the model should be executed
#' loc_years <- 1990:2018
#' # Computing the source apportionment
#' basin_sa <- green_shares(catch_data_TN, annual_data_TN, alpha_p, alpha_l,
#' sd_coef, loc_years)
#' # Basin Output Load  Maps by source
#' Lpos <- 1
#' nutrient_maps(basin_sa, sh_file, plot.type = "gr1", style = "log10", legend_position = Lpos)
#' # Basin Output Specific Load  Maps
#' Lpos <- 1
#' nutrient_maps(basin_sa, sh_file, plot.type = "gr2", style = "log10", legend_position = Lpos)
#' # Basin Output Specific Load by km2 Maps
#' Lpos <- 1
#' nutrient_maps(basin_sa, sh_file, plot.type = "gr3", style = "fisher", legend_position = Lpos)
#' }
#'
#' @export
#'
nutrient_maps <- function(green_file, sh_file, plot.type, style,
                          legend_position = 1) {

  sh_file$AreaSkm <- sf::st_area(sh_file) / 1000000
  sh_file$AreaSkm <- as.numeric(sh_file$AreaSkm)

  basin_area <- as.numeric(sum(sf::st_area(sh_file) ) / 1000000)
  long_basin <- sqrt(basin_area)

  df_no_sf <- as.data.frame(sh_file)
  df_no_sf <- df_no_sf[, c("HydroID", "NextDownID", "Shreve", "DrainAreaS",
                           "AreaSkm")]

  result <- create_levels(green_file, df_no_sf)
  green_file <- result$df

  index1 <- length(green_file) - 5
  index2 <- length(green_file) - 2

  switch(plot.type,
         "gr1" = {
           df_load_SA_hydro <- green_file[, c(1, 2, 4:index1)] %>%
             dplyr::group_by(HydroID) %>%
             dplyr::summarise(dplyr::across(dplyr::everything(), list(mean)))
           df_load_SA_hydro[, 3:(index1 - 1)] <- df_load_SA_hydro[, 3:(index1 - 1)] + 0.1

           hydroSf_SA_merge <- merge(sh_file, df_load_SA_hydro[, c(1, 3:(index1 - 1))],
                                     by = "HydroID")

           numvar <- length(hydroSf_SA_merge)

           load_SA_map(hydroSf_merge = hydroSf_SA_merge, refN_P = numvar,
                       long_basin = long_basin, unity = "(t/y)", style,
                       legend_position = legend_position)
         },
         "gr2" = {
           df_load_hydro <- green_file[, c(1, 2, index1)] %>%
             dplyr::group_by(HydroID) %>%
             dplyr::summarise(dplyr::across(dplyr::everything(), list(mean)))

           df_load_hydro  <- data.frame(df_load_hydro)
           names(df_load_hydro) <- c("HydroID", "Year", "CatchLoad")

           hydroSf_merge <- merge(sh_file, df_load_hydro[, c("HydroID", "CatchLoad")],
                                  by = "HydroID")

           plot_title <- "Total load (t/y)"
           load_map(hydroSf_merge, long_basin, plot_title, style,
                    legend_position = legend_position)
         },
         "gr3" = {
           df_load_hydro_km <- green_file[, c(1, 2, index1, index2)] %>%
             dplyr::group_by(HydroID, DrainAreaS) %>%
             dplyr::summarise(dplyr::across(dplyr::everything(), list(mean)),
                              .groups = "rowwise")

           df_load_hydro_DA  <- data.frame(df_load_hydro_km)
           names(df_load_hydro_DA) <- c("HydroID", "DrainAreaS", "Year", "CatchLoad")
           df_load_hydro_DA$CatchLoad <- df_load_hydro_DA$CatchLoad /
             df_load_hydro_DA$DrainAreaS

           hydroSf_mergeDA <- merge(sh_file,
                                    df_load_hydro_DA[, c("HydroID", "CatchLoad")],
                                    by = "HydroID")

           plot_title <- "Specific load (t/km2/y)"
           load_map(hydroSf_mergeDA, long_basin, plot_title, style,
                    legend_position = legend_position)
         },
         {
           stop("Variable plot.type should be 'gr1','gr2' or 'gr3'")
         }
  )

}

#
#' @title Output load time series plot
#'
#' @description Creates a time series plot showing basin model results
#'
#' @param green_file data frame. Nutrient Load by source apportionment of
#' nutrient for each year and catchments.
#' @param basin_name character. The title of the plot.
#' @param plot.type character. Alternative of the plot: output load (t) by
#' source; gr1: Basin average by Shreve (t/y/km2); gr2: Outlet total (kt/y).
#' @param file_path character. The path to save the csv.
#' @return No return value, called for the side effect of drawing a plot
#'
#' @importFrom sf st_area
#' @importFrom reshape2 melt
#' @importFrom ggplot2 ggplot aes geom_line theme_bw labs scale_color_brewer
#' facet_wrap theme guides guide_legend element_text margin element_blank
#' @importFrom gridExtra grid.arrange
#' @importFrom dplyr group_by summarise across everything
#' @importFrom magrittr %>%
#' @importFrom tmap tmap_arrange
#' @importFrom graphics par barplot
#' @importFrom utils write.csv
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(catch_data_TN)
#' data(annual_data_TN)
#' data(sh_file)
#' # the parameter to assess the basin model
#' alpha_p <- 35.09
#' alpha_l <- 0.02
#' sd_coef <- 0.2
#' # years in which the model should be executed
#' loc_years <- 1990:2018
#' # Computing the source apportionment
#' basin_sa <- green_shares(catch_data_TN, annual_data_TN, alpha_p, alpha_l,
#' sd_coef, loc_years)
#' # The title of the plot
#' plotTitle <- "Time series Load Output for the Lay Basin"
#' # Output Load Basin average time series (lines)
#' nutrient_tserie(basin_sa, basin_name = plotTitle, plot.type = "gr1")
#' # Total Load in the Basin Outlet time series (lines)
#' nutrient_tserie(basin_sa, basin_name = plotTitle, plot.type = "gr2")
#' }
#'
#' @export
#'
nutrient_tserie <- function(green_file, basin_name, plot.type, file_path = NULL) {

  ymin<- min(green_file$Year)
  ymax <- max(green_file$Year)

  greenFile_OUT <- green_file %>%
    filter(To_catch == -1) %>%
    group_by(Year) %>%
    summarize_all(sum, na.rm = TRUE)
  greenFile_OUT <- as.data.frame(greenFile_OUT)
  greenFile_OUT$HydroID <- NULL
  greenFile_OUT$To_catch <- NULL

  if (!is.null(file_path)) {
    utils::write.csv(greenFile_OUT, file = file_path, row.names = FALSE)
  }

  switch (plot.type,
          "gr1" = {
            print(
              ggplot2::ggplot(data = greenFile_OUT,
                              ggplot2::aes(x = Year, y = CatchLoad/1000000)) +
                ggplot2::geom_line(alpha = 1.5, size = 1.2) +
                ggplot2::theme_bw(base_size = 15) +
                ggplot2::labs(title = paste0(basin_name," Outlet Load time serie "),
                              y = "Outlet annual load (Tg/y)",
                              x = "Year")  +
                ggplot2::scale_color_brewer(palette = "Set1", name = "")
            )
          },
          "gr2" = {

            if(length(greenFile_OUT)==9){
              colores <- "Set1"      # Para N esta paleta
              greenFileOut_melt <- reshape2::melt(greenFile_OUT[, c(1:8)],
                                                  id.vars = c( "Year"))
              # Change the order of levels
              new_order <- c("Min","Man","Atm","Fix","Soil","Sd","Ps")
              greenFileOut_melt$variable <- factor(greenFileOut_melt$variable, levels = new_order)
            }
            if(length(greenFile_OUT)==7){
              colores <- "Dark2"  # para P esta paleta
              greenFileOut_melt <- reshape2::melt(greenFile_OUT[, c(1:6)],
                                                  id.vars = c( "Year"))
              # Change the order of levels
              new_order <- c("Min","Man","Bg","Sd","Ps")
              greenFileOut_melt$variable <- factor(greenFileOut_melt$variable, levels = new_order)
            }

            print(
              ggplot2::ggplot(data = greenFileOut_melt,
                              ggplot2::aes(x = Year, y = value /1000000, fill = variable)) +
                ggplot2::geom_area(alpha = 1.5,size = 1.2) +
                ggplot2::theme_bw(base_size = 15  ) +
                ggplot2::labs(title = basin_name,
                              y = "Outlet annual load with source contribution (Tg/y)",
                              x = "Year")  +
                ggplot2::scale_fill_brewer(palette = colores) +
                ggplot2::theme(legend.position = "bottom",
                               legend.text = element_text(size = 12),
                               legend.margin = margin(t = -0.5, unit = 'cm') ) +
                ggplot2::guides(fill = guide_legend(nrow = 1, byrow = TRUE, title = "")) +
                ggplot2::scale_x_continuous(breaks = seq(ymin, ymax, by = 2))
            )
          },
          {
            stop("Variable plot.type should be 'gr1','gr2' ")
          }
  )

  return(greenFile_OUT)

}


#
#' @title Output load time series plot
#'
#' @description Creates a time series plot showing basin model results
#'
#' @param green_file data frame. Nutrient Load by source apportionment of
#' nutrient for each year and catchments.
#' @param sh_file sf object. The spatial information.
#' @param basin_name character. The title of the plot.
#' @return No return value, called for the side effect of drawing a plot
#'
#' @importFrom sf st_area
#' @importFrom reshape2 melt
#' @importFrom ggplot2 ggplot aes geom_line theme_bw labs scale_color_brewer
#' facet_wrap theme guides guide_legend element_text margin element_blank
#' @importFrom gridExtra grid.arrange
#' @importFrom dplyr group_by summarise across everything
#' @importFrom magrittr %>%
#' @importFrom tmap tmap_arrange
#' @importFrom graphics par barplot
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(catch_data_TN)
#' data(annual_data_TN)
#' data(sh_file)
#' # the parameter to assess the basin model
#' alpha_p <- 35.09
#' alpha_l <- 0.02
#' sd_coef <- 0.2
#' # years in which the model should be executed
#' loc_years <- 1990:2018
#' # Computing the source apportionment
#' basin_sa <- green_shares(catch_data_TN, annual_data_TN, alpha_p, alpha_l,
#' sd_coef, loc_years)
#' basin_name <- "Visla Basin"
#' nutrient_tserie_darea(basin_sa, sh_file, basin_name)
#' }
#'
#' @export
#'
nutrient_tserie_darea <- function(green_file, sh_file, basin_name){

  sh_file$AreaSkm <- sf::st_area(sh_file) / 1000000
  sh_file$AreaSkm <- as.numeric(sh_file$AreaSkm)

  df_no_sf <- as.data.frame(sh_file)
  df_no_sf <- df_no_sf[, c("HydroID", "Shreve", "DrainAreaS",
                           "AreaSkm")]

  result <- create_levels(green_file, df_no_sf)
  green_file <- result$df
  ShrLevel1 <- result$ShrLevel1
  ShrLevel2 <- result$ShrLevel2
  ShrLevel3 <- result$ShrLevel3

  sh_sr1DrAr <- sum(green_file[green_file$HydroID %in% ShrLevel1,
                               c("DrainAreaS")])
  sh_sr2DrAr <- sum(green_file[green_file$HydroID %in% ShrLevel2,
                               c("DrainAreaS")])
  sh_sr3DrAr <- sum(green_file[green_file$HydroID %in% ShrLevel3,
                               c("DrainAreaS")])

  if (length(green_file) == 15) {
    colores <- "Set1"
  } else if (length(green_file) == 13) {
    colores <- "Dark2"
  }

  df_load_avg_type_shr_year <- green_file[, c("Year","DrainAreaS","shrLevel","CatchLoad" )] %>%
    dplyr::group_by(Year,shrLevel) %>%
    dplyr::summarise(dplyr::across(dplyr::everything(), list(mean)),
                     .groups = "rowwise")

  df_load_avg_type_shr_year <- data.frame(df_load_avg_type_shr_year)
  df_load_avg_type_shr_year$LoadByDA <- df_load_avg_type_shr_year$CatchLoad_1 /
    df_load_avg_type_shr_year$DrainAreaS_1

  df_load_avg_type_shr_year_melt <- reshape2::melt(
    df_load_avg_type_shr_year[, c("Year","shrLevel","LoadByDA")], id.vars = c("Year","shrLevel"))
  df_load_avg_type_shr_year_melt$etiquetas <-
    factor(df_load_avg_type_shr_year_melt$shrLevel,
           labels = c(paste("DrainArea=", floor(sh_sr1DrAr)),
                      paste("DrainArea=", floor(sh_sr2DrAr)),
                      paste("DrainArea=", floor(sh_sr3DrAr))))


  print(
    ggplot2::ggplot(data = df_load_avg_type_shr_year_melt,
                    ggplot2::aes(x = Year, y = value)) +
      ggplot2::geom_line(alpha = 1.5,size = 1.2) +
      ggplot2::theme_bw(base_size = 15) +
      ggplot2::labs(title =
                      paste0(basin_name,
                             " Load time series by shreve level "),
                    y = expression("Mean annual specific load (t/y/" ~
                                     km^2 ~ ")" ) ,
                    x = "Year")  +
      ggplot2::scale_color_brewer(palette = colores, name = "") +
      ggplot2::facet_wrap(~ etiquetas)
  )

}

#'
#' @title Nutrient balance flow plot
#'
#' @description Nutrient balance flow in Sankey plot
#'
#' @param Nbalance_out data frame. Nutrient balance result from the Nutbalance()
#' function
#'
#' @return A Sankey diagram and a data frame with the some variable values
#'
#' @importFrom networkD3 sankeyNetwork JS
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(catch_data_TN)
#' data(annual_data_TN)
#' # the parameter to assess the basin model
#' alpha_p <- 35.09
#' alpha_l <- 0.02
#' sd_coef <- 0.2
#' # years in which the model should be executed
#' loc_years <- 1990:2018
#' # Computing the nutrient balance
#' nut_bal <- region_nut_balance(catch_data_TN, annual_data_TN, alpha_p, alpha_l,
#' sd_coef, loc_years)
#' # Plot the sankey plot with the result of the balance
#' sank <- N4_sankey(nut_bal)
#' }
#'
#' @export
#'
N4_sankey <- function(Nbalance_out) {

  data <- as.data.frame(Nbalance_out)
  n_years <- length(levels(as.factor(data$Year)))

  PS <- sum(data$PS) / n_years
  min <- sum(data$Min) / n_years
  man <- sum(data$Man) / n_years
  fix <- sum(data$Fix) / n_years
  soil <- sum(data$Soil) / n_years
  atm <- sum(data$Atm) / n_years
  sd <- sum(data$Sd) / n_years
  diffuse_agri <- sum(data$DiffuseAgri) / n_years
  diffuse <- PS + sum(data$Diffuse) / n_years
  atm_2_agri <- sum(data$Atm2DiffAgri) / n_years
  atm_2_diff <- sum(data$Atm2Diff) / n_years
  atm_2_land_ret <- sum(data$Atm2LandRet) / n_years
  sd_2_diff <- sum(data$SD2Diff) / n_years
  sd_2_land_ret <- sum(data$SD2LandRet) / n_years
  agri_2_diffuse <- diffuse_agri - sum(data$DiffAgri2LandRet) / n_years
  agri_2_land_ret <- sum(data$DiffAgri2LandRet) / n_years
  load_2_riv_ret <- sum(data$CatchRivRet) / n_years
  load_2_lake_ret <- sum(data$CatchLakeRet) / n_years
  load_2_outlet <- sum(data$CatchLoad[which(data$To_catch < 1)]) / n_years

  name <- c("PS", "Min", "Man", "Fix", "Soil", "Atm", "Sd",
            "Agriculture land diffuse sources", "Load to stream", "Land Ret",
            "River Ret", "Lake Ret", "Load to outlet")
  nodesID <- c(1:length(name)) - 1
  nodes <- data.frame(nodesID, name)

  source <- c(0,
              1, 2, 3, 4,
              5, 5, 5,
              6, 6,
              7, 7,
              8, 8, 8)

  target <- c(8,
              7, 7, 7, 7,
              7, 8, 9,
              8, 9,
              8, 9,
              10, 11, 12)

  value <- c(PS,
             min, man, fix, soil,
             atm_2_agri, atm_2_diff, atm_2_land_ret,
             sd_2_diff, sd_2_land_ret,
             agri_2_diffuse, agri_2_land_ret,
             load_2_riv_ret, load_2_lake_ret, load_2_outlet)

  links <- data.frame(source, target, value)

  print(
    networkD3::sankeyNetwork(
      Links = links,
      Nodes = nodes,
      Source = "source",
      Target = "target",
      Value = "value",
      NodeID = "name",
      units = "TP T/y",
      colourScale = networkD3::JS("d3.scaleOrdinal(d3.schemeCategory20);"),
      fontSize = 30, nodeWidth = 30, sinksRight = TRUE)
  )

  name <- c("PS", "Min", "Man", "Fix", "Soil", "Atm", "Sd",
            "Agriculture land diffuse sources", "Load to stream", "Land Ret",
            "River Ret", "Lake Ret", "Load to Outlet")

  output <- c(PS, min, man, fix, soil, atm, sd,
              diffuse_agri,
              diffuse,
              (agri_2_land_ret + atm_2_land_ret + sd_2_land_ret),
              load_2_riv_ret, load_2_lake_ret,
              load_2_outlet)

  outloads <- data.frame(name, "Mean_TN_t/y" = output)

  outloads
}

#'
#' @title Reference summary plot
#'
#' @description Summary of the reference values in the stations
#'
#' @param annual_data data frame. Sources of nutrient for each year and
#' catchments.
#'
#' @return A barplot, a histogram-density and a boxplot
#'
#' @importFrom graphics hist grid
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(annual_data_TN)
#' references_plot(annual_data_TN)
#' }
#' @export
#'
references_plot <- function(annual_data) {

  # Conteo del número de referencias totales
  TN_ref_values <- annual_data[!is.na(annual_data$YearlyMass),]
  total_ref <- nrow(TN_ref_values)
  # Conteo de referencias por año
  counts <- table(TN_ref_values$YearValue)
  # Crear el gráfico de barras
  par(mfrow = c(1, 1))
  barplot(counts, ylab = "References by Year", xlab = "Year",
          main = paste("References by year (Total references: ", total_ref, ")"),
          col = "steelblue", border = "black")
  graphics::grid(nx = NA, ny = NULL)    # Añadir una línea de rejilla

  #distribucion de los valores de referencia
  TheRefMass <- annual_data$YearlyMass[!is.na(annual_data$YearlyMass)]
  par(mfrow = c(1, 2))
  # Crear histograma con densidad
  graphics::hist(TheRefMass, probability = TRUE, main = "References Distribution", xlab = "Yearly Mass", col = "lightblue")
  # Crear densidad
  densidad <- density(TheRefMass)
  # Superponer curva de densidad
  lines(densidad, col = "darkblue", lwd = 2)
  # Agregar una leyenda
  legend('topright', legend = c("Histogram", "Density"), fill = c("lightblue", "darkblue"))
  # boxplot
  boxplot(TheRefMass, main = "Reference values", ylab = "Values",col = "lightblue")

}

#'
#' @title Scatter plot of goodness-of-fit metric vs parameters
#'
#' @description Scatter plot of goodness-of-fit metric vs parameters
#'
#' @param df_cb data frame. A table with the result of the calibration process.
#' @param param character. Goodness of fit metric:"NSE", "rNSE", "NSE",
#' "mNSE", "MAE", "PBIAS", "cp", "R2",...
#'
#' @importFrom graphics plot
#'
#' @return Multiple scatter plot
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(catch_data_TN)
#' data(annual_data_TN)
#' # the parameter for the calibration of the model
#' n_iter <- 2 # number of iterations
#' # the lower limits for all params (alpha_P, alpha_L, sd_coef)
#' low <- c(10, 0.000, 0.1)
#' # the upper limits for all params (alpha_P, alpha_L, sd_coef)
#' upp <- c(70, 0.3,  0.9)
#' # years in which the model should be executed
#' years <- 1990:2018
#' # execution of the calibration
#' df_calib <- calib_green(catch_data_TN, annual_data_TN, n_iter, low, upp,
#' years)
#' gof_mes <- "NSE"
#' scatter_plot(df_calib, gof_mes)
#' }
#'
#' @export
#'
scatter_plot <- function(df_cb, param) {

  oldpar <- par(no.readonly = TRUE)
  on.exit(par(oldpar))
  graphics::par(mfrow = c(1, 3))
  if (param == "PBIAS") {
    df_cb[, param] <- abs(df_cb[, param])
  }

  graphics::plot(df_cb$alpha_P, df_cb[, param],
                 ylab = param,
                 xlab = "alpha_P")
  graphics::plot(df_cb$alpha_L, df_cb[, param],
                 ylab = param,
                 xlab = "alpha_L")
  graphics::plot(df_cb$sd_coeff, df_cb[, param],
                 ylab = param,
                 xlab= "sd_coeff")

}

#'
#' @title Selection of best calibration parameters
#'
#' @description Return the best calibration parameter set according to one
#' goodness-of-fit metric
#'
#' @param df_cb data frame. The result of the calibration process.
#' @param param numeric. Goodness-of-fit measures. "NSE", "rNSE", "NSE",
#' "mNSE", "MAE", "PBIAS", "cp", "R2",...
#'
#' @return A vector with the 3 parameters
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(catch_data_TN)
#' data(annual_data_TN)
#' # the parameter for the calibration of the model
#' n_iter <- 2 # number of iterations
#' # the lower limits for all params (alpha_P, alpha_L, sd_coef)
#' low <- c(10, 0.000, 0.1)
#' # the upper limits for all params (alpha_P, alpha_L, sd_coef)
#' upp <- c(70, 0.3,  0.9)
#' # years in which the model should be executed
#' years <- 1990:2018
#' # execution of the calibration
#' df_calib <- calib_green(catch_data_TN, annual_data_TN, n_iter, low, upp,
#' years)
#' # Extract the best set of parameter according to a Goodnes of fit metric
#' gof_mes <- "NSE"
#' NSE_bestParams <- select_params(df_calib, gof_mes)
#' }
#'
#' @export
#'
select_params <- function(df_cb, param){

  best_par <- NULL
  ind_mtr <- c(6, 15, 9, 10, 11, 17, 8, 19)

  for (ind in ind_mtr) {
    if (ind == 6 | ind == 17 ) {
      irow <- which(abs(df_cb[, ind]) ==  min(abs(df_cb[, ind]),
                                              na.rm = TRUE))[1]
    } else if (ind %in% c(15, 9, 10, 11,8,19)) {
      irow <- which(df_cb[, ind] == max(df_cb[, ind], na.rm = TRUE))[1]
    }
    best_par <- rbind(best_par, data.frame(metric = names(df_cb)[ind],
                                           df_cb[irow, ]))

  }

  best_par[best_par$metric == param, c(param, "alpha_P", "alpha_L", "sd_coeff")]
}


#
#' @title Facet year plot
#'
#' @description This function executes a parallelized hydrological simulation
#' and generates a comparison plot of simulated vs. observed loads. The function
#' leverages multiple CPU cores to run the launch_green model across several
#' years, aggregates the results, and produces a faceted scatter plot with a
#' 1:1 reference line to evaluate model performance by year.
#'
#' @param catch_data data frame. Definition of the topological sequence of
#' catchments.
#' @param annual_data data frame. Sources of nutrient for each year and
#' catchments.
#' @param alpha_p numeric. First model parameter, the basin retention
#' coefficient.
#' @param alpha_l numeric. Second model parameter, the river retention
#' coefficient.
#' @param sd_coef numeric. Third model parameter, fraction of domestic diffuse
#' sources that reaches the stream network.
#' @param years integer. Years to be used in the calibration. For sequences use
#' c(yearini:yearend).
#' @param name_basin character. The name of the basin
#' @param maxvalue numeric. The maximum value
#' @return One object, a data frame
#'
#' @importFrom ggplot2 ggplot ggtitle geom_point xlim ylim theme_bw geom_abline facet_wrap
#' @importFrom parallelly availableCores
#' @importFrom parallel makeCluster clusterExport clusterEvalQ parLapply stopCluster
#'
#' @export
#'
simobs_annual_plot <- function(catch_data, annual_data, alpha_p, alpha_l,
                            sd_coef, years, name_basin, maxvalue) {

  years <- lapply(years, function(x){x})

  n_cores <- parallelly::availableCores()
  cluster <- parallel::makeCluster(n_cores)
  parallel::clusterExport(cluster, list("launch_green", "check_colnames_annual",
                                        "check_colnames_catch",
                                        "aggregate_loop", "check_years",
                                        "append_empty_cols",
                                        "data_preparation"),
                          envir = environment())
  parallel::clusterEvalQ(cluster, c(library("data.table"), library("dplyr")))
  DF_ScenGlobal <- parallel::parLapply(cluster, years, launch_green,
                                       annual_data = annual_data,
                                       catch_data = catch_data,
                                       alpha_p = alpha_p,
                                       alpha_l = alpha_l,
                                       sd_coef = sd_coef)
  parallel::stopCluster(cluster)
  DF_ScenGlobal <- do.call("rbind", DF_ScenGlobal)

  DF_ObservadoG <- DF_ScenGlobal[!is.na(DF_ScenGlobal$ObsLoad), ]

  names(DF_ObservadoG)
  p <- ggplot2::ggplot(data = DF_ObservadoG, ggplot2::aes(x = ObsLoad,
                                                          y = CatchLoad)) +
    ggplot2::ggtitle(name_basin)  +
    ggplot2::geom_point() +
    ggplot2::xlim(0, maxvalue) +
    ggplot2::ylim(0, maxvalue) +
    ggplot2::theme_bw(base_size = 10  ) +
    ggplot2::geom_abline(intercept = 0, slope = 1) +
    ggplot2::facet_wrap(~Year)
  print(p)

  return(DF_ScenGlobal)

}

#' @title Annual data TN
#' @description Defines the sources of nutrient (nitrogen) for each year and catchments.
#' @format A data frame with 14 variables:
#' \describe{
#'   \item{\code{BasinID}}{integer. The basin unique identifier.}
#'   \item{\code{YearValue}}{integer. The year for which data are defined.}
#'   \item{\code{HydroID}}{integer positive. Unique catchment identifier.}
#'   \item{\code{NextDownID}}{integer. Unique identifier of the catchment to
#'   which the catchment goes.}
#'   \item{\code{Atm}}{double. Annual nitrogen deposition from atmosphere
#'   (ton/yr).}
#'   \item{\code{Min}}{double. Annual amount of nitrogen from mineral
#'   fertilisers (ton/yr).}
#'   \item{\code{Man}}{double. Annual amount of nitrogen in manure fertilisers
#'   (ton/yr).}
#'   \item{\code{Fix}}{double. Annual amount of nitrogen fixation by leguminous
#'   crops and fodder (ton/yr).}
#'   \item{\code{Soil}}{double. Annual amount of nitrogen fixation by bacteria
#'   in soils (ton/yr).}
#'   \item{\code{Sd}}{double. Nitrogen input from scattered dwellings (ton/yr).}
#'   \item{\code{Ps}}{double. Nitrogen input from point sources (ton/yr).}
#'   \item{\code{YearlyMass}}{double. Observed annual total nitrogen load
#'   (TN ton/yr) from monitoring station data.}
#'   \item{\code{ForestFraction}}{double. Non-agricultural land cover in the
#'   catchment (fraction).}
#'   \item{\code{InvNrmRain}}{double. Inverse of normalized rainfall.}
#'}
"annual_data_TN"

#' @description
#' The package provides tools and methods to apply the model Geospatial
#' Regression Equation for European Nutrient losses (GREEN;
#' Grizzetti et al. (2005); Grizzetti et al. (2012); Grizzetti et al. (2021)) to
#' an area of interest in R environment. The package comprises functions for
#' assessing annual nutrient (nitrogen and phosphorus) loads from a basin or
#' region of interest, land and river retention, and contribution shares by
#' sources. A brief description of the model, including sources and parameters,
#' can be found at the end of this document. Further, the package includes
#' functions for loading spatio-temporal data , calibrating basin parameters,
#' performing an advanced sensitivity analysis to evaluate the calibration
#' results, and visualizing model inputs and outputs through plots and maps.
#' The package is parallel-capable to alleviate the computational burden in
#' large basins.
#'
#'
#' @references
#' Grizzetti, B., Bouraoui, F., De Marsily, G., & Bidoglio, G. (2005). A
#' statistical method for source apportionment of riverine nitrogen loads.
#' Journal of Hydrology, 304(1-4), 302-315.
#' \doi{doi:10.1016/j.jhydrol.2004.07.036}
#'
#' Grizzetti, B., Bouraoui, F., De Marsily, G., (2008). Assessing nitrogen
#' pressures on European surface water. Global Biogeochem. Cycles 22..
#'
#' Grizzetti, B., Bouraoui, F., & Aloe, A. (2012). Changes of nitrogen and
#' phosphorus loads to European seas. Global Change Biology, 18(2), 769-782.
#' \doi{doi:10.1111/j.1365-2486.2011.02576.x}
#'
#' Grizzetti, B., Vigiak, O., Udias, A., Aloe, A., Zanni, M., Bouraoui, F.,
#' Pistocchi, A., Dorati, C., Friedland, R., De Roo, A., others & Bielza, M.
#' (2021). How EU policies could reduce nutrient pollution in European inland
#' and coastal waters. Global Environmental Change, 69, 102281.
#' \doi{doi:10.1016/j.gloenvcha.2021.102281}
#' @keywords internal
"_PACKAGE"

load("./data/annual_data_TN.rda")
load("./data/catch_data_TN.rda")
load("./data/annual_data_TP.rda")
load("./data/catch_data_TP.rda")
load("./data/sh_file.rda")
globalVariables(c("AA", "SumDS", "SumPSCmp1", "CatchToRiver", "BB", "Year",
                  "CatchLoad", "ForestFraction", "HydroID", "InvNrmRain",
                  "LakeFrRet", "NrmLengthKm", "Shreve", "To_catch", "YearValue",
                  "YearlyNMass", ":=", ".", "%dopar%", "alpha_p", "alpha_l",
                  "sd_C", "num_cores", "n_iter", "task", "launch_green",
                  "ObsLoad", "shrLevel", "value", "DF_LoadAvg_Type_Shr_Year",
                  "DF_LoadAvg_Type_Year", "Total", "variable", "scale_barRef",
                  "scale_barTextS", "scale_barRefs", "scale_barTexts",
                  "DrainAreaS", "Atm", "Atm2Diff", "Atm2DiffAgri",
                  "Atm2LandRet", "Bg", "CatchLakeRet", "CatchRivRet",
                  "DiffAgri2LandRet", "Diffuse", "Fix", "LandRet", "Man", "Min",
                  "Ps", "SD2Diff", "SD2LandRet", "Sd", "Soil", "YearlyMass",
                  "AllDesting", "AllOrig", "BasinID",  "DiffuseAgri",
                  "NextDownID", "RemaiHydroId", "WholeHydroID", "PredictLoad"))

#' @title Catch data TN
#' @description Defines the topological sequence of catchments for nitrogen.
#' @format A data frame with 5 variables:
#' \describe{
#'   \item{\code{HydroID}}{integer positive. Unique catchment identifier.}
#'   \item{\code{To_catch}}{integer. Unique identifier of the catchment to which
#'   the catchment goes. Note that for the outlet To_catch== -1.}
#'   \item{\code{Shreve}}{integer. this indicates the Shreve order of the
#'   topological sequence in the stream network.}
#'   \item{\code{LakeFrRet}}{fraction, 0-1. Lake retention fraction.}
#'   \item{\code{NrmLengthKm}}{double. Normalized length of catchment reach.}
#'}
"catch_data_TN"

#
#' @title Aggregate loop
#'
#' @description This function aggregate the variables of data frame.
#'
#' @param listaShrt, cdat_greenLt A data frame
#' @param years A vector of integer
#' @param type A character with values "L" when running launch_green and "N"
#' when running region_nut_balance
#' @return A matrix
#'
#' @importFrom data.table := setkey
#'
#' @keywords internal
#'
aggregate_loop <- function(listaShrt, cdat_greenLt, years, type){

  cdat_total <- NULL

  for (yr in years) {
    print(paste("Calculating year:", yr))
    shr_hydro_IDt <- unique(listaShrt[[1]][Year == yr]$HydroID)
    cdat_greenLt_Year <- cdat_greenLt[Year == yr]
    cdat_greenLt_Year[cdat_greenLt_Year$HydroID %in% shr_hydro_IDt,
                      CatchLoad := (1 - LakeFrRet) *
                        (CatchToRiver + CatchLoad) * BB]
    if (type == "N"){
      cdat_greenLt_Year[cdat_greenLt_Year$HydroID %in% shr_hydro_IDt,
                        CatchRivRet := (CatchToRiver + CatchLoad) * (1 - BB)]
      cdat_greenLt_Year[cdat_greenLt_Year$HydroID %in% shr_hydro_IDt,
                        CatchLakeRet := LakeFrRet *
                          (CatchToRiver + CatchLoad) * BB]
    }

    data.table::setkey(cdat_greenLt_Year, "To_catch")

    if(length(listaShrt)>1){
      for (iShr in 2:(length(listaShrt))) {
        shr_hydro_IDt <- listaShrt[[iShr]][Year == yr]$HydroID
        df_agt <- cdat_greenLt_Year[.(shr_hydro_IDt)]
        df_agt <- df_agt[, .(To_catch, CatchLoad)]
        df_agt <- df_agt[, .(CatchLoad = sum(CatchLoad)), .(HydroID = To_catch)]

        cdat_greenLt_Year[match(df_agt$HydroID,cdat_greenLt_Year$HydroID),
                          CatchLoad := df_agt$CatchLoad]
        cdat_greenLt_Year[cdat_greenLt_Year$HydroID %in% shr_hydro_IDt,
                          CatchLoad := (1 - LakeFrRet) *
                            (CatchToRiver + CatchLoad) * BB]
        if (type == "N"){
          cdat_greenLt_Year[cdat_greenLt_Year$HydroID %in% shr_hydro_IDt,
                            CatchRivRet := (CatchToRiver + CatchLoad) * (1 - BB)]
          cdat_greenLt_Year[cdat_greenLt_Year$HydroID %in% shr_hydro_IDt,
                            CatchLakeRet := LakeFrRet *
                              (CatchToRiver + CatchLoad) * BB]
        }

      }
    }

    cdat_total <- rbind(cdat_total, cdat_greenLt_Year)
  }

  cdat_total
}

#'
#' @title Append empty columns
#'
#' @description This function add empty values to the data frame.
#'
#' @param df data frame. A data frame to add two new columns, Fix and Soil
#'
#' @return One object, a data frame.
#'
#' @keywords internal
#'
append_empty_cols <- function(df){

  if (length(df) == 12) {
    df <- df %>%
      dplyr::rename(Atm = Bg) %>%
      dplyr::mutate("Soil" = 0, "Fix" = 0)
  }

  df
}

#
#' @title Calibration of the GREEN model
#'
#' @description Runs GREEN model calibration
#'
#' @param catch_data data frame. Definition of the topological sequence of
#' catchments.
#' @param annual_data data frame. Sources of nutrient for each year and
#' catchments.
#' @param n_iter numeric. Number of iterations for the calibration process.
#' @param low numeric. Lower bounds of the calibration parameters.
#' @param upp numeric. Upper bounds of the calibration parameters.
#' @param years integer. Years to be used in the calibration. For sequences use
#' c(yearini:yearend).
#' @return One object, a data frame with the model calibration
#'
#' @importFrom parallel detectCores parSapply
#' @importFrom parallelly availableCores
#' @importFrom dplyr rename bind_cols
#' @importFrom magrittr %>%
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(catch_data_TN)
#' data(annual_data_TN)
#' # the parameter for the calibration of the model
#' n_iter <- 2 # number of iterations
#' # the lower limits for all params (alpha_P, alpha_L, sd_coef)
#' low <- c(10, 0.000, 0.1)
#' # the upper limits for all params (alpha_P, alpha_L, sd_coef)
#' upp <- c(70, 0.3,  0.9)
#' # years in which the model should be executed
#' years <- 1990:2018
#' # execution of the calibration
#' dF_calib <- calib_green(catch_data_TN, annual_data_TN, n_iter, low, upp,
#' years)
#' }
#'
#' @export
calib_green <- function(catch_data, annual_data, n_iter, low, upp, years){

  catch_data <- check_colnames_catch(catch_data)
  annual_data <- check_colnames_annual(annual_data)

  annual_data <- append_empty_cols(annual_data)

  par_range <- data.frame(min = low, max = upp)
  latin_range <- as.data.frame(FME::Latinhyper(par_range, n_iter))

  n_cores <- max(1, parallelly::availableCores() - 2)
  cluster <- parallel::makeCluster(n_cores)
  parallel::clusterExport(cluster, list("launch_green", "check_colnames_annual",
                                        "check_colnames_catch",
                                        "calib_green_help", "check_years",
                                        "append_empty_cols", "aggregate_loop",
                                        "data_preparation"),
                          envir = environment())
  parallel::clusterEvalQ(cluster, c(library("data.table"), library("dplyr")))
  time <- system.time(
    r <- parallel::parSapply(cluster, 1:n_iter, calib_green_help,
                             catch_data = catch_data,
                             annual_data = annual_data,
                             years = years,
                             latin_range = latin_range,
                             simplify = FALSE)
  )
  parallel::stopCluster(cluster)

  r <- do.call("cbind", r)

  print(paste("Elapsed time:", time[3]))
  data.frame(t(r))
}

#
#' @title Parallel calibration help function
#'
#' @description This is a help function for calib_green function
#'
#' @param task An integer with the number of simulation
#' @param catch_data data frame. It defines the topological sequence
#' of catchments.
#' @param annual_data data frame. It defines the sources of nutrient for each
#' year and catchments.
#' @param latin_range data frame. It defines the values for the params taking +
#' into account the limits.
#' @param years integer. It defines the years of interest.
#' @return A matrix
#'
#' @keywords internal
#'
calib_green_help <- function(task, catch_data, annual_data, years, latin_range){

  alpha_p <- as.vector(latin_range[task, 1])
  alpha_l <- as.vector(latin_range[task, 2])
  sd_c <- as.vector(latin_range[task,3])
  df_model_res <- launch_green(catch_data, annual_data, alpha_p, alpha_l,
                               sd_c, years)
  df_res_calib <- df_model_res[!is.na(df_model_res$ObsLoad), ]
  df_res_calib <- df_res_calib[order(df_res_calib$Year,
                                     df_res_calib$HydroID),]

  gof_val <- gof(sim = df_res_calib$CatchLoad,
                 obs = df_res_calib$ObsLoad,
                 digits = 4)

  S_OutLoad <- sum(df_model_res[df_model_res$To_catch== -1,
                                c(CatchToRiver)] )

  result <- rbind(as.data.frame(gof_val),
                  alpha_P = alpha_p,
                  alpha_L = alpha_l,
                  sd_coeff = sd_c,
                  SumOutLoad = S_OutLoad)

  result
}

#'
#' @title Check colnames
#'
#' @description This function checks that all column names for the catch_data
#' data frame are correct.
#'
#' @param df data frame.
#' @return A data frame
#'
#' @importFrom dplyr select
#' @importFrom magrittr %>%
#'
#' @keywords internal
#'
check_colnames_annual <- function(df){

  header_TP <- c('BasinID', 'YearValue', 'HydroID', 'NextDownID', 'Bg', 'Min',
                 'Man', 'Sd', 'Ps', 'YearlyMass', 'ForestFraction', 'InvNrmRain')

  header_TN <- c('BasinID', 'YearValue', 'HydroID', 'NextDownID', 'Atm', 'Min',
                 'Man', 'Fix', 'Soil', 'Sd', 'Ps', 'YearlyMass',
                 'ForestFraction', 'InvNrmRain')

  if (all(header_TP %in% names(df))) {
    df <- df %>%
      dplyr::select(BasinID, YearValue, HydroID, NextDownID, Bg, Min, Man,
                    Sd, Ps, YearlyMass, ForestFraction, InvNrmRain)
    return(df)
  }

  if (all(header_TN %in% names(df))) {
    df <- df %>%
      dplyr::select(BasinID, YearValue, HydroID, NextDownID, Atm, Min, Man,
                    Fix, Soil, Sd, Ps, YearlyMass, ForestFraction, InvNrmRain)
    return(df)
  }

  stop('The names of the variables of the data frame does not match')

}

#'
#' @title Check colnames catch_data
#'
#' @description This function checks that all column names for the annual_data
#' data frame are correct.
#'
#' @param df data frame.
#' @return A data frame
#'
#' @importFrom dplyr select
#' @importFrom magrittr %>%
#'
#' @keywords internal
#'
check_colnames_catch <- function(df){

  header <- c("HydroID", "To_catch", "Shreve", "LakeFrRet", "NrmLengthKm")

  if (all(header %in% names(df))) {
    df <- df %>%
      dplyr::select(HydroID, To_catch, Shreve, LakeFrRet, NrmLengthKm)
    return(df)
  }

  stop('The names of the variables of the data frame does not match')

}

#'
#' @title Check years
#'
#' @description This function check if years are in data frame.
#'
#' @param loc_years numeric. A vector of integer with the years of interest
#' @param years numeric. A vector of integer with the years with information in
#' data frame
#' @return A vector of integer
#'
#' @keywords internal
#'
check_years <- function(loc_years, years){

  if (length(loc_years[loc_years %in% years]) != length(loc_years)) {
    warning("Some years are not in the scenario")
  }

  if (is.null(loc_years)) {
    stop("Years can not be null")
  } else {
    years <- loc_years[loc_years %in% years]
  }

  years
}

#
#' @title data_preparation
#'
#' @description Prepare data to launch green
#'
#' @param catch_data data frame. Definition of the topological sequence of
#' catchments.
#' @param annual_data data frame. Sources of nutrient for each year and
#' catchments.
#' @return One list with two data table
#'
#' @importFrom data.table := as.data.table
#'
#' @keywords internal
#'
data_preparation <- function(catch_data, annual_data){

  catch_data <- check_colnames_catch(catch_data)
  annual_data <- check_colnames_annual(annual_data)

  annual_data <- append_empty_cols(annual_data)

  annual_datat <- data.table::as.data.table(annual_data)
  catch_datat <- data.table::as.data.table(catch_data)

  list("annual_datat" = annual_datat, "catch_datat" = catch_datat)
}

#'
#' @title Geospatial Regression Equation parallel execution returning the source
#' apportionment
#'
#' @description Run GREEN model with selected parameter set and returns the
#' nutrient load by each source for all catchments in the Basin.
#'
#' @param catch_data data frame. Definition of the topological sequence of
#' catchments.
#' @param annual_data data frame. Sources of nutrient for each year and
#' catchments.
#' @param alpha_p numeric. First model parameter, the basin retention
#' coefficient.
#' @param alpha_l numeric. Second model parameter, the river retention
#' coefficient.
#' @param sd_coef numeric. Third model parameter, fraction of domestic diffuse
#' sources that reaches the stream network.
#' @param loc_years integer. Years in which the model should be executed.
#' @return One object, a data frame with the nutrient load by each source for
#' all catchments in the Basin
#'
#' @importFrom parallel makeCluster detectCores clusterExport clusterEvalQ
#' parLapply stopCluster
#' @importFrom reshape2 dcast
#' @importFrom dplyr rename bind_rows bind_cols
#' @importFrom magrittr %>%
#' @importFrom parallelly availableCores
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(catch_data_TN)
#' data(annual_data_TN)
#' # the parameter to assess the basin model
#' alpha_p <- 35.09
#' alpha_l <- 0.02
#' sd_coef <- 0.2
#' # year in which the model should be executed
#' loc_years <- 1990:2018
#' # Computing the source apportionment
#' basin_loads_s <- green_shares(catch_data_TN, annual_data_TN, alpha_p, alpha_l,
#' sd_coef, loc_years)
#' }
#'
#' @export
#'
green_shares <- function(catch_data, annual_data, alpha_p, alpha_l, sd_coef,
                         loc_years){

  n_cores <- parallelly::availableCores() - 1

  old_threads <- Sys.getenv("OPENBLAS_NUM_THREADS")
  Sys.setenv(OPENBLAS_NUM_THREADS = 1)
  on.exit(Sys.setenv(OPENBLAS_NUM_THREADS = old_threads), add = TRUE)

  catch_data <- check_colnames_catch(catch_data)
  annual_data <- check_colnames_annual(annual_data)

  if (length(annual_data) == 12) {
    inputs <- c("Bg", "Min", "Man", "Sd", "Ps")
  } else {
    inputs <- c("Atm", "Min", "Man", "Fix", "Soil", "Sd", "Ps")
  }

  annual_data <- append_empty_cols(annual_data)

  inputs_df <- lapply(seq_len(length(inputs)), function(x){
    temp <- annual_data
    temp[inputs[-x]] <- 0
    return(temp)
  })
  names(inputs_df) <- inputs

  cluster <- parallel::makeCluster(n_cores)

  parallel::clusterExport(cluster, varlist = c("launch_green", "check_colnames_annual",
                                               "check_colnames_catch", "aggregate_loop",
                                               "check_years", "append_empty_cols",
                                               "data_preparation"),
                          envir = asNamespace("GREENeR"))

  parallel::clusterEvalQ(cluster, {
    library(data.table)
    library(dplyr)
  })

  results <- parallel::parLapply(cluster, inputs_df, launch_green,
                                 catch_data = catch_data,
                                 alpha_p = alpha_p,
                                 alpha_l = alpha_l,
                                 sd_coef = sd_coef,
                                 loc_years = loc_years)

  parallel::stopCluster(cluster)
  names(results) <- inputs

  results2 <- dplyr::bind_rows(results, .id = "group")

  results2_cast <- data.table::dcast(data.table::as.data.table(results2),
                                     HydroID + To_catch + Year ~ group,
                                     value.var = "CatchLoad")

  idx <- which(names(results2_cast) %in% inputs)
  results2_cast[["CatchLoad"]] <- rowSums(results2_cast[, idx, with = FALSE])

  return(as.data.frame(results2_cast))
}


#
#' @title GREEN execution
#'
#' @description Applies the model GREEN (Grizzetti et al.
#' [@grizzetti2012changes, @grizzetti2021eu]) to a basin and returns annual
#' nutrient (nitrogen or phosphorus) load for all catchments in the basin
#'
#' @param catch_data data frame. Definition of the topological sequence of
#' catchments.
#' @param annual_data data frame. Sources of nutrient for each year and
#' catchments.
#' @param alpha_p numeric. First model parameter, the basin retention
#' coefficient.
#' @param alpha_l numeric. Second model parameter, the river retention
#' coefficient.
#' @param sd_coef numeric. Third model parameter, fraction of domestic diffuse
#' sources that reaches the stream network.
#' @param loc_years integer. Years in which the model should be executed.
#' @param atm_coeff numeric. It defines the atmospheric attenuation coefficient.
#' @return One object, a data frame
#'
#' @importFrom data.table := setkey as.data.table
#'
#' @keywords internal
#'
launch_green <- function(catch_data, annual_data, alpha_p, alpha_l, sd_coef,
                         loc_years, atm_coeff = 0.38) {

  years <- check_years(loc_years, unique(annual_data$YearValue))

  preparation <- data_preparation(catch_data, annual_data)
  annual_datat <- preparation$annual_datat
  catch_datat <- preparation$catch_datat

  annual_datat[, AA := exp(-alpha_p * InvNrmRain)]

  annual_datat[, SumDS := (Min + Man + Fix + Soil + Atm * (1 - ForestFraction))]

  annual_datat[, SumPSCmp1 := (Ps + atm_coeff * Atm * ForestFraction +
                               sd_coef * Sd )]

  annual_datat[, CatchToRiver := (SumDS * AA + SumPSCmp1)]

  green_loadst <- annual_datat[, .(HydroID, Year = YearValue,
                                   ObsLoad = YearlyMass, CatchToRiver)]

  catch_datat[, BB := exp(-1 * alpha_l * NrmLengthKm)]

  data.table::setkey(catch_datat, "HydroID")
  data.table::setkey(green_loadst, "HydroID")
  cdat_greenLt <- catch_datat[green_loadst]

  data.table::setkey(cdat_greenLt, "Shreve")

  listaShrt <- split(cdat_greenLt[, .(HydroID, Shreve, Year)], by="Shreve")

  cdat_greenLt$CatchLoad <- 0

  type <- "L"
  cdat_total <- aggregate_loop(listaShrt, cdat_greenLt, years, type)

  cdat_total

}


#'
#' @title Nutrient balance based in the application of the Geospatial Regression
#' Equation returning the diffuse, land retention, point sources
#'
#' @description Computes the basin nutrient balance.
#'
#' @param catch_data data frame. Definition of the topological sequence of
#' catchments.
#' @param annual_data data frame. Sources of nutrient for each year and
#' catchments.
#' @param alpha_p numeric. First model parameter, the basin retention
#' coefficient.
#' @param alpha_l numeric. Second model parameter, the river retention
#' coefficient.
#' @param sd_coef numeric. Third model parameter, fraction of domestic diffuse
#' sources that reaches the stream network.
#' @param loc_years integer. Years in which the model should be executed.
#' @param atm_coeff numeric. A value for atmospheric attenuation coefficient.
#' @return One object, a data frame with the basin nutrient balance
#'
#' @importFrom data.table as.data.table setkey
#' @importFrom dplyr rename bind_cols
#' @importFrom magrittr %>%
#'
#' @examples
#' \donttest{
#' # the data of the TN scenario
#' data(catch_data_TN)
#' data(annual_data_TN)
#' # the parameter to assess the basin model
#' alpha_p <- 35.09
#' alpha_l <- 0.02
#' sd_coef <- 0.2
#' # year in which the model should be executed
#' loc_years <- 1990:2018
#' # Computing the nutrient balance
#' basin_loads_b <- region_nut_balance(catch_data_TN, annual_data_TN, alpha_p, alpha_l,
#' sd_coef, loc_years)
#' }
#'
#' @export
#'
region_nut_balance <- function(catch_data, annual_data, alpha_p, alpha_l, sd_coef,
                      loc_years, atm_coeff = 0.38) {

  years <- check_years(loc_years, unique(annual_data$YearValue))

  preparation <- data_preparation(catch_data, annual_data)
  annual_datat <- preparation$annual_datat
  catch_datat <- preparation$catch_datat

  annual_datat[, AA := exp(-alpha_p * InvNrmRain)]

  annual_datat[, Atm2DiffAgri := Atm * (1 - ForestFraction) ]
  annual_datat[, SumDS := (Min + Man + Fix + Soil + Atm2DiffAgri)]

  annual_datat[, Atm2Diff :=  atm_coeff * Atm * ForestFraction]
  annual_datat[, SD2Diff :=  sd_coef * Sd]
  annual_datat[, Diffuse := (SumDS * AA + Atm2Diff + SD2Diff )]

  annual_datat[, Atm2LandRet := (1 - atm_coeff) * Atm * ForestFraction]
  annual_datat[, SD2LandRet := ((1 - sd_coef) * Sd) ]
  annual_datat[, DiffAgri2LandRet := (SumDS * (1 - AA))]
  annual_datat[, LandRet := (DiffAgri2LandRet + Atm2LandRet + SD2LandRet)]
  annual_datat[, SumPSCmp1 := (Ps +  Atm2Diff + SD2Diff)]
  annual_datat[, CatchToRiver := (SumDS * AA + SumPSCmp1)]

  green_loadst <- annual_datat[, .(HydroID, Year = YearValue,
                                   Atm, Sd, Min, Man, Fix, Soil,
                                   PS = Ps,
                                   DiffuseAgri = SumDS,
                                   Atm2DiffAgri,
                                   Atm2Diff,
                                   SD2Diff,
                                   Diffuse,
                                   DiffAgri2LandRet,
                                   Atm2LandRet,
                                   SD2LandRet,
                                   LandRet,
                                   CatchToRiver)]

  catch_datat[, BB := exp(-1 * alpha_l * NrmLengthKm)]

  data.table::setkey(catch_datat, "HydroID")
  data.table::setkey(green_loadst, "HydroID")
  cdat_greenLt <- catch_datat[green_loadst]

  data.table::setkey(cdat_greenLt, "Shreve")

  listaShrt <- split(cdat_greenLt[, .(HydroID, Shreve, Year)],
                     by = "Shreve")

  cdat_greenLt$CatchLoad <- 0
  cdat_greenLt$CatchRivRet <- 0
  cdat_greenLt$CatchLakeRet <- 0

  type <- "N"
  cdat_total <- aggregate_loop(listaShrt, cdat_greenLt, years, type)

  cdat_total
}

#'
#' @title Read NS data
#'
#' @description Function to read the data and return the data frame for GREEN
#' execution.
#'
#' @param path string. A string with the path of the CSV files.
#' @param tsn file. A CSV file with nine variables YearValue (integer),
#' HydroID (integer), Atm (float), Min (float), Man (float), Fix (float),
#' Soil (float), Sd (float) and Ps (float).
#' @param obs file. A CSV file with three variables YearValue (integer),
#' HydroID (integer) and YearlyMass (float).
#' @param ff file. A CSV file with three variables YearValue (integer),
#' HydroID (integer) and ForestFraction (float).
#' @param rain file. A CSV file with three variables YearValue (integer),
#' HydroID (integer) and Rain (float).
#' @param topo file. A CSV file with two variables HydroID (integer) and
#' Next_HydroID (integer).
#' @param lr file. A CSV file with three variables HydroID (integer),
#' AvgDepth (float) and ResTime (float).
#' @param length file. A CSV file with two variables HydroID (integer) and
#' LengthKm (float).
#'
#' @return One object, a list with two data frame. First position of the list
#' contains the catch data and the second one the annual data.
#'
#' @importFrom utils read.csv
#'
#' @examples
#' \donttest{
#' path <- "https://raw.githubusercontent.com/calfarog/GREENeR_data/main/data/csv/"
#' ns_data <- read_NSdata(path, "TS_nutrients.csv", "Obs_monitoring.csv",
#' "ForestFr.csv", "Precipitation.csv", "Topology.csv", "LakeProperties.csv",
#' "Length.csv")
#' }
#'
#' @export
#'
read_NSdata <- function(path, tsn, obs, ff, rain, topo, lr, length) {

  tsn <- read.csv(paste0(path, tsn), T)

  if (length(tsn) == 9){
    type_SC <- "TN"
  }else{
    type_SC <- "TP"
  }

  topodat <- read.csv(paste0(path, topo), T)
  lakprop <- read.csv(paste0(path, lr), T)
  length <- read.csv(paste0(path, length), T)

  if (type_SC == "TN"){
    lakeR_par <- 7.3
  }else{
    lakeR_par <- 26
  }

  lakprop$LakeFrRet <- 1 - 1 / (1 + lakeR_par / lakprop$AvgDepth_m *
                                  lakprop$ResTime)
  lakprop$LakeFrRet[is.na(lakprop$LakeFrRet)] <- 0
  lakprop$LakeFrRet[lakprop$LakeFrRet > 0.1] <- 0.1

  lakprop$AvgDepth_m <- NULL
  lakprop$ResTime <- NULL

  length$NrmLengthKm <- length$LengthKm / max(length$LengthKm)
  length$LengthKm <- NULL

  names(topodat)[2] <- c("To_catch")

  topodat <- shreve(topodat)

  scen1 <- merge(topodat, lakprop, by = "HydroID")
  scen1 <- merge(scen1, length, by = "HydroID")

  lr <- read.csv(paste0(path, obs), T)
  ff <- read.csv(paste0(path, ff), T)
  pre <- read.csv(paste0(path, rain), T)

  minRain <- 50
  pre$InvNrmRain <- (1 / apply(cbind(minRain, pre$Rain), 1, max)) /
    (1 / minRain)

  scen2 <- merge(tsn, lr, by = c("YearValue", "HydroID"), all.x= TRUE)
  scen2 <- merge(scen2, ff, by = c("YearValue", "HydroID"),all.x= TRUE)
  scen2 <- merge(scen2, pre[, c(1,2,4)], by = c("YearValue", "HydroID"),
                 all.x= TRUE)

  scen2 <- merge(scen2, scen1[, c(1:2)], by = "HydroID")

  scen2$BasinID <- c(1234)

  if(length(scen2) == 14){
    scen2 <- scen2[, c(14,2,1,13,3,4,5,6,7,8,9,10,11,12)]
  }else{
    scen2 <- scen2[, c(12,2,1,11,3,4,5,6,7,8,9,10)]
  }

  names(scen2)[4] <- c("NextDownID")

  l_scen <- list(scen1, scen2)

  return(l_scen)

}

#'
#' @title Read geometry
#'
#' @description Function to read the geometry file.
#'
#' @param file string. A string with the name and extension of the geometry file.
#'
#' @return One object, a sf file.
#'
#' @importFrom sf st_read
#'
#' @export
#'
read_geometry<- function(file){

  the_sf_shape <- st_read(file)

  return(the_sf_shape)

}


#'
#' @title Shreve
#'
#' @description Function to read the data and return the data frame for GREEN
#' execution.
#'
#' @param the_SC table. A table with topology data.
#'
#' @return One object, a data frame with the shreve.
#'
#' @export
#'
shreve <- function(the_SC){

  the_SC$Shreve<-NA

  AllOrig <- unique(the_SC$HydroID)
  AllDesting <- unique(the_SC$To_catch)
  WholeHydroID <- unique(c(AllOrig, AllDesting))
  WholeHydroID <- setdiff(WholeHydroID, -1)
  remai_hydroId <- WholeHydroID

  shr1 <- setdiff(AllOrig, AllDesting)

  the_SC$Shreve[the_SC$HydroID %in% shr1] <- 1
  TheSrhTable <- rbind(data.frame(HydroID = shr1, shreve = 1))

  remai_hydroId <- setdiff(remai_hydroId, shr1)

  while(length(remai_hydroId)>0){

    the_Scen2 <- the_SC[the_SC$To_catch %in% remai_hydroId[], ]
    DestToRemov <- the_Scen2$To_catch[is.na(the_Scen2$Shreve)]
    the_Scen3 <- the_Scen2[!(the_Scen2$To_catch %in% DestToRemov),]

    NexShr <- aggregate(the_Scen3$Shreve, by = list(the_Scen3$To_catch), max)
    names(NexShr) <-c("HydroID ", "Shreve")

    NexShr$Shreve <- NexShr$Shreve + 1

    the_SC$Shreve[the_SC$HydroID %in% NexShr$HydroID ] <- NexShr$Shreve

    remai_hydroId <- setdiff(remai_hydroId,NexShr$HydroID)

  }

  return(the_SC)

}

---
title: "Geospatial Regression Equation for European Nutrient losses (GREEN)"
output: rmarkdown::html_vignette
bibliography: papers_GREENeR.bib
vignette: >
  %\VignetteIndexEntry{Geospatial Regression Equation for European Nutrient losses (GREEN)}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

\renewcommand{\[}{\begin{equation}}
\renewcommand{\]}{\end{equation}}

<script type="text/x-mathjax-config">
MathJax.Hub.Config({
  TeX: { equationNumbers: { autoNumber: "AMS" } }
});
</script>

<style>
body {
text-align: justify}
</style>

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

---
**This Vignette illustrates how to use the GREENeR package for assessing annual time series of nutrient (total nitrogen TN and total phosphorus TP) loads in surface water from a basin or region of interest, including assessing land and river retention, and contribution shares by source.**

## Description of the tool

GREENeR [@RJ-2023-065] is an open-source R package for assessing annual time series of nutrient loads in a river network and at the basin outlets, and contributions of nutrient sources to these loads. The package provides tools and methods to apply the model Geospatial Regression Equation for European Nutrient losses (GREEN; [@grizzetti2005statistical; @grizzetti2008assessing; @grizzetti2012changes; @grizzetti2021eu]) to an area of interest. A brief description of the model, including sources and parameters, can be found at the end of this document.

The package includes functions to perform graphical summaries of model inputs, calibrate model parameters, run sensitivity analysis checks, and visualize model inputs and outputs through graphs and maps of total loads and contributions by source. The package works for both total nitrogen (TN) and total phosphorus (TP). It allows the analysis of different scenarios of nutrient input in the river basin or region of study. The package is parallel-capable to alleviate the computational burden in large basins.

## An example case: the Lay river Basin
GREENeR functionalities are illustrated for the [Lay river](https://en.wikipedia.org/wiki/Lay_(river)) basin, in France. Please note that the basin data are extracted from a pan-European dataset [@grizzetti2021eu] for the purpose of showing the package tools and the analysis steps for a generic region. Since the dataset may not reflect local sources correctly, implications of results in terms of nutrient fluxes may be incorrect and are not evaluated nor commented herein. 

The package includes two nutrient type scenarios (one for TN and one for TP), with disaggregated information on nutrient inputs for all catchments that compose the basin (spatial resolution of the  Catchment Characterisation Model; [@de2007rivers]). Both input (TN and TP) time series span from 1990 to 2018. The dataset includes also spatial information in vectorial format to enable mapping of tool results.

## Loading and description of the scenario

```{r message=FALSE}
# load the GREENeR package
library(GREENeR)
```

GreeneR needs information on the topology of the catchment and nutrient sources.
All information is contained in two data.frames. **Note that data.frame 
structures and column names must be respected when creating a new dataset, 
otherwise the functions will fail.**

The topology is contained in the data.frame "*catch_data_TN*" (*or 
catch_data_TP*; in the example dataset, the  Lay river Basin topology is 
presented).

```{r}
# load the topological sequence of catchments and complementary info
data(catch_data_TN)
head(catch_data_TN)
```

The fields are defined as follow:

+ HydroID (integer): unique identifier of the catchment 

+ To_catch (integer): unique identifier of the catchment to which the catchment 
goes. Note that for the outlet To_catch == -1

+ Shreve (integer positive): this indicates the Shreve order of the topological 
sequence in the stream network. GREEN resolves sequentially all loads for 
increasing numbers of Shreve order (headwaters, i.e. catchments that do not 
receive water, are Shreve == 1, catchments that receive waters only from Shreve 
1 are Shreve == 2, etc till the outlet) 

+ LakeFrRet (fraction, 0-1): lake retention fraction (Lret in eqs 1, 6, 7). 
Note, this is different for TN or TP module

+ NrmLengthKm (double): normalized length of catchment reach (eq 5) 

The nutrient sources in each catchment per year are contained in the data.frame 
"*annual_data_TN*" for the TN scenario ("*annual_data_TP*" for the TP scenario).

```{r}
# load the sources of TN for each year and catchment
data(annual_data_TN)
head(annual_data_TN)

# load the sources of TP for each year and catchment
data(annual_data_TP)
head(annual_data_TP)
```

In the annual data frames some fields are in common:

+ *BasinID* (integer): unique identifier of the Basin. This may be useful when the
region of interest encompasses more than one basin; all catchments belonging to a same Basin share the same BasinID. 
+ *YearValue* (integer): the year for which data are defined
+ *HydroID* (integer positive): unique catchment identifier (matches the ID in 
catch_data)
+ *NextDownID* (integer positive): corresponds to To_catch 
+ *ForestFraction*: Non-agricultural land cover in the catchment (fraction, FF in 
eqs 6 and 7)
+ *InvNrmRain*: inverse of normalized rainfall (eq 4)
+ *YearlyNMass*: observed annual total nitrogen load (TN ton/yr) from monitoring 
station data where available (NA when no data is present for that catchment and 
year). These data allow calibration of model parameter; thus sufficient observed 
loads are essential for the calibration process.

The other fields specify nutrient sources, and differs in the two nutrient 
scenarios. In the case of TN, fields are:  

+ Atm: Annual nitrogen deposition from atmosphere (ton/yr)
+ Min: Annual amount of nitrogen from mineral fertilisers (ton/yr)
+ Man: Annual amount of nitrogen in manure fertilisers (ton/yr)
+ Fix: Annual amount of nitrogen fixation by leguminous crops and fodder 
(ton/yr)
+ Soil: Annual amount of nitrogen fixation by bacteria in soils (ton/yr)
+ Sd: Nitrogen input from scattered dwellings (ton/yr)
+ Ps: Nitrogen input from point sources (ton/yr)

In the case of TP fields are: 

+ Bg: annual amount of phosphorus background losses (ton/yr; eq 7)
+ Min: annual amount of phosphorus mineral fertilisers (ton/yr)
+ Man: annual amount of phosphorus in manure fertilisers (ton/yr)
+ Sd: phosphorus input from scattered dwellings (ton/yr)
+ Ps: phosphorus input from point sources (ton/yr)

For some functionalities of the GREENeR package, the geographical information of
the catchments is required in the form of an sf (simple feature) object (the sf 
package, Pebesma & Bivand, implements the Simple Features open standard)

```{r}
# load the geographical information of the basin (require for some functionalities)
data(sh_file)
head(sh_file)
```

## Exploring scenario inputs
The library includes functions to examine the nutrient sources in a Basin, and explore how these are applied, either globally, or distributed over time or in space.

A first summary is provided by the `input_plot()` function, which - when option "B" is given, shows the mean annual nutrient loads per source for the whole period of analysis.

```{r, message=FALSE, eval=FALSE}
# the barplot for the Lay TN and TP Scenarios
input_plot(annual_data_TN, sh_file, "Lay", "B")
input_plot(annual_data_TP, sh_file, "Lay", "B")
```

```{r, echo=FALSE, out.width='100%', fig.cap="Barplot for the Lay TN scenario."}
knitr::include_graphics('figures/barplot_LayTN.png')
```

```{r, echo=FALSE, out.width='100%', fig.cap="Barplot for the Lay TP scenario."}
knitr::include_graphics('figures/barplot_LayTP.png')
```

By selecting option "D", `input_plot()` returns density plots of nutrient loads per catchment. These are useful to see the distribution of nutrient source inputs in the Basin. The `coef_SD` parameter is a scale factor for the standard deviation of the data. This can be useful for defining the limits on a graph, for example. If cSD is 2, then `xlim` will be calculated as the mean plus two standard deviations, which could represent, for example, 95% of the data in a normal distribution.

```{r, message=FALSE, eval=FALSE}
# the density plots for the Lay TN and TP Scenarios
input_plot(annual_data_TN, "Lay", "D",coef_SD=1, sh_file)
input_plot(annual_data_TP, "Lay", "D",coef_SD=1, sh_file)
```

```{r, echo=FALSE, out.width='100%', fig.cap="Density plot for the Lay TN scenario."}
knitr::include_graphics('figures/densityplot_LayTN.png')
```

```{r, echo=FALSE, out.width='100%', fig.cap="Density plot for the Lay TP scenario."}
knitr::include_graphics('figures/densityplot_LayTP.png')
```

The function `input_Tserie()` allows to examine the temporal evolution of the inputs. There are two options, the first, displaying cumulative
total amounts by source ("gr1"). 

```{r, message=FALSE, eval=FALSE}
# the time serie plot type 1 (areas)
input_Tserie(catch_data_TN, annual_data_TN, "Lay", "gr1")
input_Tserie(catch_data_TP, annual_data_TP, "Lay", "gr1")
```

```{r, echo=FALSE, out.width="45%", out.height="25%",fig.cap="Time series plot type 1 (areas) for TN and TP.",fig.show='hold',fig.align='center'}
knitr::include_graphics(c("figures/timeserieTN_type1.png","figures/timeserieTP_type1.png"))
``` 

The second, total amounts and by source total amount("gr2")

```{r, message=FALSE, eval=FALSE}
# the time serie plot type 2 (lines)
input_Tserie(catch_data_TN, annual_data_TN,  "Lay", "gr2")
input_Tserie(catch_data_TP, annual_data_TP,  "Lay", "gr2")
```

```{r, echo=FALSE, out.width="45%", out.height="25%",fig.cap="Time series plot type 2 (lines) for TN and TP.",fig.show='hold',fig.align='center'}
knitr::include_graphics(c("figures/timeserieTN_type2.png","figures/timeserieTP_type2.png"))
```

The function `input_Tserie_area()` allows to examine the temporal
evolution of the inputs in relation to the area. There are also two
options, the first, displaying cumulative total amounts by source per
unit area ("gr1")

```{r, echo=FALSE, eval=FALSE}
# the time serie plot type 3 (areas by year and km2)
input_Tserie_area(catch_data_TN, annual_data_TN, sh_file, "Lay", "gr1")
input_Tserie_area(catch_data_TP, annual_data_TP, sh_file, "Lay", "gr1")
```

```{r, echo=FALSE, out.width="45%", out.height="25%",fig.cap="Time series plot type 3 (areas by year and km2) for TN and TP.",fig.show='hold',fig.align='center'}
knitr::include_graphics(c("figures/timeserieTN_type3.png","figures/timeserieTP_type3.png"))
```

The latter graph is particularly useful when the range of catchment
areas is large to compare nutrient application intensity between
catchments. In the Lay river example, these graphs show, among other
things, how the amount of P from mineral fertilization (Min) has
decreased over time, whereas the amount of P from manure has increased.

There is a second alternative for the function `input_Tseries_area()`,
"gr2". This option compares the levels of nutrient inputs in three zones
of the basin (upper, middle and lower part). The upper zone includes the
catchments up to the median (50%) shreve value, the middle zone includes
the catchments from 50% to 75% of the shreve values, and the lower part
includes the catchments from 75% to 100%. At the top of each graph, the
corresponding area of each part is indicated. In the example below, it
can be seen that differences in N inputs between the upper and lower
part of the basin are noticeable over the whole period.

```{r, message=FALSE, eval=FALSE}
# the time serie plot type 4 (by km2 and Shreve)
input_Tserie_area(catch_data_TN, annual_data_TN, sh_file, "Lay", "gr2")
input_Tserie_area(catch_data_TP, annual_data_TP, sh_file, "Lay", "gr2")
```

```{r, echo=FALSE, out.width="45%", out.height="25%",fig.cap="Time series plot type 4, thatcompares the levels of nutrient inputs in three zones of the basin.",fig.show='hold',fig.align='center'}
knitr::include_graphics(c("figures/timeserieTN_type4.png","figures/timeserieTP_type4.png"))
```

The function `input_maps()` shows nutrient inputs distribution in the Basin. The function generates a map for each nutrient source, plus a map (in yellow-green scale) of the total diffuse nutrient sources, which can be contrasted to the point sources map. The first option ("gr1") shows total inputs (kt/year).



```{r, message=FALSE, eval=FALSE}
# the Input Load Map by source type 1 (kt/year)
input_maps(catch_data_TN, annual_data_TN, sh_file, "gr1", legend_position = 3)
```
```{r, echo=FALSE, out.width='100%', fig.cap="Input map by source type 1 for TN scenario"}
#![Input maps.](figures/input_maps.png){width=75%} #300px for HTML
knitr::include_graphics('figures/input_mapsTN_type1.png')
```

The legend_position parameter allows you to modify the position where
the legend will be located in the figure. For the TP scenario:

```{r, message=FALSE, eval=FALSE}
# the Input Load Map by source type 1 (kt/year)
input_maps(catch_data_TP, annual_data_TP, sh_file, "gr1", legend_position = 3)
```
```{r, echo=FALSE, out.width='100%', fig.cap="Input map by source type 1 for TP scenario"}
#![Input maps.](figures/input_maps.png){width=75%} #300px for HTML
knitr::include_graphics('figures/input_mapsTP_type1.png')
```

Option "gr2" shows inputs per area (in kt/y/km2):

```{r, message=FALSE, eval=FALSE}
# the Input Load Map by source type 2 (kt/y/km2)
input_maps(catch_data_TN, annual_data_TN, sh_file, "gr2", legend_position = 3)
```
```{r, echo=FALSE, out.width='100%', fig.cap="Input map by source type 2 for TN scenario"}
knitr::include_graphics('figures/input_mapsTN_type2.png')
```

For the TP scenario:

```{r, message=FALSE, eval=FALSE}
# the Input Load Map by source type 2 (kt/y/km2)
input_maps(catch_data_TP, annual_data_TP, sh_file, "gr2", legend_position = 3)
```
```{r, echo=FALSE, out.width='100%', fig.cap="Input map by source type 2 for TP scenario"}
knitr::include_graphics('figures/input_mapsTP_type2.png')
```

## Lake Retention

In the Lake Retention section, we note that the scenarios for TN or TP
are also linked with information about lake retention. This data is
often not dependent on the model used to generate the scenario and
includes actual field data. Therefore, viewing the distribution of lake
retention data can be insightful because it directly impacts the
outcomes of the entire GREEN model. The function that carries out this
analysis is `LakeRetent_plot()`.

```{r, message=FALSE, eval=FALSE}
LakeRetent_plot(catch_data_TN)
```

```{r, echo=FALSE, out.width='100%', fig.cap="Lake retention values distribution for TN scenario"}
knitr::include_graphics('figures/LakeRetent_plot.png')
```

## Observed Load

Under the Observed Load section, we discuss the importance of
information on the annual total nitrogen load (TN ton/yr) or total
phosphorous (TN ton/yr) gathered from monitoring station data, wherever
available. If no data is available for a particular catchment and year,
it is noted as NA. This information is crucial for the calibration
process of the model, as it allows the adjustment of model parameters.
Therefore, having a substantial amount of observed loads is essential
for the calibration process. The package includes a function called
`references_plot()`, which can be used to create a preliminary summary
of the data available at the monitoring stations. Before diving into
calibration it is thus useful to start by exploring observations.

```{r, message=FALSE, eval=FALSE}
TN_ref_values <- annual_data_TN[!is.na(annual_data_TN$YearlyMass),]
nrow(TN_ref_values)
table(TN_ref_values$HydroID,TN_ref_values$YearValue)
  
references_plot(annual_data_TN)
```

```{r, echo=FALSE, out.width='100%', fig.cap="Observed references distribution for TN scenario"}
knitr::include_graphics('figures/references.png')
```

In the example basin there are 22 observations (derived from publicly
available dataset of European Environment Agency,
[EEA](https://www.eea.europa.eu/data-and-maps/data/waterbase-water-quality-icm-1)
for 6 different stations from 2003 to 2009; note however that only one
station has references for all years. (For the TP case there are 58
observations, collected in eight catchments from 1997 to 2012, with one
complete time-series).

## Model calibration

To run the calibration process for a scenario (function
`calib_green()`), the following settings must be defined:

#. The expected range for each parameter. This range is defined by two vectors of three values, one for the lower limits and one for the upper limits of the three parameters. The values correspond to each of the parameters of the model in sequence: alpha_P (eq 4), alpha_L (eq 5), and sd_coeff (eqs 6 and 7).
#. The number of iterations to be performed during the calibration process. The higher the number of iterations, the more likely it is to achieve quality parameters, but the longer the computation time required. Although it depends on the catchment and the proposed intervals, it is recommended to run at least 200 iterations to have enough information to continue the calibration process.
#. The years to be included in the calibration process.

```{r, message=FALSE}
# Parameter for the calibration of the model
# the lower limits for all params (alpha_P, alpha_L, sd_coef)
low <- c(10, 0.000, 0.1) 
# the upper limits for all params (alpha_P, alpha_L, sd_coef)  
upp <- c(50, 0.08, 0.9)       

# number of iterations
n_iter <- 200 

# years in which the model should be executed
years <- c(2003:2009)
```

In the example below, a calibration of TN in the Lay basin is performed
with the parameters indicated above. The process is parallelised and
will use all the cores of the computer. The computation time depends on
the computer, the basin, and the number of iterations.

```{r, message=FALSE, eval=FALSE}
# execution of the calibration  
DF_calib_TN <- calib_green(catch_data_TN, annual_data_TN, n_iter, low, upp, years)
DF_calib <- data.frame(DF_calib)
```

The calibration function applies a Latin Hypercube sampling scheme to
the three parameters within the possible range (defined by lower and
upper limits) and evaluates model performance (predictions against
available observations) for the calibration period (specified in
'years') by calculating several "goodness-of-fit" metrics. The function
returns a data frame with parameters and goodness-of-fit scores that can
be further analyzed.

The function applies the following goodness-of-fit metrics
[@althoff2021goodness]:

-   NSE: Nash-Sutcliffe efficiency
-   rNSE: Relative Nash-Sutcliffe efficiency
-   mNSE: Modified Nash-Sutcliffe efficiency
-   cp: Coefficient of Persistence
-   VE: Volumetric Efficiency
-   KGE: Kling-Gupta efficiency
-   d: Index of Agreement
-   md: Modified index of agreement
-   rd: Relative Index of Agreement
-   r: Linear correlation coefficient
-   R2: Coefficient of determination
-   PBIAS: Percent bias
-   MAE: mean absolute error
-   RMSE: Root mean square error
-   ME: Mean error
-   MSE: Mean square error
-   NRMSE: Normalized Root Mean Square Error

## Exploring calibration results

Choosing the right goodness-of-fit metric to select the best set of parameters for a model largely depends on the overall study scope, the area of interest (e.g. high or low load, upper or lower catchment area), and the available observation dataset (size and quality). The library includes a series of functions to examine the calibration results and help select the most suitable set of parameters.

The `calib_boxplot()` function shows relationships between best parameter sets chosen according to one goodness-of-fit parameter (title of each boxplot) according to one goodness-of-fit metric (title of plot) in relation to others (x label). Only the six metrics that are used most frequently in hydrological calibration are included in this figure. In the lower panel, the figure shows as well the distribution of model parameters in the best parameter sets. The absolute best parameter set according to each goodness-of-fit metrics is marked as red dots in each boxplot. 

```{r, message=FALSE, eval=FALSE}
# Generating the box plots  
rateBS <-  5  #  percent of parameters selected from the whole calibration set
calib_boxplot(DF_calib_TN, rateBS)
```

```{r, echo=FALSE, out.width='100%'}
knitr::include_graphics('figures/calibration_boxplotTN.png')
```

The `calib_dot()` function shows the distribution of parameters in relation to each other for a chosen goodness-of-fit metric. With this function the figure can be generated for all the previously described goodness-of-fit metrics.

```{r, message=FALSE, eval=FALSE}
# Generating the dot plots  
Gof_mes <- "NSE"
calib_dot(DF_calib_TN, Gof_mes)
```

```{r, echo=FALSE, out.width='100%'}
knitr::include_graphics('figures/calibration_dotTN.png')
```

In the example figure, the best results for alpha_P to achieve best NSE are in the range 0.25-0.30 (cyan color), while there are no clear patterns for the other two parameters. Also, no significant interaction between parameters can be appreciated. We can conclude that the most sensitive parameter in this case is alpha_P.
 
The `scatter_plot()` function shows all parameter realizations in the calibration dataset against a selected goodness-of-fit metric to visualize the influence of each parameter on model results. The result depends on the goodness-of-fit metric the user wants to consider. With this function the figure can be generated for all metrics generated by `green_calib()` function.

```{r, message=FALSE, eval=FALSE}
# Generating the scatter plots  
Gof_mes <- "NSE"
scatter_plot(DF_calib_TN, Gof_mes)
```

```{r, echo=FALSE, out.width='100%'}
knitr::include_graphics('figures/calibration_scatterTN.png')
```

## Selecting the best parameter sets

The best parameter set according to a selected goodness-of-fit metrics can be extracted with the `select_params()` function:


```{r, message=FALSE, eval=FALSE}
Gof_mes <- "NSE" # according the NSE goodness of fit metric
NSE_bestParams <- select_params(DF_calib_TN, Gof_mes)
NSE_bestParams
Param_NSE2 <- as.numeric(NSE_bestParams[2:4])
```

The function can be applied with different goodness-of-fit metric to obtain different potential model realizations:
```{r, message=FALSE, eval=FALSE}
Gof_mes <- "rNSE" 
rNSE_bestParams <- select_params(DF_calib_TN,Gof_mes)
rNSE_bestParams    
Param_rNSE2 <- as.numeric(rNSE_bestParams[2:4])
```
(note that actual values returned by the function could change at each calibration run)

## Annual observed vs predicted values for a parameter set
Once a parameter set has been selected, it is useful to verify graphically the differences between the observations (ObsLoad) and the model predictions (PredictLoad). Included in the tool is the function `simobs_annual_plot()`, which allows a year-to-year comparison of observed and predicted values for a given parameter sets.

```{r, message=FALSE, eval=FALSE}
# annual scatter plot comparing observed vs modeled loads by year
label_plot <- "NSE best params for TN in the Lay" 
Themx <- max(annual_data_TN$YearlyMass, na.rm = TRUE)
simobs_annual_plot(catch_data_TN, annual_data_TN, Param_NSE2[1], Param_NSE2[2], Param_NSE2[3], years, label_plot, Themx)
```

```{r, echo=FALSE, out.width='100%'}
knitr::include_graphics('figures/annual_scatter_vs_modeled_loadsTN.png')
```

## Comparing Observed vs Predicted for two sets of parameters
The best parameter set depends on the goodness-of-fit metric. It is advisable to pre-select alternative parameter sets (according to different GoF) and compare model results, before making the final selection of the parameter set. The `compare_calib()` function shows simultaneously all observations against modelled values obtained from two parameter sets.

```{r, message=FALSE, eval=FALSE}
setPlabels <- c("NSE", "rNSE")    
label_plot <- "Comparing two sets of parameters for Lay TN" 
compare_calib(catch_data_TN, annual_data_TN, Param_NSE2[1], Param_NSE2[2], Param_NSE2[3], Param_rNSE2[1], Param_rNSE2[2], Param_rNSE2[3], years, label_plot, setPlabels)
```

```{r, echo=FALSE, out.width='100%'}
knitr::include_graphics('figures/comparing_twosets_paramsTN.png')
```

At the top left of the figure, the scores of six frequently used goodness-of-fit metrics for the two parameter sets (as identified in setPlabels) are reported.

## Estimation of nutrient fluxes and source contribution in the basin

Once the most appropriate parameter set has been selected, it is possible to run the model to estimate nutrient loads across the basin. 

The function `region_nut_balance()` runs the GREEN model with the selected parameter set, and returns the region nutrient mass balance of the nutrient fluxes for the whole simulation period. The results of this function can be visualized in a Sankey diagram with the function `N4_sankey()`.

```{r, message=FALSE, eval=FALSE}
# Computing the nutrient balance  
years <- c(2003:2009)
Nut_bal_TN <- region_nut_balance(catch_data_TN, annual_data_TN, Param_NSE2[1], Param_NSE2[2], Param_NSE2[3], years, atm_coeff = 0.38)
# Plot the sankey plot with the result of the balance
sank <- N4_sankey(Nut_bal_TN)
```

```{r, echo=FALSE, out.width='100%'}
knitr::include_graphics('figures/n4sankey.png')
```

## Share of nutrients
The function `green_shares()` runs the GREEN model with the selected
parameter set and returns for all catchments nutrient loads and
contributions by each nutrient source in the simulation period.

```{r, message=FALSE, eval=FALSE}
years <- c(2003:2009)
basin_sa <- green_shares(catch_data_TN, annual_data_TN, Param_NSE2[1], Param_NSE2[2], Param_NSE2[3], years)
```

The results of the model can be examined by several functions. The
function `nutrient_tseries()` allows to display the time series of the
total load (TN or TP) in the river outlet or the marian region. The
*"gr1"* option shows the temporal evolution of the total load at the
basin outlet in the simulation period.

```{r, message=FALSE, eval=FALSE}
# The title of the plot
plot_title <- "Time series Load Output for the Lay Basin"    
outFile <- tempfile(pattern = "OutletLoad", fileext = ".csv")
nutrient_tserie(basin_sa, plot_title, "gr1", outFile)
```

```{r, echo=FALSE, out.width='100%'}
knitr::include_graphics('figures/timeserie_loadoutputLay_type1.png')
```

The *"gr2"* option shows the temporal evolution of the total load at the
basin outlet in the simulation period with contributions by sources.

```{r, message=FALSE, eval=FALSE}
# Output Load Basin average time series (lines)
nutrient_tserie(basin_sa, plot_title, "gr2")
```

```{r, echo=FALSE, out.width='100%'}
knitr::include_graphics('figures/timeserie_loadoutputLay_type2.png')
```

The function `nutrient_tserie_darea()`, with the option *"gr1"*, shows
the temporal evolution of total load outputs for the upper, central and
lower catchment area.

```{r, message=FALSE, eval=FALSE}
# Total Load in the Basin Outlet by source apportionment time series (lines)
nutrient_tserie_darea(basin_sa, sh_file, "Lay Basin", "gr1")
```
```{r, echo=FALSE, out.width='100%'}
knitr::include_graphics('figures/timeserie_totalloadLay.png')
```

The function `nutrient_maps()` shows maps of nutrient loads in the
basin. The option *"gr1"* shows loads by source (in logarithmic scale to
improve the visualization as the differences in a basin may vary over
several orders of magnitude).

```{r, message=FALSE, eval=FALSE}
style <- "log10" 
nutrient_maps(basin_sa, sh_file, "gr1", style, legend_position = 3)
```

```{r, echo=FALSE, out.width='100%'}
knitr::include_graphics('figures/nutrient_maps_outputLoadsType1.png')
```

The "***style***" parameter allows the user to automatically create
breaks by specifying algorithms. Among others, the following styles can
be passed: "fisher", "log10_pretty", "log10", "order", "cont",
"quantile", "jenks".

The alternative *"gr2"* shows the total load at catchment outlets (in
logarithmic scale to improve their visualisation, as the differences
between sources and outlets are usually several orders of magnitude).

```{r, message=FALSE, eval=FALSE}
# Basin Output Total Load  Maps 
nutrient_maps(basin_sa, sh_file, style, "gr2", legend_position = 3)
```

```{r, echo=FALSE, out.width='100%'}
knitr::include_graphics('figures/nutrient_maps_outputLoadsType2.png')
```

The alternative "gr3" produces maps of total load by catchment area (in
kt/km2/y) in the basin.

```{r, message=FALSE, eval=FALSE}
# Basin Output Specific Load by km2 Maps 
nutrient_maps(basin_sa, sh_file, style, "gr3", legend_position = 3)
```

```{r, echo=FALSE, out.width='100%'}
knitr::include_graphics('figures/nutrient_maps_outputLoadsType3.png')
```

## GREEN model equations, sources and resources for input data
The Geospatial Regression Equation for European Nutrient losses (GREEN; @grizzetti2005statistical; @grizzetti2008assessing; @grizzetti2012changes; @grizzetti2021eu) is a conceptual model to assess nutrient loads (total nitrogen and total phosphorous) from a basin given diffuse and point emissions to the land and river network. The model comprises two major nutrient pathways: diffuse sources that undergo retention in the land phase (basin retention) before reaching the stream, and point sources that are directly discharged into surface waters. Once in the river network, all sources are reduced by the in-stream retention (river retention).

Diffuse sources include nutrient from mineral fertilisers, manure application, nitrogen crop and soil fixation, inputs from scattered dwellings (i.e. isolated houses and small agglomerations that are not connected to sewerage systems), and nitrogen atmospheric deposition (for nitrogen module) or background losses (for phosphorus module). Point sources consist of urban and industrial wastewater discharges. Basin retention is modelled as a decay function proportional to the inverse of the total annual precipitation in the catchment; river retention is estimated as a decay function proportional to the river length, considered as a proxy for water residence time. In addition, lake retention is simulated as a function of lakes residence time and average depth.

The basin is divided into spatial subunits (called catchments), which have a given area, a river reach, an in-let node, and an outlet node. The catchments are topologically connected from basin heads to the outlet in cascading sequence. The sequence of nutrient load accumulation through the stream network is defined by @shreve1966statistical order. Nutrient input from the different sources, basin and river retention are simulated in each small catchment and routed through the river network. For each catchment i in the basin, the GREEN nutrient load $L_i$ is estimated by the general equation:

\begin{equation} \label{eq:green_nutrient_load}
  L_i = (1 - Lret_i) * (DS_i * (1 - Bret_i) + PS_i + U_i) * (1 - Rret_i)
\end{equation}

where:

+ $L_i$ = Nutrient load at the catchment outlet (out-node) (ton/yr) 

Sources of nutrients are:

i. $DS_i$ = Nutrient diffuse sources in the catchment (ton/yr)
ii. $PS_i$ = Nutrient point sources in the catchment (ton/yr)
iii. $U_i$ = Nutrient load from upstream catchments (ton/yr)

Sinks of nutrients are:

i. $Lret_i$ = Lake retention (fraction). Lret can be defined according to @kronvang2004nutrient:
\begin{equation}
  Lret_i = 1 - \frac{1}{[1 + (\frac{7.3}{z}) * RT]} \text{(for nitrogen)}
\end{equation}
\begin{equation}
  Lret_i = 1 - \frac{1}{[1 + (\frac{26}{z}) * RT]} \text{(for phosphorus)}
\end{equation}

  where:

  + $z$ average lake depth (m), 
  + $RT$ = hydraulic residence time (yr).
  
  At European scale, average lake depth and hydraulic residence time can be obtained from HydroLAKES database             (https://www.hydrosheds.org/pages/hydrolakes, @messager2016estimating).
  
ii. $Bret_i$ = Basin retention (fraction):
\begin{equation} \label{eq:basin_retention}
  Bret_i = 1 - \exp(-alpha_p * NrmInvRainy)
\end{equation}
where $NrmInvRain$ is the inverse of annual precipitation (in mm) of year y, normalized by the maximum in the dataset:
  $NrmInvRain = (1 / precipitationy ) / max(1 / precipitation))$
  
iii. $Rret_i$ = River retention (fraction):
\begin{equation}
  Rret_i = 1 - \exp(-alpha_l * NrmLengthKmi)
\end{equation}
  
where $NrmLengthKm$ is the length (in km) of the catchment reach, normalized by the maximum in the dataset: $NrmLengthKm$ = (length of catchment $i$ reach, in km) / $max$(Reach length in the basin).

Thus, the inverse of precipitation and the river length are scaled by maximum scaling [@frank1994data]. Note that in this way basin retention varies from year to year according to annual precipitation, whereas river retention for a given catchment is constant. The equation is applied sequentially from the most upstream nodes to the outlet. The model parameters are:

#. the basin retention coefficient ($alpha_p$), which together with annual precipitation in each catchment regulates nutrient retention of diffuse sources $\eqref{eq:basin_retention}$;
#. the river retention coefficient ($alpha_l$), which regulates the river retention per river km.
#. the fraction of domestic diffuse sources that reaches the stream network ($sd_{coef}$). Note that in Europe the amount ot nturients from scattered dwellings is a minor source of nutrients, therefore this parameter is not very sensitive, and a fixed value of $0.50$ is recommended. In other regions of the world however this may not be the case, hence the model in GREENeR package defines this fraction as a calibration parameter.

$\eqref{eq:green_nutrient_load}$ defines the general GREEN function. However, diffuse and point sources are defined differently for each nutrient module, i.e. nitrogen or phosphorus. More specifically in GREEN **nitrogen** model, for the total nitrogen load $L_i$ of catchment $i$ is estimated as:

\begin{equation}
  L_i = (1 - Lret_i) * ((MinN_i + ManN_i + FixN_i + SoilN_i + (1 - FF_i) * AtmN_i) * \\ 
  (1 - Bret_i) + 0.38 * FF_i * AtmN_i + sd_{coef} * SdN_i + PsN_i + U_i) * (1 - Rret_i)
\end{equation}

where:

+ $MinN_i$ = Annual amount of nitrogen from mineral fertilisers (ton/yr)
+ $ManN_i$ = Annual amount of nitrogen in manure fertilisers (ton/yr)
+ $FixN_i$ = Annual amount of nitrogen fixation by leguminous crops and fodder (ton/yr)
+ $SoilN_i$ = Annual amount of nitrogen fixation by bacteria in soils (ton/yr)
+ $AtmN_i$ = Annual nitrogen deposition from atmosphere (ton/yr)
+ $FF_i$ = Non-agricultural land cover in the catchment (fraction) 
+ $SdN_i$ = Nitrogen input from scattered dwellings (ton/yr)
+ $PsN_i$ = Nitrogen input from point sources (ton/yr)
+ $U_i$ = Nitrogen load from upstream catchments (ton/yr).

Note that nitrogen atmospheric deposition losses are split into two parts, i.e. inputs to agricultural land undergo the basin retention (which includes also the crop uptake), while in all other areas (FF fraction) they are reduced by a fixed rate, derived from the literature, before entering into the stream. Background losses for nitrogen are thus estimated as $0.38 * FF * AtmN$. For an atmospheric deposition of $10$ kgN/ha this corresponds to a background of $3.8$ kgN/ha (in line with the values reported by @helsinki2004fourth).

In GREEN **phosphorus** model, the total annual phosphorus load $L_i$ of catchment $i$ is estimated by the equation:
\begin{equation}
L_i = (1 - Lret_i) * ((MinP_i + ManP_i + (1 - FF_i) * BgP_i) * (1 - Bret_i) + FF_i * BgP_i + \\ 
sd_{coef} * SdP_i + PsP_i + U_i) * (1 - Rret_i) 
\end{equation}

+ $MinP_i$ = annual amount of phosphorus mineral fertilisers (ton/yr)
+ $ManP_i$ = annual amount of phosphorus in manure fertilisers (ton/yr)
+ $FF_i$ = Non-agricultural land cover in the catchment (fraction)
+ $BgP_i$ = annual amount of phosphorus background losses (ton/yr)
+ $SdP_i$ = phosphorus input from scattered dwellings (ton/yr)
+ $PsP_i$ = phosphorus input from point sources (ton/yr)
+ $U_i$ = Phosphorus load from upstream catchments (ton/yr)

Note that phosphorous background losses are splited into two parts, with the inputs to agricultural land undergoing basin retention, while in all other areas (FF fraction) they are considered entering into the stream. Background losses for phosphorus are estimated at $0.15$ kgP/ha (in line with the values reported by @helsinki2004fourth).

Assembling input data for running the GREEN model is time consuming. Data sources for application at European scale are described in @grizzetti2021eu. Domestic sources of nitrogen and phosphorus can also be found in @vigiak2020domestic.


# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{references_plot}
\alias{references_plot}
\title{Reference summary plot}
\usage{
references_plot(annual_data)
}
\arguments{
\item{annual_data}{data frame. Sources of nutrient for each year and
catchments.}
}
\value{
A barplot, a histogram-density and a boxplot
}
\description{
Summary of the reference values in the stations
}
\examples{
\donttest{
# the data of the TN scenario
data(annual_data_TN)
references_plot(annual_data_TN)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annual_data_TP.R
\docType{data}
\name{annual_data_TP}
\alias{annual_data_TP}
\title{Annual data TP}
\format{
A data frame with 12 variables:
\describe{
  \item{\code{BasinID}}{integer. The basin unique identifier.}
  \item{\code{YearValue}}{integer. The year for which data are defined.}
  \item{\code{HydroID}}{integer positive. Unique catchment identifier.}
  \item{\code{NextDownID}}{integer. Unique identifier of the catchment to
  which the catchment goes.}
  \item{\code{Bg}}{double. Annual amount of phosphorus background losses
  (ton/yr).}
  \item{\code{Min}}{double. Annual amount of phosphorus mineral fertilisers
  (ton/yr).}
  \item{\code{Man}}{double. Annual amount of phosphorus in manure fertilisers
  (ton/yr).}
  \item{\code{Sd}}{double. Phosphorus input from scattered dwellings
  (ton/yr).}
  \item{\code{Ps}}{double. Phosphorus input from point sources (ton/yr).}
  \item{\code{YearlyMass}}{double. Observed annual total phosphorus load
  (TP ton/yr) from monitoring station data.}
  \item{\code{ForestFraction}}{double. Non-agricultural land cover in the
  catchment (fraction).}
  \item{\code{InvNrmRain}}{double. Inverse of normalized rainfall.}
}
}
\usage{
annual_data_TP
}
\description{
Defines the sources of nutrient (phosphorus) for each year and catchments.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen.R
\name{data_preparation}
\alias{data_preparation}
\title{data_preparation}
\usage{
data_preparation(catch_data, annual_data)
}
\arguments{
\item{catch_data}{data frame. Definition of the topological sequence of
catchments.}

\item{annual_data}{data frame. Sources of nutrient for each year and
catchments.}
}
\value{
One list with two data table
}
\description{
Prepare data to launch green
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{input_Tserie_area}
\alias{input_Tserie_area}
\title{Time series of annual load inputs by source and km2}
\usage{
input_Tserie_area(catch_data, annual_data, sh_file, basin_name, plot.type)
}
\arguments{
\item{catch_data}{data frame. Definition of the topological sequence of
catchments.}

\item{annual_data}{data frame. Sources of nutrient for each year and
catchments.}

\item{sh_file}{sf object. The spatial information.}

\item{basin_name}{character. The title of the plot}

\item{plot.type}{character. Alternative of the plot:  “gr1”:  stacked area by km2; “gr2”  lines & area by km2 and Shreve.}
}
\value{
A time-series plot
}
\description{
Creates a time series plot showing basin inputs by
source
}
\examples{
\donttest{
# the data of the TN scenario
data(catch_data_TN)
data(annual_data_TN)
data(sh_file)
# The title of the plot
plotTitle <- "Time series for the Lay Basin"
# the time serie plot 1 (by km2)
input_Tserie(catch_data_TN, annual_data_TN, sh_file, plotTitle, "gr1")
# the time serie plot 2 (by km2 and Shreve)
input_Tserie(catch_data_TN, annual_data_TN, sh_file, plotTitle, "gr2")
}
# catch_data <- The_Scen[[1]]
# annual_data <- The_Scen[[2]]
# sh_file <- The_Sf_shape
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{load_map}
\alias{load_map}
\title{Load DA map}
\usage{
load_map(hydroSf, long_basin, title, style_map, legend_position = 1)
}
\arguments{
\item{hydroSf}{data frame.}

\item{long_basin}{numeric.}

\item{title}{string. The title of the map}

\item{style_map}{character. The style of the map}

\item{legend_position}{numeric. It indicates de position of the legend of the
plot. (Default: 1)}
}
\value{
A plot with map of the basin
}
\description{
This function creates a tmap
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{create_map}
\alias{create_map}
\title{Create map}
\usage{
create_map(
  hydroSf,
  var_name,
  style_map,
  scale_barRefs,
  scale_barTexts,
  title,
  palette,
  legend_position = 1
)
}
\arguments{
\item{hydroSf}{data frame.}

\item{var_name}{character. The name of a data variable that it is contains
in data frame}

\item{style_map, scale_barRefs, scale_barTexts}{numeric.}

\item{title}{character. The title of the plot}

\item{palette}{character. The colors of the map}

\item{legend_position}{numeric. It indicates de position of the legend of the
plot. (Default: 1)}
}
\value{
A plot with map of the basin
}
\description{
This function create a tm map.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{evolution_plot_area}
\alias{evolution_plot_area}
\title{Evolution plot area}
\usage{
evolution_plot_area(
  annual_data,
  new_data,
  title_plot = NULL,
  xaxis.title = NULL,
  yaxis.title,
  x,
  y,
  colour = NULL,
  wrap = NULL
)
}
\arguments{
\item{annual_data, new_data}{data frame.}

\item{title_plot}{character. The plot title}

\item{xaxis.title}{character. The title of x axis}

\item{yaxis.title}{character. The title of y axis}

\item{x}{character. The name of variable in data to represent in x axis}

\item{y}{character. The name of variable in data to represent in y axis}

\item{colour}{character. The name of variable in data to represent colour}

\item{wrap}{character. The name of variable in data to represent wrap}
}
\value{
A plot
}
\description{
This function generates a stacked area plot to visualize the
temporal evolution of a variable. The function automatically adjusts the
X-axis scale based on the input data range and applies specific ColorBrewer
palettes according to the dimensions of the dataset.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{simobs_annual_plot}
\alias{simobs_annual_plot}
\title{Facet year plot}
\usage{
simobs_annual_plot(
  catch_data,
  annual_data,
  alpha_p,
  alpha_l,
  sd_coef,
  years,
  name_basin,
  maxvalue
)
}
\arguments{
\item{catch_data}{data frame. Definition of the topological sequence of
catchments.}

\item{annual_data}{data frame. Sources of nutrient for each year and
catchments.}

\item{alpha_p}{numeric. First model parameter, the basin retention
coefficient.}

\item{alpha_l}{numeric. Second model parameter, the river retention
coefficient.}

\item{sd_coef}{numeric. Third model parameter, fraction of domestic diffuse
sources that reaches the stream network.}

\item{years}{integer. Years to be used in the calibration. For sequences use
c(yearini:yearend).}

\item{name_basin}{character. The name of the basin}

\item{maxvalue}{numeric. The maximum value}
}
\value{
One object, a data frame
}
\description{
This function executes a parallelized hydrological simulation
and generates a comparison plot of simulated vs. observed loads. The function
leverages multiple CPU cores to run the launch_green model across several
years, aggregates the results, and produces a faceted scatter plot with a
1:1 reference line to evaluate model performance by year.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen.R
\name{check_years}
\alias{check_years}
\title{Check years}
\usage{
check_years(loc_years, years)
}
\arguments{
\item{loc_years}{numeric. A vector of integer with the years of interest}

\item{years}{numeric. A vector of integer with the years with information in
data frame}
}
\value{
A vector of integer
}
\description{
This function check if years are in data frame.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{create_lits_of_maps}
\alias{create_lits_of_maps}
\title{Preprocessing data for scenario summary}
\usage{
create_lits_of_maps(
  hydroSf,
  long_basin,
  unit,
  legend_position,
  param_list,
  style
)
}
\arguments{
\item{hydroSf}{data frame.}

\item{long_basin}{numeric. The longitude of the basin.}

\item{unit}{character. The unit of the data.}

\item{legend_position}{numeric. It indicates de position of the legend of the
plot. (Default: 1)}

\item{param_list}{list. A list with the maps to plot.}

\item{style}{character. The style of the plot.}
}
\value{
A list with maps
}
\description{
This function iterates through a list of parameters to generate multiple
thematic maps based on a shared spatial dataset. It automates the styling,
scaling, and labeling for each map to ensure consistency across a series.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen.R
\name{read_NSdata}
\alias{read_NSdata}
\title{Read NS data}
\usage{
read_NSdata(path, tsn, obs, ff, rain, topo, lr, length)
}
\arguments{
\item{path}{string. A string with the path of the CSV files.}

\item{tsn}{file. A CSV file with nine variables YearValue (integer),
HydroID (integer), Atm (float), Min (float), Man (float), Fix (float),
Soil (float), Sd (float) and Ps (float).}

\item{obs}{file. A CSV file with three variables YearValue (integer),
HydroID (integer) and YearlyMass (float).}

\item{ff}{file. A CSV file with three variables YearValue (integer),
HydroID (integer) and ForestFraction (float).}

\item{rain}{file. A CSV file with three variables YearValue (integer),
HydroID (integer) and Rain (float).}

\item{topo}{file. A CSV file with two variables HydroID (integer) and
Next_HydroID (integer).}

\item{lr}{file. A CSV file with three variables HydroID (integer),
AvgDepth (float) and ResTime (float).}

\item{length}{file. A CSV file with two variables HydroID (integer) and
LengthKm (float).}
}
\value{
One object, a list with two data frame. First position of the list
contains the catch data and the second one the annual data.
}
\description{
Function to read the data and return the data frame for GREEN
execution.
}
\examples{
\donttest{
path <- "https://raw.githubusercontent.com/calfarog/GREENeR_data/main/data/csv/"
ns_data <- read_NSdata(path, "TS_nutrients.csv", "Obs_monitoring.csv",
"ForestFr.csv", "Precipitation.csv", "Topology.csv", "LakeProperties.csv",
"Length.csv")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{input_maps}
\alias{input_maps}
\title{Map average load input by source}
\usage{
input_maps(
  catch_data,
  annual_data,
  sh_file,
  plot.type,
  style_map = "fisher",
  scale_barTextS = 0.7,
  legend_position = 1
)
}
\arguments{
\item{catch_data}{data frame. Definition of the topological sequence of
catchments.}

\item{annual_data}{data frame. Sources of nutrient for each year and
catchments.}

\item{sh_file}{sf object. The spatial information.}

\item{plot.type}{character. Alternatives of the map: input load (kt) by type
divided by year and catchment. “gr1”: by km2; “gr2”: by year/km2.}

\item{style_map}{character. Alternatives to create the intervals in the maps.
Chosen style: one of "fixed", "sd", "equal", "pretty", "quantile", "kmeans",
"hclust", "bclust", "fisher", "jenks".}

\item{scale_barTextS}{numeric. To modify the size of the text in the legend.}

\item{legend_position}{numeric. Legend position: 1 (default): "right",
"bottom"; 2: "left", "up"; 3: "right", "bottom"; 4: "right", "up".}
}
\value{
No return value, called for the side effect of drawing a plot
}
\description{
Map showing the mean load input by source
}
\examples{
\donttest{
# the data of the TN scenario
data(catch_data_TN)
data(annual_data_TN)
data(sh_file)
# the Input Load Map by source type 1 (lines)
input_maps(catch_data_TN, annual_data_TN, sh_file, plot.type = "gr1",
legend_position = 2)
# the Input Load Map by source type 2 (lines & area)
input_maps(catch_data_TN, annual_data_TN, sh_file, plot.type = "gr2",
legend_position = 2)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{LakeRetent_plot}
\alias{LakeRetent_plot}
\title{Lake retention values summary}
\usage{
LakeRetent_plot(catch_data_TN)
}
\arguments{
\item{catch_data_TN}{data frame. Sources of nutrient for each year and
catchments.}
}
\value{
barplot & histogram-density
}
\description{
Summary of the reference values in the stations
}
\examples{
\donttest{
# the data of the TN scenario
data(catch_data_TN)
LakeRetent_plot(catch_data_TN)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GREENeR-package.R
\docType{package}
\name{GREENeR-package}
\alias{GREENeR}
\alias{GREENeR-package}
\title{GREENeR: Geospatial Regression Equation for European Nutrient Losses (GREEN)}
\description{
The package provides tools and methods to apply the model Geospatial
Regression Equation for European Nutrient losses (GREEN;
Grizzetti et al. (2005); Grizzetti et al. (2012); Grizzetti et al. (2021)) to
an area of interest in R environment. The package comprises functions for
assessing annual nutrient (nitrogen and phosphorus) loads from a basin or
region of interest, land and river retention, and contribution shares by
sources. A brief description of the model, including sources and parameters,
can be found at the end of this document. Further, the package includes
functions for loading spatio-temporal data , calibrating basin parameters,
performing an advanced sensitivity analysis to evaluate the calibration
results, and visualizing model inputs and outputs through plots and maps.
The package is parallel-capable to alleviate the computational burden in
large basins.
}
\references{
Grizzetti, B., Bouraoui, F., De Marsily, G., & Bidoglio, G. (2005). A
statistical method for source apportionment of riverine nitrogen loads.
Journal of Hydrology, 304(1-4), 302-315.
\doi{doi:10.1016/j.jhydrol.2004.07.036}

Grizzetti, B., Bouraoui, F., De Marsily, G., (2008). Assessing nitrogen
pressures on European surface water. Global Biogeochem. Cycles 22..

Grizzetti, B., Bouraoui, F., & Aloe, A. (2012). Changes of nitrogen and
phosphorus loads to European seas. Global Change Biology, 18(2), 769-782.
\doi{doi:10.1111/j.1365-2486.2011.02576.x}

Grizzetti, B., Vigiak, O., Udias, A., Aloe, A., Zanni, M., Bouraoui, F.,
Pistocchi, A., Dorati, C., Friedland, R., De Roo, A., others & Bielza, M.
(2021). How EU policies could reduce nutrient pollution in European inland
and coastal waters. Global Environmental Change, 69, 102281.
\doi{doi:10.1016/j.gloenvcha.2021.102281}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/calfarog/GREENeR}
  \item \url{https://calfarog.github.io/GREENeR/}
  \item Report bugs at \url{https://github.com/calfarog/GREENeR/issues}
}

}
\author{
\strong{Maintainer}: C. Alfaro \email{c.alfarog@gmail.com}

Authors:
\itemize{
  \item A. Udias \email{angeludias@gmail.com}
  \item B. Grizzetti \email{bruna.grizzetti@jrc.it}
  \item O. Vigiak \email{Olga.VIGIAK@ec.europa.eu}
  \item J. Gomez \email{javier.gomez@urjc.es}
  \item A. Aloe \email{alberto.aloe@jrc.it}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen.R
\name{read_geometry}
\alias{read_geometry}
\title{Read geometry}
\usage{
read_geometry(file)
}
\arguments{
\item{file}{string. A string with the name and extension of the geometry file.}
}
\value{
One object, a sf file.
}
\description{
Function to read the geometry file.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{N4_sankey}
\alias{N4_sankey}
\title{Nutrient balance flow plot}
\usage{
N4_sankey(Nbalance_out)
}
\arguments{
\item{Nbalance_out}{data frame. Nutrient balance result from the Nutbalance()
function}
}
\value{
A Sankey diagram and a data frame with the some variable values
}
\description{
Nutrient balance flow in Sankey plot
}
\examples{
\donttest{
# the data of the TN scenario
data(catch_data_TN)
data(annual_data_TN)
# the parameter to assess the basin model
alpha_p <- 35.09
alpha_l <- 0.02
sd_coef <- 0.2
# years in which the model should be executed
loc_years <- 1990:2018
# Computing the nutrient balance
nut_bal <- region_nut_balance(catch_data_TN, annual_data_TN, alpha_p, alpha_l,
sd_coef, loc_years)
# Plot the sankey plot with the result of the balance
sank <- N4_sankey(nut_bal)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{calculate_shreve}
\alias{calculate_shreve}
\title{Calculate shreve}
\usage{
calculate_shreve(catch_data)
}
\arguments{
\item{catch_data}{data frame.}
}
\value{
A data frame
}
\description{
Calculate shreve
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{calib_dot}
\alias{calib_dot}
\title{Dot plot of goodness-of-fit metric vs parameter value}
\usage{
calib_dot(df_cb, param)
}
\arguments{
\item{df_cb}{data frame. A table with the result of the calibration process.}

\item{param}{character. Goodness of fit measures. See alternatives link "NSE"
"rNSE", "NSE", "mNSE", "MAE", "PBIAS", "cp", "R2".}
}
\value{
Multiple dot plots
}
\description{
Dot plot of goodness-of-fit metric vs parameters value
}
\examples{
\donttest{
# the data of the TN scenario
data(catch_data_TN)
data(annual_data_TN)
# the parameter for the calibration of the model
n_iter <- 2 # number of iterations
# the lower limits for all params (alpha_P, alpha_L, sd_coef)
low <- c(10, 0.000, 0.1)
# the upper limits for all params (alpha_P, alpha_L, sd_coef)
upp <- c(70, 0.3,  0.9)
# years in which the model should be executed
years <- 1990:2018
# execution of the calibration
df_calib <- calib_green(catch_data_TN, annual_data_TN, n_iter, low, upp,
years)
# Generating the dot plots
gof_mes <- "NSE"
calib_dot(df_calib, gof_mes)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{nutrient_tserie}
\alias{nutrient_tserie}
\title{Output load time series plot}
\usage{
nutrient_tserie(green_file, basin_name, plot.type, file_path = NULL)
}
\arguments{
\item{green_file}{data frame. Nutrient Load by source apportionment of
nutrient for each year and catchments.}

\item{basin_name}{character. The title of the plot.}

\item{plot.type}{character. Alternative of the plot: output load (t) by
source; gr1: Basin average by Shreve (t/y/km2); gr2: Outlet total (kt/y).}

\item{file_path}{character. The path to save the csv.}
}
\value{
No return value, called for the side effect of drawing a plot
}
\description{
Creates a time series plot showing basin model results
}
\examples{
\donttest{
# the data of the TN scenario
data(catch_data_TN)
data(annual_data_TN)
data(sh_file)
# the parameter to assess the basin model
alpha_p <- 35.09
alpha_l <- 0.02
sd_coef <- 0.2
# years in which the model should be executed
loc_years <- 1990:2018
# Computing the source apportionment
basin_sa <- green_shares(catch_data_TN, annual_data_TN, alpha_p, alpha_l,
sd_coef, loc_years)
# The title of the plot
plotTitle <- "Time series Load Output for the Lay Basin"
# Output Load Basin average time series (lines)
nutrient_tserie(basin_sa, basin_name = plotTitle, plot.type = "gr1")
# Total Load in the Basin Outlet time series (lines)
nutrient_tserie(basin_sa, basin_name = plotTitle, plot.type = "gr2")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{multiple_map}
\alias{multiple_map}
\title{Multiple Map}
\usage{
multiple_map(hydroSf, refN_P, long_basin, unit, legend_position = 1)
}
\arguments{
\item{hydroSf}{data frame.}

\item{refN_P}{numeric. The number of variables of the data frame}

\item{long_basin}{numeric.}

\item{unit}{character. The units of hydroSf}

\item{legend_position}{numeric. It indicates de position of the legend of the
plot. (Default: 1)}
}
\value{
A plot with map of the basin
}
\description{
This function creates an arrange of tmap
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{create_levels}
\alias{create_levels}
\title{Create levels}
\usage{
create_levels(df, no_sf)
}
\arguments{
\item{df, no_sf}{data frame.}
}
\value{
A list
}
\description{
This function create the levels for the data frame.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen.R
\name{region_nut_balance}
\alias{region_nut_balance}
\title{Nutrient balance based in the application of the Geospatial Regression
Equation returning the diffuse, land retention, point sources}
\usage{
region_nut_balance(
  catch_data,
  annual_data,
  alpha_p,
  alpha_l,
  sd_coef,
  loc_years,
  atm_coeff = 0.38
)
}
\arguments{
\item{catch_data}{data frame. Definition of the topological sequence of
catchments.}

\item{annual_data}{data frame. Sources of nutrient for each year and
catchments.}

\item{alpha_p}{numeric. First model parameter, the basin retention
coefficient.}

\item{alpha_l}{numeric. Second model parameter, the river retention
coefficient.}

\item{sd_coef}{numeric. Third model parameter, fraction of domestic diffuse
sources that reaches the stream network.}

\item{loc_years}{integer. Years in which the model should be executed.}

\item{atm_coeff}{numeric. A value for atmospheric attenuation coefficient.}
}
\value{
One object, a data frame with the basin nutrient balance
}
\description{
Computes the basin nutrient balance.
}
\examples{
\donttest{
# the data of the TN scenario
data(catch_data_TN)
data(annual_data_TN)
# the parameter to assess the basin model
alpha_p <- 35.09
alpha_l <- 0.02
sd_coef <- 0.2
# year in which the model should be executed
loc_years <- 1990:2018
# Computing the nutrient balance
basin_loads_b <- region_nut_balance(catch_data_TN, annual_data_TN, alpha_p, alpha_l,
sd_coef, loc_years)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen.R
\name{calib_green_help}
\alias{calib_green_help}
\title{Parallel calibration help function}
\usage{
calib_green_help(task, catch_data, annual_data, years, latin_range)
}
\arguments{
\item{task}{An integer with the number of simulation}

\item{catch_data}{data frame. It defines the topological sequence
of catchments.}

\item{annual_data}{data frame. It defines the sources of nutrient for each
year and catchments.}

\item{years}{integer. It defines the years of interest.}

\item{latin_range}{data frame. It defines the values for the params taking +
into account the limits.}
}
\value{
A matrix
}
\description{
This is a help function for calib_green function
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen.R
\name{check_colnames_annual}
\alias{check_colnames_annual}
\title{Check colnames}
\usage{
check_colnames_annual(df)
}
\arguments{
\item{df}{data frame.}
}
\value{
A data frame
}
\description{
This function checks that all column names for the catch_data
data frame are correct.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen.R
\name{launch_green}
\alias{launch_green}
\title{GREEN execution}
\usage{
launch_green(
  catch_data,
  annual_data,
  alpha_p,
  alpha_l,
  sd_coef,
  loc_years,
  atm_coeff = 0.38
)
}
\arguments{
\item{catch_data}{data frame. Definition of the topological sequence of
catchments.}

\item{annual_data}{data frame. Sources of nutrient for each year and
catchments.}

\item{alpha_p}{numeric. First model parameter, the basin retention
coefficient.}

\item{alpha_l}{numeric. Second model parameter, the river retention
coefficient.}

\item{sd_coef}{numeric. Third model parameter, fraction of domestic diffuse
sources that reaches the stream network.}

\item{loc_years}{integer. Years in which the model should be executed.}

\item{atm_coeff}{numeric. It defines the atmospheric attenuation coefficient.}
}
\value{
One object, a data frame
}
\description{
Applies the model GREEN (Grizzetti et al.
[@grizzetti2012changes, @grizzetti2021eu]) to a basin and returns annual
nutrient (nitrogen or phosphorus) load for all catchments in the basin
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimization_metrics.R
\name{gof}
\alias{gof}
\title{GOF}
\usage{
gof(sim, obs, na.rm = TRUE, digits = 2)
}
\arguments{
\item{sim}{numeric 'data.frame', 'matrix' or 'vector' with simulated values}

\item{obs}{numeric 'data.frame', 'matrix' or 'vector' with observed values}

\item{na.rm}{Should remove NA}

\item{digits}{Number of digits}
}
\value{
A data frame
}
\description{
GOF
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{input_plot}
\alias{input_plot}
\title{Plot input load by source}
\usage{
input_plot(annual_data, sh_file, basin_name, plot.type, coef_SD = 1)
}
\arguments{
\item{annual_data}{data frame. Sources of nutrient for each year and
catchments.}

\item{sh_file}{sf object. The spatial information.}

\item{basin_name}{character. The title of the plot.}

\item{plot.type}{character. Possible values: Bar plot ("B") or
Density plot (“D”).}

\item{coef_SD}{numeric. The standard deviation coefficient.}
}
\value{
No return value, called for the side effect of drawing a plot
}
\description{
A grouped barplot representing the average input load by source
for the whole basin or a three density plots showing the distribution of
nutrient sources (7 for nitrogen, 5 for phosphorous).
}
\examples{
# the data of the TN scenario
data(annual_data_TN)
data(sh_file)
# The name of the basin
basin_name <- "Lay"
# the barplot
input_plot(annual_data = annual_data_TN, basin_name = basin_name, plot.type = "B")
# the density plots
input_plot(annual_data_TN, sh_file, basin_name, "D")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen.R
\name{calib_green}
\alias{calib_green}
\title{Calibration of the GREEN model}
\usage{
calib_green(catch_data, annual_data, n_iter, low, upp, years)
}
\arguments{
\item{catch_data}{data frame. Definition of the topological sequence of
catchments.}

\item{annual_data}{data frame. Sources of nutrient for each year and
catchments.}

\item{n_iter}{numeric. Number of iterations for the calibration process.}

\item{low}{numeric. Lower bounds of the calibration parameters.}

\item{upp}{numeric. Upper bounds of the calibration parameters.}

\item{years}{integer. Years to be used in the calibration. For sequences use
c(yearini:yearend).}
}
\value{
One object, a data frame with the model calibration
}
\description{
Runs GREEN model calibration
}
\examples{
\donttest{
# the data of the TN scenario
data(catch_data_TN)
data(annual_data_TN)
# the parameter for the calibration of the model
n_iter <- 2 # number of iterations
# the lower limits for all params (alpha_P, alpha_L, sd_coef)
low <- c(10, 0.000, 0.1)
# the upper limits for all params (alpha_P, alpha_L, sd_coef)
upp <- c(70, 0.3,  0.9)
# years in which the model should be executed
years <- 1990:2018
# execution of the calibration
dF_calib <- calib_green(catch_data_TN, annual_data_TN, n_iter, low, upp,
years)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{nutrient_maps}
\alias{nutrient_maps}
\title{Map average load output by source}
\usage{
nutrient_maps(green_file, sh_file, plot.type, style, legend_position = 1)
}
\arguments{
\item{green_file}{data frame of GREEN model results from green_shares()
function. Nutrient Load by source apportionment of nutrient for each year
and catchments.}

\item{sh_file}{sf object. The spatial information of the basin.}

\item{plot.type}{character. Alternatives of the map: “gr1”: output load
(kt/y) by source; “gr2”: Total Load, log10 (kt/y); “gr3”: Total Load
by km2 (kt/year/km2).}

\item{style}{character. The style of the plot.}

\item{legend_position}{numeric. Legend position: 1 (default): "right",
"bottom"; 2: "left", "up"; 3: "right", "bottom"; 4: "right", "up".}
}
\value{
No return value, called for the side effect of drawing a plot
}
\description{
Creates maps showing basin output total or by source loads
}
\examples{
\donttest{
# the data of the TN scenario
data(catch_data_TN)
data(annual_data_TN)
data(sh_file)
# the parameter to assess the basin model
alpha_p <- 35.09
alpha_l <- 0.02
sd_coef <- 0.2
# years in which the model should be executed
loc_years <- 1990:2018
# Computing the source apportionment
basin_sa <- green_shares(catch_data_TN, annual_data_TN, alpha_p, alpha_l,
sd_coef, loc_years)
# Basin Output Load  Maps by source
Lpos <- 1
nutrient_maps(basin_sa, sh_file, plot.type = "gr1", style = "log10", legend_position = Lpos)
# Basin Output Specific Load  Maps
Lpos <- 1
nutrient_maps(basin_sa, sh_file, plot.type = "gr2", style = "log10", legend_position = Lpos)
# Basin Output Specific Load by km2 Maps
Lpos <- 1
nutrient_maps(basin_sa, sh_file, plot.type = "gr3", style = "fisher", legend_position = Lpos)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen.R
\name{append_empty_cols}
\alias{append_empty_cols}
\title{Append empty columns}
\usage{
append_empty_cols(df)
}
\arguments{
\item{df}{data frame. A data frame to add two new columns, Fix and Soil}
}
\value{
One object, a data frame.
}
\description{
This function add empty values to the data frame.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen.R
\name{shreve}
\alias{shreve}
\title{Shreve}
\usage{
shreve(the_SC)
}
\arguments{
\item{the_SC}{table. A table with topology data.}
}
\value{
One object, a data frame with the shreve.
}
\description{
Function to read the data and return the data frame for GREEN
execution.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{gr_density_plot}
\alias{gr_density_plot}
\title{Density plot}
\usage{
gr_density_plot(df_plot, plot_index, basin_name, cSD)
}
\arguments{
\item{df_plot}{data frame.}

\item{plot_index}{numeric. The indexes to plot}

\item{basin_name}{character. The name of the basin}

\item{cSD}{numeric. The standard deviation}
}
\value{
No return value, called for the side effect of drawing a plot
}
\description{
This function calculates and plots kernel density estimations
for normalized hydrological indices. The function normalizes selected
variables by a reference column, computes statistical parameters (mean and
standard deviation) to automatically scale the axes, and overlays density
lines for comparison across different data groups.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{calib_boxplot}
\alias{calib_boxplot}
\title{Boxplot of best parameters}
\usage{
calib_boxplot(df_cb, rate_bs)
}
\arguments{
\item{df_cb}{data frame. Table with the result of the calibration process.}

\item{rate_bs}{numeric. Rate (\%) of parameters selected from the whole set
produced in the calibration.}
}
\value{
Multiple boxplots
}
\description{
Returns boxplots of best model parameters ranked according to
different goodness-of-fit measures, and also boxplot with the distribution of
 the parameters values.
}
\examples{
\donttest{
# the data of the TN scenario
data(catch_data_TP)
data(annual_data_TP)
# the parameter for the calibration of the model
n_iter <- 2 # number of iterations
# the lower limits for all params (alpha_P, alpha_L, sd_coef)
low <- c(10, 0.000, 0.1)
# the upper limits for all params (alpha_P, alpha_L, sd_coef)
upp <- c(70, 0.3,  0.9)
# years in which the model should be executed
years <- 1990:2018
# execution of the calibration
df_calib <- calib_green(catch_data_TP, annual_data_TP, n_iter, low, upp,
years)
# Generating the box plots
rateBS <- 5 # rate of best set of parameter to include in the plots
calib_boxplot(df_calib, rateBS)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/catch_data_TN.R
\docType{data}
\name{catch_data_TN}
\alias{catch_data_TN}
\title{Catch data TN}
\format{
A data frame with 5 variables:
\describe{
  \item{\code{HydroID}}{integer positive. Unique catchment identifier.}
  \item{\code{To_catch}}{integer. Unique identifier of the catchment to which
  the catchment goes. Note that for the outlet To_catch== -1.}
  \item{\code{Shreve}}{integer. this indicates the Shreve order of the
  topological sequence in the stream network.}
  \item{\code{LakeFrRet}}{fraction, 0-1. Lake retention fraction.}
  \item{\code{NrmLengthKm}}{double. Normalized length of catchment reach.}
}
}
\usage{
catch_data_TN
}
\description{
Defines the topological sequence of catchments for nitrogen.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen_plots.R
\name{compare_calib}
\alias{compare_calib}
\title{Plot comparing observed vs modeled loads for two set of parameters}
\usage{
compare_calib(
  catch_data,
  annual_data,
  alpha_p1,
  alpha_l1,
  sd_coef1,
  alpha_p2,
  alpha_l2,
  sd_coef2,
  years,
  name_basin,
  setPlabels
)
}
\arguments{
\item{catch_data}{data frame. Definition of the topological sequence of
catchments.}

\item{annual_data}{data frame. Sources of nutrient for each year and
catchments.}

\item{alpha_p1}{numeric. The basin retention coefficient of the first set of
parameters.}

\item{alpha_l1}{numeric. The river retention coefficient of the first set of
parameters.}

\item{sd_coef1}{numeric. Fraction of domestic diffuse sources that reaches
the stream network of the first set of parameters.}

\item{alpha_p2}{numeric. The basin retention coefficient of the second set of
parameters.}

\item{alpha_l2}{numeric. The river retention coefficient of the second set of
parameters.}

\item{sd_coef2}{numeric. Fraction of domestic diffuse sources that reaches
the stream network of the second set of parameters.}

\item{years}{numeric. Years to be shown in the plot.}

\item{name_basin}{character. Name of the basin (title of the plot).}

\item{setPlabels}{character. Labels identifying each set of parameter.}
}
\value{
A scatter plot and a list with two data frames with model GREEN
applied to two model parameter sets
}
\description{
Returns a scatter plot comparing observed versus modeled loads
obtained with two model parameter sets
}
\examples{
\donttest{
# the data of the TN scenario
data(catch_data_TN)
data(annual_data_TN)

# the first set of parameters to assess the basin model
alpha_p <- 35.09
alpha_l <- 0.02
sd_coef <- 0.2

# the second set of parameters to assess the basin model
alpha_p2 <- 41.23
alpha_l2 <- 0.0015
sd_coef2 <- 0.6

# years in which the plot will we shown
years <- 1990:2018

nameBasin <- "Lay"

# generating the scatter plot comparing two set of parameters observed
# versus modeled loads by year
setPlabels <- c("bestNSE","bestR2")
compare_calib(catch_data_TN, annual_data_TN, alpha_p , alpha_l, sd_coef,
alpha_p2, alpha_l2, sd_coef2, years, nameBasin, setPlabels)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sh_file.R
\docType{data}
\name{sh_file}
\alias{sh_file}
\title{Sh file}
\format{
Simple feature collection with 6 features and 14 fields:
\describe{
  \item{\code{OBJECTID}}{integer Identifier of the spatial object (one for
  each catchment)}
  \item{\code{HydroID}}{integer Unique catchment identifier}
  \item{\code{NextDownID}}{integer Unique identifier of the catchment to
  which the catchment goes}
  \item{\code{BasinID}}{integer Identifier of the Basin to all  catchment
  belong}
  \item{\code{JunctionID}}{integer Unique identifier of the catchment to
  which the catchment goes. Note that for the outlet JunctionID == -1}
  \item{\code{AreaSqkm}}{double Area of each catchment}
  \item{\code{DrainAreaS}}{double Drain Area of each catchment}
  \item{\code{Shreve}}{integer Indicates Shreve order, i.e. in headwater = 1;
  at the outlet Shreve = max(Shreve)}
  \item{\code{geometry}}{sfc objects represent the geometry of a single
  feature and contains information about the feature’s coordinates,
  dimension, and type of geometry}
}
}
\usage{
sh_file
}
\description{
An sf (simple features) object  that includes attributes and
geometries in the form of a data frame. Data frame (or tibble) with rows of
features, columns of attributes, and a special geometry column that contains
the spatial aspects of the features
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annual_data_TN.R
\docType{data}
\name{annual_data_TN}
\alias{annual_data_TN}
\title{Annual data TN}
\format{
A data frame with 14 variables:
\describe{
  \item{\code{BasinID}}{integer. The basin unique identifier.}
  \item{\code{YearValue}}{integer. The year for which data are defined.}
  \item{\code{HydroID}}{integer positive. Unique catchment identifier.}
  \item{\code{NextDownID}}{integer. Unique identifier of the catchment to
  which the catchment goes.}
  \item{\code{Atm}}{double. Annual nitrogen deposition from atmosphere
  (ton/yr).}
  \item{\code{Min}}{double. Annual amount of nitrogen from mineral
  fertilisers (ton/yr).}
  \item{\code{Man}}{double. Annual amount of nitrogen in manure fertilisers
  (ton/yr).}
  \item{\code{Fix}}{double. Annual amount of nitrogen fixation by leguminous
  crops and fodder (ton/yr).}
  \item{\code{Soil}}{double. Annual amount of nitrogen fixation by bacteria
  in soils (ton/yr).}
  \item{\code{Sd}}{double. Nitrogen input from scattered dwellings (ton/yr).}
  \item{\code{Ps}}{double. Nitrogen input from point sources (ton/yr).}
  \item{\code{YearlyMass}}{double. Observed annual total nitrogen load
  (TN ton/yr) from monitoring station data.}
  \item{\code{ForestFraction}}{double. Non-agricultural land cover in the
  catchment (fraction).}
  \item{\code{InvNrmRain}}{double. Inverse of normalized rainfall.}
}
}
\usage{
annual_data_TN
}
\description{
Defines the sources of nutrient (nitrogen) for each year and catchments.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rgreen.R
\name{aggregate_loop}
\alias{aggregate_loop}
\title{Aggregate loop}
\usage{
aggregate_loop(listaShrt, cdat_greenLt, years, type)
}
\arguments{
\item{listaShrt, }{cdat_greenLt A data frame}

\item{years}{A vector of integer}

\item{type}{A character with values "L" when running launch_green and "N"
when running region_nut_balance}
}
\value{
A matrix
}
\description{
This function aggregate the variables of data frame.
}
\keyword{internal}
